@@ -12,7 +12,7 @@
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -328,7 +328,7 @@ int performListPrependOnEntry(chain *symTbl, char *ident, node *tree) {
   oldChecked = 0;
   newChecked = 0;
   addedChecked = 0;
-  if ((tree->nodeType == MEMREF) && (tree->isCorrectlyTyped)) addedChecked = 1;
+  if ((tree->nodeType == MEMREF) && (tree->cache->isCorrectlyTyped)) addedChecked = 1;
 
   okay = 0;
   curr = symTbl;
@@ -336,23 +336,23 @@ int performListPrependOnEntry(chain *symTbl, char *ident, node *tree) {
     if (strcmp(((entry *) (curr->value))->name,ident) == 0) {
       oldNode = (node *) (((entry *) curr->value)->value);
       if (oldNode->nodeType == MEMREF) {
-	oldNode->hashComputed = 0;
+	oldNode->cache->hashComputed = 0;
       }
-      if ((oldNode->nodeType == MEMREF) && (oldNode->isCorrectlyTyped)) oldChecked = 1;
+      if ((oldNode->nodeType == MEMREF) && (oldNode->cache->isCorrectlyTyped)) oldChecked = 1;
       newChecked = oldChecked && addedChecked;
       while (1) {
-	if (oldNode->nodeType != MEMREF) break; else { if (!newChecked) oldNode->isCorrectlyTyped = 0; }
+	if (oldNode->nodeType != MEMREF) break; else { if (!newChecked) oldNode->cache->isCorrectlyTyped = 0; }
 	if (oldNode->libFunDeriv > 1) break;
 	oldNode = getMemRefChild(oldNode);
       }
       switch (oldNode->nodeType) {
       case MEMREF:
-	if (!newChecked) oldNode->isCorrectlyTyped = 0;
+	if (!newChecked) oldNode->cache->isCorrectlyTyped = 0;
 	if (oldNode->libFunDeriv > 1) {
 	  if ((getMemRefChild(oldNode)->nodeType == LIST) ||
 	      (getMemRefChild(oldNode)->nodeType == FINALELLIPTICLIST)) {
 	    newArgs = addElement(copyChainWithoutReversal(getMemRefChild(oldNode)->arguments, copyThingOnVoid), tree);
-	    newNode = (node *) safeMalloc(sizeof(node));
+	    newNode = allocateNode();
 	    newNode->nodeType = getMemRefChild(oldNode)->nodeType;
 	    newNode->argArray = NULL;
 	    newNode->argArraySize = 0;
@@ -360,7 +360,7 @@ int performListPrependOnEntry(chain *symTbl, char *ident, node *tree) {
 	    newNode->arguments = newArgs;
 	    newNode = addMemRef(newNode);
 	    if (newChecked && (newNode->nodeType == MEMREF)) {
-	      newNode->isCorrectlyTyped = 1;
+	      newNode->cache->isCorrectlyTyped = 1;
 	    }
 	    ((entry *) curr->value)->value = newNode;
 	    freeThing(oldNode);
@@ -380,7 +380,7 @@ int performListPrependOnEntry(chain *symTbl, char *ident, node *tree) {
 	      */
 	      ((entry *) curr->value)->value = newNode;
 	      if (newChecked && (newNode->nodeType == MEMREF)) {
-		newNode->isCorrectlyTyped = 1;
+		newNode->cache->isCorrectlyTyped = 1;
 	      }
 	      okay = 1;
 	    } else {
@@ -453,9 +453,9 @@ int performListTailOnEntry(chain *symTbl, char *ident) {
     if (strcmp(((entry *) (curr->value))->name,ident) == 0) {
       oldNode = (node *) (((entry *) curr->value)->value);
       if (oldNode->nodeType == MEMREF) {
-	oldNode->hashComputed = 0;
+	oldNode->cache->hashComputed = 0;
       }
-      if ((oldNode->nodeType == MEMREF) && (oldNode->isCorrectlyTyped)) oldChecked = 1;
+      if ((oldNode->nodeType == MEMREF) && (oldNode->cache->isCorrectlyTyped)) oldChecked = 1;
       while (1) {
 	if (oldNode->nodeType != MEMREF) break;
 	if (oldNode->libFunDeriv > 1) break;
@@ -470,7 +470,7 @@ int performListTailOnEntry(chain *symTbl, char *ident) {
 	       (getMemRefChild(oldNode)->arguments->next != NULL) &&
 	       (getMemRefChild(oldNode)->arguments->next != NULL))) {
 	    newArgs = copyChainWithoutReversal(getMemRefChild(oldNode)->arguments->next, copyThingOnVoid);
-	    newNode = (node *) safeMalloc(sizeof(node));
+	    newNode = allocateNode();
 	    newNode->nodeType = getMemRefChild(oldNode)->nodeType;
 	    newNode->arguments = newArgs;
 	    newNode->argArray = NULL;
@@ -478,7 +478,7 @@ int performListTailOnEntry(chain *symTbl, char *ident) {
 	    newNode->argArrayAllocSize = 0;
 	    newNode = addMemRef(newNode);
 	    if (oldChecked && (newNode->nodeType == MEMREF)) {
-	      newNode->isCorrectlyTyped = 1;
+	      newNode->cache->isCorrectlyTyped = 1;
 	    }
 	    ((entry *) curr->value)->value = newNode;
 	    freeThing(oldNode);
@@ -503,7 +503,7 @@ int performListTailOnEntry(chain *symTbl, char *ident) {
 		 of the original expression here 
 	      */
 	      if (oldChecked && (newNode->nodeType == MEMREF)) {
-		newNode->isCorrectlyTyped = 1;
+		newNode->cache->isCorrectlyTyped = 1;
 	      }
 	      ((entry *) curr->value)->value = newNode;
 	      okay = 1;
