@@ -7,7 +7,7 @@
 
   Contributor Ch. Lauter
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
 
   This software is a computer program whose purpose is to provide an
   environment for safe floating-point code development. It is
