@@ -1,18 +1,27 @@
 /*
 
-  Copyright 2006-2014 by
+  Copyright 2006-2017 by
 
   Laboratoire de l'Informatique du Parallelisme,
-  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668
+  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
+
+  Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
+  Sorbonne Universités
+  UPMC Univ Paris 06
+  UMR 7606, LIP6
+  Boîte Courrier 169
+  4, place Jussieu
+  F-75252 Paris Cedex 05
+  France
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
   Sophia Antipolis, France.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -332,11 +341,11 @@ chain *makeConstantIntChainFromTo(int m, int n) {
 }
 
 
-int lengthChain(chain *c) {
-  int i;
+unsigned long long int lengthChain(chain *c) {
+  unsigned long long int i;
   chain *curr;
 
-  i = 0;
+  i = 0ull;
   curr = c;
   while (curr != NULL) {
     i++;
@@ -346,30 +355,40 @@ int lengthChain(chain *c) {
   return i;
 }
 
+int (*__sort_chain_global_cmp_func)(void *, void *) = NULL;
+
+int __sort_chain_cmp_func(const void *a, const void *b) {
+  return __sort_chain_global_cmp_func(*((void **) a), *((void **) b));
+}
 
 void sortChain(chain *c,  int (*f) (void *, void *)) {
-  chain *curr1, *curr2;
-  void *t;
+  chain *curr;
+  int len, i;
+  void **array;
+  int (*temp_cmp_func)(void *, void *);
 
   if (c==NULL) return;
   if (c->next == NULL) return;
 
-  /* else... */
+  for (curr=c,len=0;curr!=NULL;curr=curr->next,len++);
+  array = (void **) safeCalloc(len,sizeof(void *));
 
-  curr1 = c;
-  while (curr1 != NULL) {
-    curr2 = curr1->next;
-    while (curr2 != NULL) {
-      if (f(curr1->value, curr2->value)>=0) {
-	t = curr1->value;
-	curr1->value = curr2->value;
-	curr2->value = t;
-      }
-      curr2 = curr2->next;
-    }
-    curr1 = curr1->next;
+  for (curr=c,i=0;curr!=NULL;curr=curr->next,i++) {
+    array[i] = curr->value;
   }
-  return;
+
+  temp_cmp_func = __sort_chain_global_cmp_func;
+  __sort_chain_global_cmp_func = f;
+  
+  qsort(array, len, sizeof(void *), __sort_chain_cmp_func);
+
+  __sort_chain_global_cmp_func = temp_cmp_func;
+  
+  for (curr=c,i=0;curr!=NULL;curr=curr->next,i++) {
+    curr->value = array[i];
+  }
+    
+  safeFree(array);
 }
 
 int cmpIntPtr(void *a, void *b) {
