@@ -7,12 +7,12 @@
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
   Sophia Antipolis, France.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -94,7 +94,7 @@ chain *makeIntPtrChain(int n);
 chain *makeIntPtrChainFromTo(int m, int n);
 chain *makeConstantIntChain(int n);
 chain *makeConstantIntChainFromTo(int m, int n);
-int lengthChain(chain *c);
+unsigned long long int lengthChain(chain *c);
 void sortChain(chain *c,  int (*f) (void *, void *));
 int cmpIntPtr(void *a, void *b);
 int cmpMpfrPtr(void *a, void *b);
