@@ -1,7 +1,7 @@
 /*
   Copyright 2011-2012 by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
   Sophia Antipolis, France.
 
   Laboratoire de l'Informatique du Parallelisme,
@@ -13,7 +13,7 @@
   Contributors S. Chevillard, M. Joldes, Ch. Lauter
 
   joldes@laas.fr
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
