@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2006-2014 by
+  Copyright 2006-2021 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
@@ -8,14 +8,23 @@
   Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
   UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
+  Sorbonne Université
+  CNRS, Laboratoire d'Informatique de Paris 6, LIP6
+  F - 75005 Paris
+  France,
+
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage
+
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
   Sophia Antipolis, France.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -161,43 +170,30 @@ int round_to_format(mpfr_t rop, mpfr_t op, int prec, mp_rnd_t mode) {
 }
 
 
+static inline int __sollya_mpfr_round_to_double_mode_impl(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode) {
+  return mpfr_set_d(rop, sollya_mpfr_get_d(op, mode), GMP_RNDN);
+}
 
-int mpfr_round_to_double(mpfr_t rop, mpfr_t op) {
-  double d;
-  int res;
+int sollya_mpfr_round_to_double_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode) {
+  int tern;
 
-  d = sollya_mpfr_get_d(op,GMP_RNDN);
-  if (mpfr_set_d(rop,d,GMP_RNDN) != 0) {
+  tern = __sollya_mpfr_round_to_double_mode_impl(rop, op, mode);
+
+  if (tern != 0) {
     if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE,"Warning: double rounding occurred on invoking the double precision rounding operator.\n");
+      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE,"Warning: an undesired rounding occurred on a rounding to double.\n");
       printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
     }
   }
 
-  res = mpfr_cmp(rop,op);
-
-  return res;
+  return tern;
 }
 
-int mpfr_round_to_double_mode(mpfr_t rop, mpfr_t op, mp_rnd_t mode) {
-  double d;
-  int res;
-
-  d = sollya_mpfr_get_d(op,mode);
-  if (mpfr_set_d(rop,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE,"Warning: double rounding occurred on invoking the double precision rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
-    }
-  }
-
-  res = mpfr_cmp(rop,op);
-
-  return res;
+int sollya_mpfr_round_to_double(mpfr_t rop, mpfr_srcptr op) { 
+  return sollya_mpfr_round_to_double_mode(rop, op, GMP_RNDN);
 }
 
-
-int mpfr_round_to_prec(mpfr_t rop, mpfr_t op, mp_prec_t prec) {
+int sollya_mpfr_round_to_prec(mpfr_t rop, mpfr_t op, mp_prec_t prec) {
   mp_prec_t p;
   mpfr_t temp;
   int res;
@@ -218,273 +214,227 @@ int mpfr_round_to_prec(mpfr_t rop, mpfr_t op, mp_prec_t prec) {
   return res;
 }
 
+static inline int __sollya_mpfr_round_to_doubledouble_mode_impl(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode) {
+  mpfr_t h, d, l;
+  int tern;
 
-int mpfr_round_to_doubledouble(mpfr_t rop, mpfr_t op) {
-  double d;
-  mpfr_t accu, temp, rest;
-  mp_prec_t prec;
-  int res;
+  /* Check for NaN, Inf or zero 
+     
+     The result is just the input, the ternary is zero.
 
-  prec = mpfr_get_prec(op);
-  if (prec < 106) {
-    prec = 106;
+  */
+  if (mpfr_nan_p(op) ||
+      mpfr_inf_p(op) ||
+      mpfr_zero_p(op)) {
+    mpfr_set(rop, op, GMP_RNDN);
+    return 0;
   }
 
-  mpfr_init2(accu, prec);
-  mpfr_init2(temp, prec);
-  mpfr_init2(rest, prec);
+  /* The input is a non-zero number 
 
-  d = sollya_mpfr_get_d(op,GMP_RNDN);
-  if (mpfr_set_d(accu,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
+     We start by rounding the input to a double.
+
+  */
+  mpfr_init2(h, 64);
+  sollya_mpfr_round_to_double_mode(h, op, GMP_RNDN);
+
+  /* Check the high number for Inf or NaN.
+
+     In this case, h is the answer. The ternary is 
+     the one obtained when setting rop.
+
+  */
+  if (mpfr_inf_p(h) ||
+      mpfr_nan_p(h)) {
+    mpfr_clear(h);
+    return mpfr_set(rop, h, mode);
   }
-  if (mpfr_sub(rest,op,accu,GMP_RNDN) != 0) {
+
+  /* Here the high order double h is a finite number. 
+
+     Compute the difference 
+
+     d = op - h
+
+     That operation is Sterbenz-exact if d has at least 
+     the precision of op and h.
+
+  */
+  mpfr_init2(d, ((mpfr_get_prec(op) > mpfr_get_prec(h)) ? mpfr_get_prec(op) : mpfr_get_prec(h)));
+  mpfr_sub(d, op, h, GMP_RNDN); /* exact */
+
+  /* Compute the low order double, this time taking mode into
+     account 
+  */
+  mpfr_init2(l, 64);
+  sollya_mpfr_round_to_double_mode(l, d, mode);
+
+  /* Clear d */
+  mpfr_clear(d);
+
+  /* The answer is h + l. The ternary is the one obtained when adding
+     h and l. 
+  */
+  tern = mpfr_add(rop, h, l, mode);
+
+  /* Clear h and l */
+  mpfr_clear(h);
+  mpfr_clear(l);
+
+  /* Return the ternary */
+  return tern;
+}
+
+int sollya_mpfr_round_to_doubledouble_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode) {
+  int tern;
+
+  tern = __sollya_mpfr_round_to_doubledouble_mode_impl(rop, op, mode);
+
+  if (tern != 0) {
     if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  d = sollya_mpfr_get_d(rest,GMP_RNDN);
-  if (mpfr_set_d(temp,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_add(accu,accu,temp,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_set(rop,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
+      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: an undesired rounding occurred on a rounding to double-double.\n");
       printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
     }
   }
-
-  res = mpfr_cmp(rop,op);
-
-  mpfr_clear(accu);
-  mpfr_clear(temp);
-  mpfr_clear(rest);
-  return res;
+  
+  return tern;
 }
 
-int mpfr_round_to_doubledouble_mode(mpfr_t rop, mpfr_t op, mp_rnd_t mode) {
-  double d;
-  mpfr_t accu, temp, rest;
-  mp_prec_t prec;
-  int res;
+int sollya_mpfr_round_to_doubledouble(mpfr_t rop, mpfr_srcptr op) {
+  return sollya_mpfr_round_to_doubledouble_mode(rop, op, GMP_RNDN);
+}
 
-  prec = mpfr_get_prec(op);
-  if (prec < 106) {
-    prec = 106;
+static inline int __sollya_mpfr_round_to_tripledouble_mode_impl(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode) {
+  mpfr_t h, d, m, l, dd, t;
+  int tern;
+
+  /* Check for NaN, Inf or zero 
+     
+     The result is just the input, the ternary is zero.
+
+  */
+  if (mpfr_nan_p(op) ||
+      mpfr_inf_p(op) ||
+      mpfr_zero_p(op)) {
+    mpfr_set(rop, op, GMP_RNDN);
+    return 0;
   }
 
-  mpfr_init2(accu, prec);
-  mpfr_init2(temp, prec);
-  mpfr_init2(rest, prec);
+  /* The input is a non-zero number 
 
-  d = sollya_mpfr_get_d(op,GMP_RNDN);
-  if (mpfr_set_d(accu,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
+     We start by rounding the input to a double.
+
+  */
+  mpfr_init2(h, 64);
+  sollya_mpfr_round_to_double_mode(h, op, GMP_RNDN);
+
+  /* Check the high number for Inf or NaN.
+
+     In this case, h is the answer. The ternary is 
+     the one obtained when setting rop.
+
+  */
+  if (mpfr_inf_p(h) ||
+      mpfr_nan_p(h)) {
+    mpfr_clear(h);
+    return mpfr_set(rop, h, mode);
   }
-  if (mpfr_sub(rest,op,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
+
+  /* Here the high order double h is a finite number. 
+
+     Compute the difference 
+
+     d = op - h
+
+     That operation is Sterbenz-exact if d has at least 
+     the precision of op and h.
+
+  */
+  mpfr_init2(d, ((mpfr_get_prec(op) > mpfr_get_prec(h)) ? mpfr_get_prec(op) : mpfr_get_prec(h)));
+  mpfr_sub(d, op, h, GMP_RNDN); /* exact */
+
+  /* Compute the middle order double. */
+  mpfr_init2(m, 64);
+  sollya_mpfr_round_to_double_mode(m, d, GMP_RNDN);
+
+  /* Check the middle number for Inf or NaN.
+
+     In this case, h + m is the answer. The ternary is 
+     the one obtained when adding h and m.
+
+  */
+  if (mpfr_inf_p(m) ||
+      mpfr_nan_p(m)) {
+    tern = mpfr_add(rop, h, m, mode);
+    mpfr_clear(d);
+    mpfr_clear(m);
+    mpfr_clear(h);
+    return tern;
   }
-  d = sollya_mpfr_get_d(rest,mode);
-  if (mpfr_set_d(temp,d,GMP_RNDN) != 0) {
+
+  /* Here m is a finite number.
+
+     Exactly add h and m, yielding t. 
+
+     This addition is exact if t has at least twice the precision of op.
+
+  */
+  mpfr_init2(t, 2 * mpfr_get_prec(op));
+  mpfr_add(t, h, m, GMP_RNDN); /* exact */
+
+  /* Compute dd = op - t. 
+
+     This operation is Sterbenz-exact if dd has the same precision as
+     t.
+
+  */
+  mpfr_init2(dd, mpfr_get_prec(t));
+  mpfr_sub(dd, op, t, GMP_RNDN); /* exact */
+
+  /* Compute the low order double, this time taking mode into
+     account 
+  */
+  mpfr_init2(l, 64);
+  sollya_mpfr_round_to_double_mode(l, dd, mode);
+
+  /* Clear d and dd */
+  mpfr_clear(d);
+  mpfr_clear(dd);
+
+  /* The answer is t + l. The ternary is the one obtained when adding
+     t and l. 
+  */
+  tern = mpfr_add(rop, t, l, mode);
+
+  /* Clear h, m, l and t */
+  mpfr_clear(h);
+  mpfr_clear(m);
+  mpfr_clear(l);
+  mpfr_clear(t);
+
+  /* Return the ternary */
+  return tern;  
+}
+
+int sollya_mpfr_round_to_tripledouble_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode) {
+  int tern;
+
+  tern = __sollya_mpfr_round_to_tripledouble_mode_impl(rop, op, mode);
+
+  if (tern != 0) {
     if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_add(accu,accu,temp,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_set(rop,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_DOUBLE_DOUBLE,"Warning: double rounding occurred on invoking the double-double rounding operator.\n");
+      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: an undesired rounding occurred on a rounding to triple-double.\n");
       printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
     }
   }
-
-  res = mpfr_cmp(rop,op);
-
-  mpfr_clear(accu);
-  mpfr_clear(temp);
-  mpfr_clear(rest);
-  return res;
+  
+  return tern;
 }
 
-
-int mpfr_round_to_tripledouble(mpfr_t rop, mpfr_t op) {
-  double d;
-  mpfr_t accu, temp, rest;
-  mp_prec_t prec;
-  int res;
-
-  prec = mpfr_get_prec(op);
-  if (prec < 159) {
-    prec = 159;
-  }
-
-  mpfr_init2(accu, prec);
-  mpfr_init2(temp, prec);
-  mpfr_init2(rest, prec);
-
-  d = sollya_mpfr_get_d(op,GMP_RNDN);
-  if (mpfr_set_d(accu,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_sub(rest,op,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  d = sollya_mpfr_get_d(rest,GMP_RNDN);
-  if (mpfr_set_d(temp,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_add(accu,accu,temp,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_sub(rest,op,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  d = sollya_mpfr_get_d(rest,GMP_RNDN);
-  if (mpfr_set_d(temp,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_add(accu,accu,temp,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_set(rop,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
-    }
-  }
-
-  res = mpfr_cmp(rop,op);
-
-  mpfr_clear(accu);
-  mpfr_clear(temp);
-  mpfr_clear(rest);
-
-  return res;
+int sollya_mpfr_round_to_tripledouble(mpfr_t rop, mpfr_srcptr op) {
+  return sollya_mpfr_round_to_tripledouble_mode(rop, op, GMP_RNDN);
 }
 
-int mpfr_round_to_tripledouble_mode(mpfr_t rop, mpfr_t op, mp_rnd_t mode) {
-  double d;
-  mpfr_t accu, temp, rest;
-  mp_prec_t prec;
-  int res;
-
-  prec = mpfr_get_prec(op);
-  if (prec < 159) {
-    prec = 159;
-  }
-
-  mpfr_init2(accu, prec);
-  mpfr_init2(temp, prec);
-  mpfr_init2(rest, prec);
-
-  d = sollya_mpfr_get_d(op,GMP_RNDN);
-  if (mpfr_set_d(accu,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_sub(rest,op,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  d = sollya_mpfr_get_d(rest,GMP_RNDN);
-  if (mpfr_set_d(temp,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_add(accu,accu,temp,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_sub(rest,op,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  d = sollya_mpfr_get_d(rest,mode);
-  if (mpfr_set_d(temp,d,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on recasting to MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_add(accu,accu,temp,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"The rounding occurred on subtracting in MPFR. This should not occur.\n");
-    }
-  }
-  if (mpfr_set(rop,accu,GMP_RNDN) != 0) {
-    if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_TRIPLE_DOUBLE,"Warning: double rounding occurred on invoking the triple-double rounding operator.\n");
-      printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
-    }
-  }
-
-  res = mpfr_cmp(rop,op);
-
-  mpfr_clear(accu);
-  mpfr_clear(temp);
-  mpfr_clear(rest);
-
-  return res;
-}
-
-
 int printDoubleInHexa(mpfr_t x) {
   int res;
   double d;
@@ -541,8 +491,9 @@ int printSimpleInHexa(mpfr_t x) {
     return 0;
   }
 
-  mpfr_init2(tempRound,24);
-  res = mpfr_round_to_single(tempRound, x);
+  mpfr_init2(tempRound,64);
+  sollya_mpfr_round_to_single(tempRound, x);
+  res = mpfr_cmp(tempRound, x);
 
   d = sollya_mpfr_get_d(tempRound,GMP_RNDN);
   xfloat = d;
@@ -667,6 +618,10 @@ int readHexa(mpfr_t res, char *c) {
   return 0;
 }
 
+static inline int __funny_ternary_wrapper(int (*roundFunc)(mpfr_t, mpfr_srcptr), mpfr_t rop, mpfr_srcptr op) {
+  roundFunc(rop, op);
+  return mpfr_cmp(rop, op);
+}
 
 node *roundPolynomialCoefficients(node *poly, chain *formats, mp_prec_t prec) {
   int degree, listLength, i, deg, res, fillUp, k;
@@ -784,30 +739,32 @@ node *roundPolynomialCoefficients(node *poly, chain *formats, mp_prec_t prec) {
     }
     switch (formatsArray[i]) {
     case 7:
-      if (mpfr_round_to_quad(tempMpfr, fpcoefficients[i]) != 0) res = 1;
+      if (__funny_ternary_wrapper(sollya_mpfr_round_to_quad,tempMpfr, fpcoefficients[i]) != 0) res = 1;
       break;
     case 6:
-      if (mpfr_round_to_halfprecision(tempMpfr, fpcoefficients[i]) != 0) res = 1;
+      if (__funny_ternary_wrapper(sollya_mpfr_round_to_halfprecision,tempMpfr, fpcoefficients[i]) != 0) res = 1;
       break;
     case 5:
-      if (mpfr_round_to_single(tempMpfr, fpcoefficients[i]) != 0) res = 1;
+      if (__funny_ternary_wrapper(sollya_mpfr_round_to_single,tempMpfr, fpcoefficients[i]) != 0) res = 1;
       break;
     case 4:
-      if (mpfr_round_to_doubleextended(tempMpfr, fpcoefficients[i]) != 0) res = 1;
+      if (__funny_ternary_wrapper(sollya_mpfr_round_to_doubleextended,tempMpfr, fpcoefficients[i]) != 0) res = 1;
       break;
     case 3:
-      if (mpfr_round_to_tripledouble(tempMpfr, fpcoefficients[i]) != 0) res = 1;
+      if (__funny_ternary_wrapper(sollya_mpfr_round_to_tripledouble,tempMpfr, fpcoefficients[i]) != 0) res = 1;
       break;
     case 2:
-      if (mpfr_round_to_doubledouble(tempMpfr, fpcoefficients[i]) != 0) res = 1;
+      if (__funny_ternary_wrapper(sollya_mpfr_round_to_doubledouble,tempMpfr, fpcoefficients[i]) != 0) res = 1;
       break;
     case 1:
-      if (mpfr_round_to_double(tempMpfr, fpcoefficients[i]) != 0) res = 1;
+      if (__funny_ternary_wrapper(sollya_mpfr_round_to_double,tempMpfr, fpcoefficients[i]) != 0) res = 1;
       break;
     default:
       if (formatsArray[i] > 7) {
-	if (mpfr_round_to_prec(tempMpfr, fpcoefficients[i], (mp_prec_t) (formatsArray[i] - 6)) != 0)
+	sollya_mpfr_round_to_prec(tempMpfr, fpcoefficients[i], (mp_prec_t) (formatsArray[i] - 6));
+	if (mpfr_cmp(tempMpfr, fpcoefficients[i]) != 0) {
 	  res = 1;
+	}
       } else {
 	printMessage(1,SOLLYA_MSG_ERROR_UNKNOWN_EXPANSION_FORMAT,"Warning: unknown expansion format found. No rounding will be performed.\n");
 	mpfr_set(tempMpfr,fpcoefficients[i],GMP_RNDN);
@@ -933,6 +890,7 @@ int printPolynomialAsDoubleExpansion(node *poly, mp_prec_t prec) {
   node **coefficients;
   node *tempNode, *simplifiedTreeSafe, *simplifiedTree, *myTree;
   mpfr_t tempValue, tempValue2;
+  int printedSomething;
 
   roundingOccured = 0;
 
@@ -967,6 +925,7 @@ int printPolynomialAsDoubleExpansion(node *poly, mp_prec_t prec) {
   mpfr_set_d(tempValue2,1.0,GMP_RNDN);
 
   k = 0; l = 0;
+  printedSomething = 0;
   for (i=0;i<=degree;i++) {
     if (coefficients[i] != NULL) {
       if (k > 0) {
@@ -985,6 +944,7 @@ int printPolynomialAsDoubleExpansion(node *poly, mp_prec_t prec) {
       tempNode = simplifyTreeErrorfree(coefficients[i]);
       if (accessThruMemRef(tempNode)->nodeType == CONSTANT) {
 	roundingOccured |=  printDoubleExpansion(*(accessThruMemRef(tempNode)->value));
+	printedSomething = 1;
       } else {
 	if (!isConstant(tempNode)) {
 	  printMessage(1,SOLLYA_MSG_POLY_COEFF_IS_NOT_CONSTANT,"Error: a coefficient of a polynomial is not constant.\n");
@@ -996,6 +956,7 @@ int printPolynomialAsDoubleExpansion(node *poly, mp_prec_t prec) {
 	  evaluate(tempValue, tempNode, tempValue2, 256 * prec);
 	}
 	printDoubleExpansion(tempValue);
+	printedSomething = 1;
 	roundingOccured = 1;
       }
       free_memory(tempNode);
@@ -1012,6 +973,10 @@ int printPolynomialAsDoubleExpansion(node *poly, mp_prec_t prec) {
   for (i=0;i<l;i++)
     sollyaPrintf(")");
 
+  if (!printedSomething) {
+    sollyaPrintf("0.0");
+  }
+  
   safeFree(coefficients);
   mpfr_clear(tempValue);
   mpfr_clear(tempValue2);
@@ -1021,28 +986,28 @@ int printPolynomialAsDoubleExpansion(node *poly, mp_prec_t prec) {
 }
 
 
-void mpfr_round_to_format(mpfr_t rop, mpfr_t op, int format) {
+void sollya_mpfr_round_to_format(mpfr_t rop, mpfr_srcptr op, int format) {
   switch (format) {
   case 7:
-    mpfr_round_to_quad(rop, op);
+    sollya_mpfr_round_to_quad(rop, op);
     break;
   case 6:
-    mpfr_round_to_halfprecision(rop, op);
+    sollya_mpfr_round_to_halfprecision(rop, op);
     break;
   case 5:
-    mpfr_round_to_single(rop, op);
+    sollya_mpfr_round_to_single(rop, op);
     break;
   case 4:
-    mpfr_round_to_doubleextended(rop, op);
+    sollya_mpfr_round_to_doubleextended(rop, op);
     break;
   case 3:
-    mpfr_round_to_tripledouble(rop, op);
+    sollya_mpfr_round_to_tripledouble(rop, op);
     break;
   case 2:
-    mpfr_round_to_doubledouble(rop, op);
+    sollya_mpfr_round_to_doubledouble(rop, op);
     break;
   case 1:
-    mpfr_round_to_double(rop, op);
+    sollya_mpfr_round_to_double(rop, op);
     break;
   default:
     sollyaFprintf(stderr,"Error: mpfr_round_to_format: unknown format type.\n");
@@ -1050,31 +1015,34 @@ void mpfr_round_to_format(mpfr_t rop, mpfr_t op, int format) {
   }
 }
 
-int mpfr_round_to_doubleextended_mode(mpfr_t rop, mpfr_t op, mp_rnd_t mode);
+int sollya_mpfr_round_to_quad_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode);
+int sollya_mpfr_round_to_halfprecision_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode);
+int sollya_mpfr_round_to_single_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode);
+int sollya_mpfr_round_to_doubleextended_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode);
 
-int round_to_expansion_format(mpfr_t rop, mpfr_t op, int format, mp_rnd_t mode) {
+int round_to_expansion_format(mpfr_t rop, mpfr_srcptr op, int format, mp_rnd_t mode) {
   int res;
   switch (format) {
   case 7:
-    mpfr_round_to_quad_mode(rop, op, mode);
+    sollya_mpfr_round_to_quad_mode(rop, op, mode);
     break;
   case 6:
-    mpfr_round_to_halfprecision_mode(rop, op, mode);
+    sollya_mpfr_round_to_halfprecision_mode(rop, op, mode);
     break;
   case 5:
-    mpfr_round_to_single_mode(rop, op, mode);
+    sollya_mpfr_round_to_single_mode(rop, op, mode);
     break;
   case 4:
-    mpfr_round_to_doubleextended_mode(rop, op, mode);
+    sollya_mpfr_round_to_doubleextended_mode(rop, op, mode);
     break;
   case 3:
-    mpfr_round_to_tripledouble_mode(rop, op, mode);
+    sollya_mpfr_round_to_tripledouble_mode(rop, op, mode);
     break;
   case 2:
-    mpfr_round_to_doubledouble_mode(rop, op, mode);
+    sollya_mpfr_round_to_doubledouble_mode(rop, op, mode);
     break;
   case 1:
-    mpfr_round_to_double_mode(rop, op, mode);
+    sollya_mpfr_round_to_double_mode(rop, op, mode);
     break;
   default:
     sollyaFprintf(stderr,"Error: round_to_expansion_format: unknown format type.\n");
@@ -1097,7 +1065,7 @@ int round_to_format_or_expansion_format(mpfr_t rop, mpfr_t op, int expansion_for
   return round_to_format(rop, op, prec, mode);
 }
 
-int round_constant_expr_to_format_or_expansion_format(int *ternary, int *ternaryOkay, mpfr_t rop, node *func, int expansion_format_round, int format, mp_prec_t prec, mp_rnd_t mode) {
+int round_constant_expr_to_format_or_expansion_format_inner(int *ternary, int *ternaryOkay, mpfr_t rop, node *func, int expansion_format_round, int format, mp_prec_t prec, mp_rnd_t mode) {
   mp_prec_t startPrec, p;
   sollya_mpfi_t y, x;
   mpfr_t ya, yb, rya, ryb, tya;
@@ -1124,6 +1092,12 @@ int round_constant_expr_to_format_or_expansion_format(int *ternary, int *ternary
     if (prec + 10 > startPrec) startPrec = prec + 10;
   }
 
+  if (expansion_format_round) {
+    if (startPrec < 2100) {
+      startPrec = 2100;
+    }
+  }
+
   sollya_mpfi_init2(y, startPrec);
   mpfr_init2(ya, startPrec);
   mpfr_init2(yb, startPrec);
@@ -1381,7 +1355,18 @@ int round_constant_expr_to_format_or_expansion_format(int *ternary, int *ternary
   return 1;
 }
 
-int mpfr_mant_exp(mpfr_t rop, mp_exp_t *expo, mpfr_t op) {
+int round_constant_expr_to_format_or_expansion_format(int *ternary, int *ternaryOkay, mpfr_t rop, node *func, int expansion_format_round, int format, mp_prec_t prec, mp_rnd_t mode) {
+  int res;
+  node *f;
+
+  f = simplifyRationalErrorfree(func);
+  res = round_constant_expr_to_format_or_expansion_format_inner(ternary, ternaryOkay, rop, f, expansion_format_round, format, prec, mode);
+  free_memory(f);
+  
+  return res;
+}
+
+int sollya_mpfr_mant_exp(mpfr_t rop, mp_exp_t *expo, mpfr_t op) {
   mp_exp_t e;
   mp_prec_t p;
   mpfr_t temp;
@@ -1571,10 +1556,10 @@ node *rationalApprox(mpfr_t x, int n) {
   mpfr_init2(*denominator,mp_bits_per_limb*mpz_size(u));
   mpfr_set_z(*denominator,u,GMP_RNDN);
 
-  tree = safeMalloc(sizeof(node));
+  tree = allocateNode();
   tree->nodeType = DIV;
-  num = safeMalloc(sizeof(node));
-  denom = safeMalloc(sizeof(node));
+  num = allocateNode();
+  denom = allocateNode();
   num->nodeType = CONSTANT;
   denom->nodeType = CONSTANT;
   num->value = numerator;
@@ -1588,7 +1573,7 @@ node *rationalApprox(mpfr_t x, int n) {
   return tree;
 }
 
-int mpfr_round_to_ieee_format(mpfr_t rop, mpfr_t op, mp_prec_t prec, unsigned int width, mp_rnd_t mode) {
+static inline int __sollya_mpfr_round_to_ieee_format(mpfr_t rop, mpfr_srcptr op, mp_prec_t prec, unsigned int width, mp_rnd_t mode, int silent) {
   int res;
   mpfr_t result;
   unsigned int exponent;
@@ -1731,54 +1716,68 @@ int mpfr_round_to_ieee_format(mpfr_t rop, mpfr_t op, mp_prec_t prec, unsigned in
 
   /* Write back the result, while verifying if we don't get a double rounding there. */
   /* Double-rounding may occur if the precision of rop is smaller than the prec.     */
-  if (mpfr_set(rop,result,GMP_RNDN) != 0) {
+  if (mpfr_set(rop,result,mode) != 0) {
     if (!noRoundingWarnings) {
-      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_IEEE_754_2008_OPERATOR,"Warning: double rounding occurred on invoking the IEEE 754-2008 general rounding operator.\n");
+      if (!silent) {
+	printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_IEEE_754_2008_OPERATOR,"Warning: double rounding occurred on invoking the IEEE 754-2008 general rounding operator.\n");
+	printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
+      }
+    }
+  }
+  /* Compute the rounding direction that has finally been chosen */
+  res = mpfr_cmp(rop,result);
+
+  /* Clear the internal result */
+  mpfr_clear(result);
+
+  /* Return the ternary */
+  return res;
+}
+
+int sollya_mpfr_round_to_single_mode(mpfr_t rop, mpfr_srcptr x, mp_rnd_t mode) {
+  int tern;
+  
+  tern = __sollya_mpfr_round_to_ieee_format(rop, x, 24, 8, mode, 1);
+
+  if (tern != 0) {
+    if (!noRoundingWarnings) {
+      printMessage(1,SOLLYA_MSG_DOUBLE_ROUNDING_IN_ROUND_TO_SINGLE,"Warning: an undesired rounding occurred on a rounding to single.\n");
       printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
     }
   }
 
-  mpfr_clear(result);
-
-  /* Compute the rounding direction that has finally been chosen */
-  res = mpfr_cmp(rop,op);
-
-  return res;
+  return tern;
 }
 
-int mpfr_round_to_single(mpfr_t rop, mpfr_t x) {
-  return mpfr_round_to_ieee_format(rop, x, 24, 8, GMP_RNDN);
+int sollya_mpfr_round_to_single(mpfr_t rop, mpfr_srcptr x) {
+  return sollya_mpfr_round_to_single_mode(rop, x, GMP_RNDN);
 }
 
-int mpfr_round_to_single_mode(mpfr_t rop, mpfr_t x, mp_rnd_t mode) {
-  return mpfr_round_to_ieee_format(rop, x, 24, 8, mode);
+int sollya_mpfr_round_to_quad_mode(mpfr_t rop, mpfr_srcptr x, mp_rnd_t mode) {
+  return __sollya_mpfr_round_to_ieee_format(rop, x, 113, 15, mode, 0);
 }
 
-int mpfr_round_to_quad(mpfr_t rop, mpfr_t x) {
-  return mpfr_round_to_ieee_format(rop, x, 113, 15, GMP_RNDN);
+int sollya_mpfr_round_to_quad(mpfr_t rop, mpfr_srcptr x) {
+  return sollya_mpfr_round_to_quad_mode(rop, x, GMP_RNDN);
 }
 
-int mpfr_round_to_halfprecision(mpfr_t rop, mpfr_t x) {
-  return mpfr_round_to_ieee_format(rop, x, 11, 5, GMP_RNDN);
+int sollya_mpfr_round_to_halfprecision_mode(mpfr_t rop, mpfr_srcptr x, mp_rnd_t mode) {
+  return __sollya_mpfr_round_to_ieee_format(rop, x, 11, 5, mode, 0);
 }
 
-int mpfr_round_to_quad_mode(mpfr_t rop, mpfr_t x, mp_rnd_t mode) {
-  return mpfr_round_to_ieee_format(rop, x, 113, 15, mode);
+int sollya_mpfr_round_to_halfprecision(mpfr_t rop, mpfr_srcptr x) {
+  return sollya_mpfr_round_to_halfprecision_mode(rop, x, GMP_RNDN);
 }
 
-int mpfr_round_to_halfprecision_mode(mpfr_t rop, mpfr_t x, mp_rnd_t mode) {
-  return mpfr_round_to_ieee_format(rop, x, 11, 5, mode);
+int sollya_mpfr_round_to_doubleextended_mode(mpfr_t rop, mpfr_srcptr op, mp_rnd_t mode) {
+  return __sollya_mpfr_round_to_ieee_format(rop, op, 64, 15, mode, 0);
 }
 
-int mpfr_round_to_doubleextended(mpfr_t rop, mpfr_t op) {
-  return mpfr_round_to_ieee_format(rop, op, 64, 15, GMP_RNDN);
+int sollya_mpfr_round_to_doubleextended(mpfr_t rop, mpfr_srcptr op) {
+  return sollya_mpfr_round_to_doubleextended_mode(rop, op, GMP_RNDN);
 }
 
-int mpfr_round_to_doubleextended_mode(mpfr_t rop, mpfr_t op, mp_rnd_t mode) {
-  return mpfr_round_to_ieee_format(rop, op, 64, 15, mode);
-}
-
-int sollya_mpfr_rint_nearestint(mpfr_t rop, mpfr_t op, mpfr_rnd_t rnd) {
+int sollya_mpfr_rint_nearestint(mpfr_t rop, mpfr_srcptr op, mpfr_rnd_t rnd) {
   mpfr_t temp;
   int res;
 
