@@ -1,9 +1,12 @@
 /*
 
-  Copyright 2006-2013 by
+  Copyright 2006-2018 by
 
   Laboratoire de l'Informatique du Parallelisme,
-  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668
+  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
+
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe FACTAS,
+  Sophia Antipolis, France
 
   and by
 
@@ -12,7 +15,7 @@
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -61,29 +64,26 @@
 #include "chain.h"
 
 int round_to_format(mpfr_t rop, mpfr_t op, int prec, mp_rnd_t mode);
-int round_to_expansion_format(mpfr_t rop, mpfr_t op, int format, mp_rnd_t mode);
+int round_to_expansion_format(mpfr_t rop, mpfr_srcptr op, int format, mp_rnd_t mode);
 int round_to_format_or_expansion_format(mpfr_t rop, mpfr_t op, int expansion_format_round, int format, mp_prec_t prec, mp_rnd_t mode);
 int round_constant_expr_to_format_or_expansion_format(int *ternary, int *ternaryOkay, mpfr_t rop, node *op, int expansion_format_round, int format, mp_prec_t prec, mp_rnd_t mode);
-int mpfr_round_to_double(mpfr_t rop, mpfr_t op);
-int mpfr_round_to_single(mpfr_t rop, mpfr_t op);
-int mpfr_round_to_single_mode(mpfr_t rop, mpfr_t op, mp_rnd_t mode);
-int mpfr_round_to_quad(mpfr_t rop, mpfr_t x);
-int mpfr_round_to_halfprecision(mpfr_t rop, mpfr_t x);
-int mpfr_round_to_quad_mode(mpfr_t rop, mpfr_t x, mp_rnd_t mode);
-int mpfr_round_to_halfprecision_mode(mpfr_t rop, mpfr_t x, mp_rnd_t mode);
-int mpfr_round_to_doubledouble(mpfr_t rop, mpfr_t op);
-int mpfr_round_to_tripledouble(mpfr_t rop, mpfr_t op);
-int mpfr_round_to_doubleextended(mpfr_t rop, mpfr_t op);
+int sollya_mpfr_round_to_double(mpfr_t rop, mpfr_srcptr op);
+int sollya_mpfr_round_to_single(mpfr_t rop, mpfr_srcptr op);
+int sollya_mpfr_round_to_quad(mpfr_t rop, mpfr_srcptr x);
+int sollya_mpfr_round_to_halfprecision(mpfr_t rop, mpfr_srcptr x);
+int sollya_mpfr_round_to_doubledouble(mpfr_t rop, mpfr_srcptr op);
+int sollya_mpfr_round_to_tripledouble(mpfr_t rop, mpfr_srcptr op);
+int sollya_mpfr_round_to_doubleextended(mpfr_t rop, mpfr_srcptr op);
 int printDoubleInHexa(mpfr_t x);
 int printSimpleInHexa(mpfr_t x);
 int readHexa(mpfr_t res, char *c);
 node *roundPolynomialCoefficients(node *poly, chain *formats, mp_prec_t prec);
 int printDoubleExpansion(mpfr_t x);
 int printPolynomialAsDoubleExpansion(node *poly, mp_prec_t prec);
-void mpfr_round_to_format(mpfr_t rop, mpfr_t op, int format);
-int mpfr_mant_exp(mpfr_t rop, mp_exp_t *expo, mpfr_t op);
+void sollya_mpfr_round_to_format(mpfr_t rop, mpfr_srcptr op, int format);
+int sollya_mpfr_mant_exp(mpfr_t rop, mp_exp_t *expo, mpfr_t op);
 int roundRangeCorrectly(mpfr_t rop, mpfr_t a, mpfr_t b);
 node *rationalApprox(mpfr_t x, int n);
-int sollya_mpfr_rint_nearestint(mpfr_t rop, mpfr_t op, mpfr_rnd_t rnd);
+int sollya_mpfr_rint_nearestint(mpfr_t rop, mpfr_srcptr op, mpfr_rnd_t rnd);
 
 #endif /* ifdef DOUBLE_H*/
