@@ -1,5 +1,8 @@
 /*
-  Copyright 2007-2016 by
+  Copyright 2007-2022 by
+
+  Laboratoire de l'Informatique du Parallelisme,
+  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -8,18 +11,26 @@
   Boîte Courrier 169
   4, place Jussieu
   F-75252 Paris Cedex 05
-  France
+  France,
 
   LORIA (CNRS, INPL, INRIA, UHP, U-Nancy 2)
 
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
+
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage
+
   and by
 
-  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
-  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France.
+  Centre de recherche INRIA Sophia Antipolis Mediterranee,
+  Equipe FACTAS,
+  Sophia Antipolis, France.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -304,13 +315,21 @@ extern void endBuffer(void);
 #define ANNOTATEFUNCTION 279
 #define OBJECTNAME 280
 #define GETBACKTRACE 281
+#define GCD 282
+#define EUCLDIV 283
+#define EUCLMOD 284
+#define INTERPOLATE 285
+#define EXTERNALDATA 286
+#define EXTERNALDATAUSAGE 287
+#define BEZOUT 288
+
 
 int executeCommand(node *);
 
 node *copyThing(node *);
 node *deepCopyThing(node *);
 void *copyThingOnVoid(void *);
-void freeThing(node *);
+node *freeThing(node *);
 void rawPrintThing(node *);
 void freeThingOnVoid(void *);
 node *rewriteThingWithMemRefReuse(node *, node *);
@@ -478,6 +497,7 @@ node *makeNumerator(node *thing);
 node *makeDenominator(node *thing);
 node *makeSubstitute(node *thing1, node *thing2);
 node *makeComposePolynomials(node *thing1, node *thing2);
+node *makeBezout(node *thing1, node *thing2);
 node *makeCoeff(node *thing1, node *thing2);
 node *makeSubpoly(node *thing1, node *thing2);
 node *makeRoundcoefficients(node *thing1, node *thing2);
@@ -491,11 +511,15 @@ node *makeSupnorm(chain *thinglist);
 node *makeFindZeros(node *thing1, node *thing2);
 node *makeFPFindZeros(node *thing1, node *thing2);
 node *makeDirtyInfnorm(node *thing1, node *thing2);
+node *makeGcd(node *thing1, node *thing2);
+node *makeEuclDiv(node *thing1, node *thing2);
+node *makeEuclMod(node *thing1, node *thing2);
 node *makeNumberRoots(node *thing1, node *thing2);
 node *makeIntegral(node *thing1, node *thing2);
 node *makeDirtyIntegral(node *thing1, node *thing2);
 node *makeImplementPoly(chain *thinglist);
 node *makeImplementConst(chain *thinglist);
+node *makeInterpolate(chain *thinglist);
 node *makeNewCommand(node *thing1);
 node *makeCheckInfnorm(node *thing1, node *thing2, node *thing3);
 node *makeZeroDenominators(node *thing1, node *thing2);
@@ -543,6 +567,7 @@ node *makePrintXmlAppendFile(node *, node *);
 node *makeReadXml(node *);
 node *makeExecute(node *);
 node *makeExternalProc(char *, node *, chain *);
+node *makeExternalData(char *, node *);
 node *makeUnit();
 node *makeVariableDeclaration(chain *stringlist);
 node *makeProc(chain *stringlist, node *body, node *returnVal);
@@ -550,6 +575,7 @@ node *makeMatchElement(node *, node *, node *);
 node *makeProcIllim(char *arg, node *body, node *returnVal);
 node *makeApply(node *thing, chain *thinglist);
 node *makeExternalProcedureUsage(libraryProcedure *proc);
+node *makeExternalDataUsage(libraryData *data);
 
 node *parseString(char *str);
 
@@ -574,8 +600,10 @@ int isEmptyList(node *);
 int isStructure(node *tree);
 int isProcedure(node *tree);
 int isExternalProcedureUsage(node *tree);
+int isExternalDataUsage(node *tree);
 int isLibraryConstant(node *tree);
 int isError(node *);
+int evaluateThingToInteger(int *res, node *tree, int *defaultVal);
 int evaluateThingToConstant(mpfr_t result, node *tree, mpfr_t *defaultVal, int silent, int superSilent);
 int evaluateThingToRange(mpfr_t a, mpfr_t b, node *tree);
 int evaluateThingToString(char **result, node *tree);
