@@ -1,6 +1,12 @@
 /*
 
-  Copyright 2006-2016 by
+  Copyright 2006-2022 by
+
+  Laboratoire de l'Informatique du Parallelisme,
+  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
+
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -9,18 +15,27 @@
   Boîte Courrier 169
   4, place Jussieu
   F-75252 Paris Cedex 05
-  France
+  France,
 
-  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
-  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France
+  Sorbonne Université
+  CNRS, Laboratoire d'Informatique de Paris 6, LIP6
+  F - 75005 Paris
+  France,
+
+  LORIA (CNRS, INPL, INRIA, UHP, U-Nancy 2)
+
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
+  Sophia Antipolis, France
 
   and by
 
-  LORIA (CNRS, INPL, INRIA, UHP, U-Nancy 2).
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -71,11 +86,27 @@
 #include <stdint.h>
 #include <mpfr.h>
 #include <stdio.h>
+#include "base-functions.h"
 #include "chain.h"
 #include "library.h"
 #include "hooks.h"
 #include "polynomials.h"
 
+/* A macro to handle compiler warnings for intended fall-thrus in switch statements */
+
+#ifdef HAVE_FALLTHRU_ATTRIBUTE
+#      if HAVE_FALLTHRU_ATTRIBUTE
+#         define SOLLYA_INTENDED_FALLTHRU    __attribute__((fallthrough));
+#      else
+#         define SOLLYA_INTENDED_FALLTHRU
+#      endif
+#else
+#      define SOLLYA_INTENDED_FALLTHRU
+#endif
+
+/* End of the fall-thru macro */
+
+
 enum __point_eval_enum_t {
   POINT_EVAL_FAILURE = 0,
   POINT_EVAL_EXACT,
@@ -87,70 +118,34 @@ enum __point_eval_enum_t {
 };
 typedef enum __point_eval_enum_t point_eval_t;
 
+/* Possible values for nodeType */
 #define VARIABLE 0
 #define CONSTANT 1
 #define ADD 2
 #define SUB 3
 #define MUL 4
 #define DIV 5
-#define SQRT 6
-#define EXP 7
-#define LOG 8
-#define LOG_2 9
-#define LOG_10 10
-#define SIN 11
-#define COS 12
-#define TAN 13
-#define ASIN 14
-#define ACOS 15
-#define ATAN 16
-#define SINH 17
-#define COSH 18
-#define TANH 19
-#define ASINH 20
-#define ACOSH 21
-#define ATANH 22
-#define POW 23
-#define NEG 24
-#define ABS 25
-#define DOUBLE 26
-#define DOUBLEDOUBLE 27
-#define TRIPLEDOUBLE 28
-#define POLYNOMIAL 29
-#define ERF 30
-#define ERFC 31
-#define LOG_1P 32
-#define EXP_M1 33
-#define DOUBLEEXTENDED 34
-#define LIBRARYFUNCTION 35
-#define CEIL 36
-#define FLOOR 37
-#define PI_CONST 38
-#define SINGLE 39
-#define NEARESTINT 40
-#define LIBRARYCONSTANT 41
-#define PROCEDUREFUNCTION 42
-#define HALFPRECISION 43
-#define QUAD 44
+#define NEG 6
+#define UNARY_BASE_FUNC 7
+#define POW 8
+#define PI_CONST 9
+#define POLYNOMIAL 10
+#define LIBRARYFUNCTION 11
+#define LIBRARYCONSTANT 12
+#define PROCEDUREFUNCTION 13
+/* Attention: other values for nodeType defined in execute.h, and also #define MEMREF 278 below */
 
 #define SOLLYA_MAX_ARG_ARRAY_ALLOC_SIZE 33554432ull
 
+#ifndef NODE_TYPEDEF
+#define NODE_TYPEDEF
 typedef struct nodeStruct node;
+#endif
 
-struct nodeStruct
-{
-  int nodeType;
-  mpfr_t *value;
-  node *child1;
-  node *child2;
-  libraryFunction *libFun;
-  int libFunDeriv;
-  char *string;
-  chain *arguments;
-  libraryProcedure *libProc;
-  node **argArray;
-  int argArraySize;
-  size_t argArrayAllocSize;
+typedef struct memRefCacheStruct *memRefCache;
+struct memRefCacheStruct {
+  node *prev;
+  node *next;
   sollya_mpfi_t *evalCacheX, *evalCacheY;
   mp_prec_t evalCachePrec;
   node *derivCache;
@@ -170,6 +165,48 @@ struct nodeStruct
   int hashComputed;
   int treeSizeCache;
   int treeSizeCacheFilled;
+  node *substituteCacheY;
+  node *substituteCacheX;
+  int substituteCacheMaySimplify;
+  int containsNotANumbersIsCached;
+  int containsNotANumbersCacheResult;
+  int containsHooksIsCached;
+  int containsHooksCacheResult;
+  unsigned int containsHooksCall;
+  int isPureTreeIsCached;
+  int isPureTreeCacheResult;
+  int isConstantIsCached;
+  int isConstantCacheResult;
+  int faithEvalOptFunSupportCached;
+  int faithEvalOptFunSupportCacheResult;
+  int containsPiCached;
+  int containsPiResult;
+  int evaluateSignCached;
+  int evaluateSignResultSign;
+  int evaluateSignResultOkay;
+  int evaluateSignAuxCached;
+  int evaluateSignAuxResultSign;
+  int evaluateSignAuxResultOkay;
+  int tryRepresentAsPolynomialDoesNotWork;
+};
+
+struct nodeStruct
+{
+  int nodeType;
+  mpfr_t *value;
+  node *child1;
+  node *child2;
+  libraryFunction *libFun;
+  baseFunction const *baseFun;
+  long long int libFunDeriv;
+  char *string;
+  chain *arguments;
+  libraryProcedure *libProc;
+  libraryData *libData;
+  node **argArray;
+  size_t argArraySize;
+  size_t argArrayAllocSize;
+  memRefCache cache;
 };
 
 /* HELPER TYPE FOR THE PARSER */
@@ -191,45 +228,152 @@ struct rangetypeStruct
   mpfr_t *b;
 };
 
-/* Two static functions needed to access indirectly memory-referenced
-   node * structs.  The functions should be inlined for maximum
-   performance. They can be static for performance as they are used
-   only internally.
-*/
-
 #define MEMREF 278
 
-void *safeMalloc (size_t);
+void *safeMalloc(size_t);
+void safeFree(void *);
+
+/* Highly important and often used functions */
+
+extern node *memref_chain_start;
+void deferSignalHandling();
+void resumeSignalHandling();
+
+static inline void memref_chain_manipulate_start() {
+  deferSignalHandling();
+}
+
+static inline void memref_chain_manipulate_end() {
+  resumeSignalHandling();
+}
+
+static inline void link_memref(node *ptr) {
+  if (ptr == NULL) return;
+  if (ptr->nodeType != MEMREF) return;
+  if (ptr->cache == NULL) return;
+  memref_chain_manipulate_start();
+  ptr->cache->prev = NULL;
+  ptr->cache->next = memref_chain_start;
+  if (memref_chain_start != NULL) {
+    if (memref_chain_start->nodeType == MEMREF) {
+      if (memref_chain_start->cache != NULL) {
+	memref_chain_start->cache->prev = ptr;
+      }
+    }
+  }
+  memref_chain_start = ptr;
+  memref_chain_manipulate_end();
+}
+
+static inline void unlink_memref(node *ptr) {
+  if (ptr == NULL) return;
+  if (ptr->nodeType != MEMREF) return;
+  if (ptr->cache == NULL) return;
+  memref_chain_manipulate_start();
+  if (ptr->cache->prev == NULL) {
+    memref_chain_start = ptr->cache->next;
+    if (memref_chain_start != NULL) {
+      if (memref_chain_start->nodeType == MEMREF) {
+	if (memref_chain_start->cache != NULL) {
+	  memref_chain_start->cache->prev = NULL;
+	}
+      }
+    }
+  } else {
+    if (ptr->cache->prev->nodeType == MEMREF) {
+      if (ptr->cache->prev->cache != NULL) {
+	ptr->cache->prev->cache->next = ptr->cache->next;
+      }
+    }
+  }
+  if (ptr->cache->next != NULL) {
+    if (ptr->cache->next->nodeType == MEMREF) {
+      if (ptr->cache->next->cache != NULL) {
+	ptr->cache->next->cache->prev = ptr->cache->prev;
+      }
+    }
+  }
+  ptr->cache->prev = NULL;
+  ptr->cache->next = NULL;
+  memref_chain_manipulate_end();
+}
+
+static inline node* allocateNode() {
+  node *res;
+  res = (node *) safeMalloc(sizeof(node));
+  return res;
+}
+
+static inline void freeNode(node *ptr) {
+  if (ptr == NULL) return;
+  safeFree(ptr);
+}
+
+static inline void freeMemRef(node *ptr) {
+  if (ptr == NULL) return;
+  unlink_memref(ptr);
+  safeFree(ptr);
+}
 
 static inline node* addMemRefEvenOnNull(node *tree) {
+  void *ptr, *ptrCache;
   node *res;
 
-  res = (node *) safeMalloc(sizeof(node));
+  ptr = safeMalloc(sizeof(node) + sizeof(struct memRefCacheStruct));
+  ptrCache = ptr + sizeof(node);
+  res = (node *) ptr;
+
   res->nodeType = MEMREF;
   res->child1 = tree;
   res->libFunDeriv = 1;
   res->child2 = NULL;
   res->arguments = NULL;
   res->value = NULL;
-  res->evalCacheX = NULL;
-  res->evalCacheY = NULL;
-  res->evalCachePrec = 0;
-  res->derivCache = NULL;
-  res->derivUnsimplCache = NULL;
-  res->simplifyCacheDoesNotSimplify = -1;
-  res->simplifyCacheRationalMode = -1;
-  res->simplifyCache = NULL;
-  res->hornerCache = NULL;
-  res->isCorrectlyTyped = 0;
-  res->evaluationHook = NULL;
-  res->polynomialRepresentation = NULL;
-  res->memRefChildFromPolynomial = 0;
-  res->pointEvalCacheX = NULL;
-  res->pointEvalCacheY = NULL;
-  res->hashComputed = 0;
-  res->treeSizeCache = -1;
-  res->treeSizeCacheFilled = 0;
+  res->cache = (memRefCache) ptrCache;
 
+  res->cache->evalCacheX = NULL;
+  res->cache->evalCacheY = NULL;
+  res->cache->evalCachePrec = 0;
+  res->cache->derivCache = NULL;
+  res->cache->derivUnsimplCache = NULL;
+  res->cache->simplifyCacheDoesNotSimplify = -1;
+  res->cache->simplifyCacheRationalMode = -1;
+  res->cache->simplifyCache = NULL;
+  res->cache->hornerCache = NULL;
+  res->cache->isCorrectlyTyped = 0;
+  res->cache->evaluationHook = NULL;
+  res->cache->polynomialRepresentation = NULL;
+  res->cache->memRefChildFromPolynomial = 0;
+  res->cache->pointEvalCacheX = NULL;
+  res->cache->pointEvalCacheY = NULL;
+  res->cache->hashComputed = 0;
+  res->cache->treeSizeCache = -1;
+  res->cache->treeSizeCacheFilled = 0;
+  res->cache->substituteCacheY = NULL;
+  res->cache->substituteCacheX = NULL;
+  res->cache->substituteCacheMaySimplify = 0;
+  res->cache->containsNotANumbersIsCached = 0;
+  res->cache->containsNotANumbersCacheResult = 1;
+  res->cache->containsHooksIsCached = 0;
+  res->cache->containsHooksCacheResult = 0;
+  res->cache->containsHooksCall = 0u;
+  res->cache->isPureTreeIsCached = 0;
+  res->cache->isPureTreeCacheResult = 0;
+  res->cache->isConstantIsCached = 0;
+  res->cache->isConstantCacheResult = 0;
+  res->cache->faithEvalOptFunSupportCached = 0;
+  res->cache->faithEvalOptFunSupportCacheResult = 0;
+  res->cache->containsPiCached = 0;
+  res->cache->containsPiResult = 0;
+  res->cache->evaluateSignCached = 0;
+  res->cache->evaluateSignResultSign = 0;
+  res->cache->evaluateSignResultOkay = 0;
+  res->cache->evaluateSignAuxCached = 0;
+  res->cache->evaluateSignAuxResultSign = 0;
+  res->cache->evaluateSignAuxResultOkay = 0;
+  res->cache->tryRepresentAsPolynomialDoesNotWork = 0;
+  link_memref(res);
+  
   return res;
 }
 
@@ -240,11 +384,12 @@ static inline node* addMemRef(node *tree) {
 }
 
 static inline node* getMemRefChild(node *tree) {
+  if (tree == NULL) return NULL;
   if (tree->nodeType != MEMREF) return tree->child1;
   if (tree->child1 != NULL) return tree->child1;
-  if (tree->polynomialRepresentation == NULL) return NULL;
-  tree->child1 = polynomialGetExpressionExplicit(tree->polynomialRepresentation);
-  tree->memRefChildFromPolynomial = 1;
+  if (tree->cache->polynomialRepresentation == NULL) return NULL;
+  tree->child1 = polynomialGetExpressionExplicit(tree->cache->polynomialRepresentation);
+  tree->cache->memRefChildFromPolynomial = 1;
   return tree->child1;
 }
 
@@ -258,15 +403,18 @@ static inline node* accessThruMemRef(node *tree) {
   return res;
 }
 
-/* End of the static inline functions */
+/* End of highly used functions */
 
 void printTree(node *tree);
+char *sprintTreeImmediateWarning(node *tree);
 node* differentiate(node *tree);
+node* differentiateUnsimplified(node *tree);
 node* simplifyTree(node *tree);
 void free_memory(node *tree);
 int evaluateConstantExpression(mpfr_t result, node *tree, mp_prec_t prec);
 void evaluate(mpfr_t result, node *tree, mpfr_t x, mp_prec_t prec);
 void printValue(mpfr_t *value);
+char *sprintValueImmediateWarning(mpfr_t *value);
 node* copyTree(node *tree);
 int treeContainsHooks(node *tree);
 node* horner(node *tree);
@@ -279,6 +427,7 @@ node* expand(node *tree);
 node* simplifyTreeErrorfree(node *tree);
 int isNotUniformlyZero(node *tree);
 int isNotUniformlyInfinite(node *tree);
+int isIntegerConstant(node *tree);
 int getNumeratorDenominator(node **numerator, node **denominator, node *tree);
 node *substitute(node* tree, node *t);
 node *substituteEnhanced(node* tree, node *t, int doNotEvaluate, int maySimplify);
@@ -320,15 +469,23 @@ void simplifyMpfrPrec(mpfr_t rop, mpfr_t op);
 node *simplifyRationalErrorfree(node *tree);
 int tryEvaluateConstantTermToMpq(mpq_t res, node *tree);
 node *simplifyAllButDivision(node *tree);
-int mpfr_to_mpq( mpq_t y, mpfr_t x);
-int mpfr_to_mpz( mpz_t y, mpfr_t x);
+int sollya_mpfr_to_mpq( mpq_t y, mpfr_t x);
+int sollya_mpfr_to_mpz( mpz_t y, mpfr_t x);
 mp_prec_t getMpzPrecision(mpz_t x);
+void mpz_to_mpfr(mpfr_t res, mpz_t a);
+int mpqHoldsOnMpfr(mpfr_t res, mpq_t a);
 int containsOnlyRealNumbers(node * tree);
 void tryCopyTreeAnnotations(node *newTree, node *oldTree);
 int containsNonDifferentiableSubfunctions(node *tree);
+node *gcd(node *a, node *b);
+node *eucldiv(node *a, node *b);
+node *euclmod(node *a, node *b);
+void bezout(node **g, node **r, node **a, node **b, node *p, node *q);
+char *generateDecimalString(int, mpz_t, mpfr_exp_t, mp_prec_t);
 
 node *makeVariable();
 node *makeConstant(mpfr_t x);
+node *makeConstantMinPrec(mpfr_t x);
 node *makeConstantDouble(double x);
 node *makeConstantInt(int x);
 node *makeConstantUnsignedInt(unsigned int x);
@@ -337,45 +494,17 @@ node *makeAdd(node *op1, node *op2);
 node *makeSub(node *op1, node *op2);
 node *makeMul(node *op1, node *op2);
 node *makeDiv(node *op1, node *op2);
-node *makeSqrt(node *op1);
-node *makeExp(node *op1);
-node *makeLog(node *op1);
-node *makeLog2(node *op1);
-node *makeLog10(node *op1);
-node *makeSin(node *op1);
-node *makeCos(node *op1);
-node *makeTan(node *op1);
-node *makeAsin(node *op1);
-node *makeAcos(node *op1);
-node *makeAtan(node *op1);
-node *makePow(node *op1, node *op2);
 node *makeNeg(node *op1);
-node *makeAbs(node *op1);
-node *makeDouble(node *op1);
-node *makeSingle(node *op1);
-node *makeQuad(node *op1);
-node *makeHalfPrecision(node *op1);
-node *makeDoubledouble(node *op1);
-node *makeTripledouble(node *op1);
-node *makeErf(node *op1);
-node *makeErfc(node *op1);
-node *makeLog1p(node *op1);
-node *makeExpm1(node *op1);
-node *makeDoubleextended(node *op1);
-node *makeCeil(node *op1);
-node *makeFloor(node *op1);
-node *makeNearestInt(node *op1);
+node *makePow(node *op1, node *op2);
 node *makePi();
-node *makeSinh(node *op1);
-node *makeCosh(node *op1);
-node *makeTanh(node *op1);
-node *makeAsinh(node *op1);
-node *makeAcosh(node *op1);
-node *makeAtanh(node *op1);
-node *makeUnary(node *op1, int nodeType);
+node *makeUnary(node *op1, baseFunction const *baseFun);
 node *makeBinary(node *op1, node *op2, int opType);
 
 node* addMemRef(node *);
 node* accessThruMemRef(node *);
 
+void freeVariableCache();
+void resetVariableCacheHandling();
+
+
 #endif /* ifdef EXPRESSION_H*/
