@@ -1,11 +1,12 @@
 /*
 
-  Copyright 2006-2016 by
+  Copyright 2006-2018 by
 
   Laboratoire de l'Informatique du Parallelisme,
-  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668
+  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
-  and by
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -16,9 +17,16 @@
   F-75252 Paris Cedex 05
   France
 
+  and by
+
+  Sorbonne Université
+  CNRS, Laboratoire d'Informatique de Paris 6, LIP6
+  F - 75005 Paris
+  France.
+
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -294,6 +302,9 @@ void externalPlot(char *library, mpfr_t a, mpfr_t b, mp_prec_t samplingPrecision
   char *dataname;
   char *outputname;
   int n;
+  char *gnuplotname;
+
+  gnuplotname = getGnuplotName();
 
   if(samplingPrecision > prec) {
     printMessage(1,SOLLYA_MSG_SAMPLING_PREC_MUST_BE_LOWER_THAN_CURR_PREC,"Error: you must use a sampling precision lower than the current precision\n");
@@ -346,6 +357,9 @@ void externalPlot(char *library, mpfr_t a, mpfr_t b, mp_prec_t samplingPrecision
   sollyaFprintf(file, "set format y \"%%g\"\n");
   sollyaFprintf(file, "set xrange [%1.50e:%1.50e]\n", sollya_mpfr_get_d(a, GMP_RNDD),sollya_mpfr_get_d(b, GMP_RNDU));
   sollyaFprintf(file, "plot \"%s\" using 1:2 with dots t \"\"\n",dataname);
+  if ((name==NULL) || (type==PLOTFILE)) {
+    sollyaFprintf(file,"pause mouse close\n");
+  }
   fclose(file);
 
   file = fopen(dataname, "w");
@@ -426,10 +440,10 @@ void externalPlot(char *library, mpfr_t a, mpfr_t b, mp_prec_t samplingPrecision
   parserFlushInput();
   resumeSignalHandling();
   if ((name==NULL) || (type==PLOTFILE)) {
-    if (fork()==0) {
-      if (daemon(1,1) == 0) {
-	if (fork() == 0) {
-	  execlp("gnuplot", "gnuplot", "-persist", gplotname, NULL);
+    if (fork()==0) { /* The daemon call could fail (and will on certain Cygwin), in which case a double fork yields (almost (mod HUP etc.)) the same result. If daemon works, the fork has no effect */
+      if (daemon(1,0) == 0) { /* We want the prompt to return, so we need a daemon. We need to connect that daemon to /dev/null, which we do by setting the 2nd arg to zero */
+	if (fork() == 0) { /* We need a child of the daemon that waits for gnuplot to remove the file */
+	  execlp(gnuplotname, gnuplotname, gplotname, NULL);
 	  perror("An error occurred when calling gnuplot ");
 	  exit(1);
 	} else {
@@ -442,8 +456,8 @@ void externalPlot(char *library, mpfr_t a, mpfr_t b, mp_prec_t samplingPrecision
 	}
       } else {
 	fprintf(stderr, "Could not create a daemon for gnuplot\n");
-	if (fork() == 0) {
-	  execlp("gnuplot", "gnuplot", "-persist", gplotname, NULL);
+	if (fork() == 0) { /* We could not create a daemon but we still want a child to wait for us */
+	  execlp(gnuplotname, gnuplotname, gplotname, NULL);
 	  perror("An error occurred when calling gnuplot ");
 	  exit(1);
 	} else {
@@ -460,7 +474,7 @@ void externalPlot(char *library, mpfr_t a, mpfr_t b, mp_prec_t samplingPrecision
   }
   else { /* Case we have an output: no daemon */
     if (fork()==0) {
-      execlp("gnuplot", "gnuplot", "-persist", gplotname, NULL);
+      execlp(gnuplotname, gnuplotname, gplotname, NULL);
       perror("An error occurred when calling gnuplot ");
       exit(1);
     }
@@ -473,7 +487,7 @@ void externalPlot(char *library, mpfr_t a, mpfr_t b, mp_prec_t samplingPrecision
       sleep(1);
     }
   }
-  wait(NULL);
+
   deferSignalHandling();
   fflush(NULL);
   parserFlushInput();
