@@ -1,10 +1,13 @@
 /*
 
-  Copyright 2007-2016 by
+  Copyright 2007-2022 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
+
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
   UPMC Univ Paris 06
@@ -12,16 +15,25 @@
   Boîte Courrier 169
   4, place Jussieu
   F-75252 Paris Cedex 05
-  France
+  France,
+
+  Sorbonne Université
+  CNRS, Laboratoire d'Informatique de Paris 6, LIP6
+  F - 75005 Paris
+  France,
+
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
+  Sophia Antipolis, France
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
-  Sophia Antipolis, France.
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
 
   Contributors Ch. Lauter, S. Chevillard and J. Benoit
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -65,7 +77,7 @@
 #include "signalhandling.h"
 
 #define _POSIX_SOURCE
-#define _POSIX_C_SOURCE 199309L
+#define _POSIX_C_SOURCE 200112L
 
 #include <sys/types.h>
 #include <gmp.h>
@@ -102,18 +114,13 @@
 #include "config.h"
 #endif
 
-#if TIME_WITH_SYS_TIME
-#include <sys/time.h>
-#include <time.h>
-#else
 #if HAVE_SYS_TIME_H
 #include <sys/time.h>
-#else
+#endif
 #include <time.h>
-#endif
-#endif
 
 #include "version.h"
+#include "lto-compat.h"
 
 /* A constant for the global reused MPFI and MPFR variables */
 #define GLOBAL_REUSED_VARS_MAX_ALLOC 2048
@@ -124,12 +131,15 @@ int oldAutoPrint = 0;
 int oldVoidPrint = 0;
 int oldExternalProcedurePrint = 0;
 int printMode = PRINT_MODE_LEGACY;
+int queuedMode = 0;
+int lastModeSent = -1;
 FILE *warnFile = NULL;
 char *variablename = NULL;
 bitfield suppressedMessages = NULL;
 mp_prec_t defaultprecision = DEFAULTPRECISION;
 mp_prec_t tools_precision = DEFAULTPRECISION;
 char *temporyDirectory = NULL;
+char *gnuplotName = NULL;
 char *uniqueIdentifier = NULL;
 unsigned int globalSeed = 0;
 int defaultpoints = DEFAULTPOINTS;
@@ -146,16 +156,15 @@ int fullParentheses=0;
 int midpointMode = 0;
 int dieOnErrorMode = 0;
 int rationalMode = 0;
+unsigned long long int inexactPrinting = 0ull;
 int noRoundingWarnings = 0;
 int hopitalrecursions = DEFAULTHOPITALRECURSIONS;
 mpfr_t statediam;
 
 int eliminatePromptBackup;
 int libraryMode = 0;
-chain *readStack = NULL;
-chain *readStackTemp = NULL;
-chain *readStack2 = NULL;
 void *scanner = NULL;
+
 int promptToBePrinted = 0;
 int lastWasSyntaxError = 0;
 int lastCorrectlyExecuted = 0;
@@ -189,16 +198,16 @@ FILE *inputFile = NULL;
 int inputFileOpened = 0;
 int flushOutput = 0;
 
-node *memRefChainStart = NULL;
+int executingExternalCode = 0;
+pid_t externalCodePid = ((pid_t) 0);
+int externalCodePidSet = 0;
+
+node *memref_chain_start = NULL;
 
 /* END OF STATE OF THE TOOL */
 
 /* HELPER VARIABLES FOR THE LEXER/ PARSER */
 
-char *newReadFilename = NULL;
-char *newReadFilenameTemp = NULL;
-FILE **tempFDPtr;
-FILE *temp_fd;
 char *constBuffer = NULL;
 char *constBuffer2 = NULL;
 char *tempString = NULL;
@@ -298,6 +307,7 @@ int sollyaLibPrintmessageCalled = 0;
 
 /* Variables for the prompt-passed arguments */
 char **argsArgv = NULL;
+char *sollyaStartName = NULL;
 int argsArgc = 0;
 /* End of variables for the prompt-passed arguments */
 
@@ -329,6 +339,271 @@ void freeGlobalReusedMPFIVars();
 void freeGlobalReusedMPFRVars();
 void freeTool();
 
+void unqueueMode();
+
+static inline void __freeMemrefChainCachesUpdateChain() {
+  node *prev, *curr;
+  for (prev=NULL, curr=memref_chain_start;
+       (curr!=NULL) && ((curr->nodeType == MEMREF) && (curr->cache != NULL));
+       curr=(prev=curr)->cache->next) {
+    curr->cache->prev = prev;
+  }
+}
+
+static inline int __freeMemrefChainCachesHandleOnePrepare(node *ptr) {
+  const size_t maxNumPtrs = 6; /* Adapt if adding instructions below */
+  node *tempPtrs[maxNumPtrs];
+  size_t i, numPtrs;
+  
+  if (ptr == NULL) return 0;
+  if (ptr->nodeType != MEMREF) return 0;
+  if (ptr->cache == NULL) return 0;
+  if (ptr->libFunDeriv < 1) return 0;
+
+  for (i=((size_t) 0);i<maxNumPtrs;i++) {
+    tempPtrs[i] = NULL;
+  }
+  numPtrs = (size_t) 0;
+
+  if (ptr->cache->simplifyCache != NULL) {
+    tempPtrs[numPtrs] = ptr->cache->simplifyCache;
+    ptr->cache->simplifyCache = NULL;
+    numPtrs++;
+  }
+  if (ptr->cache->hornerCache != NULL) {
+    tempPtrs[numPtrs] = ptr->cache->hornerCache;
+    ptr->cache->hornerCache = NULL;
+    numPtrs++;
+  }
+  if (ptr->cache->derivCache != NULL) {
+    tempPtrs[numPtrs] = ptr->cache->derivCache;
+    ptr->cache->derivCache = NULL;
+    numPtrs++;
+  }
+  if (ptr->cache->derivUnsimplCache != NULL) {
+    tempPtrs[numPtrs] = ptr->cache->derivUnsimplCache;
+    ptr->cache->derivUnsimplCache = NULL;
+    numPtrs++;
+  }
+  if (ptr->cache->substituteCacheY != NULL) {
+    tempPtrs[numPtrs] = ptr->cache->substituteCacheY;
+    ptr->cache->substituteCacheY = NULL;
+    numPtrs++;
+  }
+  if (ptr->cache->substituteCacheX != NULL) {
+    tempPtrs[numPtrs] = ptr->cache->substituteCacheX;
+    ptr->cache->substituteCacheX = NULL;
+    numPtrs++;
+  }
+  
+  if (numPtrs < ((size_t) 1)) return 0;
+  for (i=((size_t) 0);i<numPtrs;i++) {
+    freeThing(tempPtrs[i]);
+  }
+  return 1;
+}
+
+static inline int __freeMemrefChainCachesHandleOneInner(node *ptr, int careful) {
+  int changed;
+  
+  if (ptr == NULL) return 0;
+  if (ptr->nodeType != MEMREF) return 0;
+  if (ptr->cache == NULL) return 0;
+  if (ptr->libFunDeriv < 1) return 0;
+  if (careful) {
+    if (__freeMemrefChainCachesHandleOnePrepare(ptr)) return 1;
+  }
+  changed = 0;
+  if (ptr->cache->simplifyCache != NULL) {
+    if ((ptr == ptr->cache->simplifyCache) && (ptr->libFunDeriv == 1)) {
+      freeThing(ptr);
+      return 1;
+    }
+    changed = 1;
+    if (freeThing(ptr->cache->simplifyCache) == ptr) return changed;
+    if (careful) return changed;
+    if ((ptr == NULL) ||
+	(ptr->nodeType != MEMREF) ||
+	(ptr->cache == NULL) ||
+	(ptr->libFunDeriv < 1)) return changed;
+    ptr->cache->simplifyCache = NULL;
+  }
+  if ((ptr == NULL) ||
+      (ptr->nodeType != MEMREF) ||
+      (ptr->cache == NULL) ||
+      (ptr->libFunDeriv < 1)) return changed;
+  if (ptr->cache->hornerCache != NULL) {
+    if ((ptr == ptr->cache->hornerCache) && (ptr->libFunDeriv == 1)) {
+      freeThing(ptr);
+      return 1;
+    }
+    changed = 1;
+    if (freeThing(ptr->cache->hornerCache) == ptr) return changed;
+    if (careful) return changed;
+    if ((ptr == NULL) ||
+	(ptr->nodeType != MEMREF) ||
+	(ptr->cache == NULL) ||
+	(ptr->libFunDeriv < 1)) return changed;
+    ptr->cache->hornerCache = NULL;
+  }
+  if ((ptr == NULL) ||
+      (ptr->nodeType != MEMREF) ||
+      (ptr->cache == NULL) ||
+      (ptr->libFunDeriv < 1)) return changed;
+  if (ptr->cache->derivCache != NULL) {
+    if ((ptr == ptr->cache->derivCache) && (ptr->libFunDeriv == 1)) {
+      freeThing(ptr);
+      return 1;
+    }
+    changed = 1;
+    if (freeThing(ptr->cache->derivCache) == ptr) return changed;
+    if (careful) return changed;
+    if ((ptr == NULL) ||
+	(ptr->nodeType != MEMREF) ||
+	(ptr->cache == NULL) ||
+	(ptr->libFunDeriv < 1)) return changed;
+    ptr->cache->derivCache = NULL;
+  }
+  if ((ptr == NULL) ||
+      (ptr->nodeType != MEMREF) ||
+      (ptr->cache == NULL) ||
+      (ptr->libFunDeriv < 1)) return changed;
+  if (ptr->cache->derivUnsimplCache != NULL) {
+    if ((ptr == ptr->cache->derivUnsimplCache) && (ptr->libFunDeriv == 1)) {
+      freeThing(ptr);
+      return 1;
+    }
+    changed = 1;
+    if (freeThing(ptr->cache->derivUnsimplCache) == ptr) return 1;
+    if (careful) return changed;
+    if ((ptr == NULL) ||
+	(ptr->nodeType != MEMREF) ||
+	(ptr->cache == NULL) ||
+	(ptr->libFunDeriv < 1)) return changed;
+    ptr->cache->derivUnsimplCache = NULL;
+  }
+  if ((ptr == NULL) ||
+      (ptr->nodeType != MEMREF) ||
+      (ptr->cache == NULL) ||
+      (ptr->libFunDeriv < 1)) return changed;
+  if (ptr->cache->substituteCacheY != NULL) {
+    if ((ptr == ptr->cache->substituteCacheY) && (ptr->libFunDeriv == 1)) {
+      freeThing(ptr);
+      return 1;
+    }
+    changed = 1;
+    if (freeThing(ptr->cache->substituteCacheY) == ptr) return 1;
+    if (careful) return changed;
+    if ((ptr == NULL) ||
+	(ptr->nodeType != MEMREF) ||
+	(ptr->cache == NULL) ||
+	(ptr->libFunDeriv < 1)) return changed;
+    ptr->cache->substituteCacheY = NULL;
+  }
+  if ((ptr == NULL) ||
+      (ptr->nodeType != MEMREF) ||
+      (ptr->cache == NULL) ||
+      (ptr->libFunDeriv < 1)) return changed;
+  if (ptr->cache->substituteCacheX != NULL) {
+    if ((ptr == ptr->cache->substituteCacheX) && (ptr->libFunDeriv == 1)) {
+      freeThing(ptr);
+      return 1;
+    }
+    changed = 1;
+    if (freeThing(ptr->cache->substituteCacheX) == ptr) return 1;
+    if (careful) return changed;
+    if ((ptr == NULL) ||
+	(ptr->nodeType != MEMREF) ||
+	(ptr->cache == NULL) ||
+	(ptr->libFunDeriv < 1)) return changed;
+    ptr->cache->substituteCacheX = NULL;
+  }
+  if ((ptr == NULL) ||
+      (ptr->nodeType != MEMREF) ||
+      (ptr->cache == NULL) ||
+      (ptr->libFunDeriv < 1)) return changed;
+  return changed;
+}
+
+static inline int __freeMemrefChainCachesHandleOne(node *ptr) {
+  if (ptr == NULL) return 0;
+  if (ptr->nodeType != MEMREF) return 0;
+  if (ptr->cache == NULL) return 0;
+  if (ptr->libFunDeriv < 1) return 0;
+  if (ptr->libFunDeriv > (((long long int) 1) << 14)) {
+    __freeMemrefChainCachesHandleOneInner(ptr, 0);
+  } 
+  return __freeMemrefChainCachesHandleOneInner(ptr, 1);
+}
+
+static inline int __freeMemrefChainCachesDoWork() {
+  node *curr, *tempNext;
+  int changed, ch;
+  changed = 0;
+  curr=memref_chain_start;
+  while ((curr!=NULL) && ((curr->nodeType == MEMREF) && (curr->cache != NULL))) {
+    tempNext=curr->cache->next;
+    ch = __freeMemrefChainCachesHandleOne(curr);
+    if (ch) changed = 1;
+    curr=tempNext;
+  }
+  return changed;
+}
+
+void freeMemrefChainCaches() {
+  int changed;
+  deferSignalHandling();
+  do {
+    __freeMemrefChainCachesUpdateChain();
+    changed = __freeMemrefChainCachesDoWork();
+    __freeMemrefChainCachesUpdateChain();
+  } while (changed);
+  resumeSignalHandling();
+}
+
+void enterExternalCode() {
+  deferSignalHandling();
+  fflush(NULL);
+  parserFlushInput();
+  resumeSignalHandling();
+  unqueueMode();
+  deferSignalHandling();
+  fflush(NULL);
+  parserFlushInput();
+  resumeSignalHandling();
+  if (!externalCodePidSet) {
+    externalCodePid = getpid();
+    externalCodePidSet = 1;
+  }
+  if (executingExternalCode < 0) executingExternalCode = 0;
+  executingExternalCode++;
+  if (executingExternalCode <= 0) executingExternalCode = 1;
+}
+
+void leaveExternalCode() {
+  pid_t myPid;
+  if (externalCodePidSet) {
+    myPid = getpid();
+    if (myPid != externalCodePid) {
+      exit(0);
+    }
+    externalCodePidSet = 0;
+    externalCodePid = ((pid_t) 0);
+  }
+  deferSignalHandling();
+  fflush(NULL);
+  parserFlushInput();
+  resumeSignalHandling();
+  unqueueMode();
+  deferSignalHandling();
+  fflush(NULL);
+  parserFlushInput();
+  resumeSignalHandling();
+  if (executingExternalCode <= 0) executingExternalCode = 1;
+  executingExternalCode--;
+  if (executingExternalCode < 0) executingExternalCode = 0;
+}
+
 void parserFlushInput() {
   if (!parserAlreadyRunning) return;
   fflush(getCurrentLexerStream());
@@ -339,7 +614,7 @@ void readManipulate() {
   fflush(NULL);
   parserFlushInput();
   resumeSignalHandling();
-  wait(NULL);
+  
   deferSignalHandling();
   fflush(NULL);
   parserFlushInput();
@@ -503,16 +778,20 @@ int sollya_ferror(FILE *fd) {
 }
 
 void makeToolDie() {
-	freeTool();
+  if (libraryMode) {
+    finalizeLibraryMode();
+  } else {
+    freeTool();
+  }
 
-	if (!eliminatePromptBackup) sollyaPrintf("\n");
+  if (!eliminatePromptBackup) sollyaPrintf("\n");
 
-	if (inputFileOpened) {
-	  fclose(inputFile);
-	  inputFileOpened = 0;
-	}
-
-	exit(2);
+  if (inputFileOpened) {
+    fclose(inputFile);
+    inputFileOpened = 0;
+  }
+  
+  exit(2);
 }
 
 void considerDyingOnError() {
@@ -523,11 +802,9 @@ void considerDyingOnError() {
   makeToolDie();
 }
 
-void normalMode() {
-  if (displayColor == 0) return;
-  displayColor = 0;
-  if (noColor) return;
-  if (eliminatePromptBackup) return;
+void sendNormalMode() {
+  if (lastModeSent == 1) return;
+  lastModeSent = 1;
   deferSignalHandling();
   fflush(NULL);
   printf("\e[0m");
@@ -535,11 +812,9 @@ void normalMode() {
   resumeSignalHandling();
 }
 
-void redMode() {
-  if (displayColor == 1) return;
-  displayColor = 1;
-  if (noColor) return;
-  if (eliminatePromptBackup) return;
+void sendRedMode() {
+  if (lastModeSent == 2) return;
+  lastModeSent = 2;
   deferSignalHandling();
   fflush(NULL);
   printf("\e[0m\e[31m");
@@ -547,11 +822,9 @@ void redMode() {
   resumeSignalHandling();
 }
 
-void blueMode() {
-  if (displayColor == 2) return;
-  displayColor = 2;
-  if (eliminatePromptBackup) return;
-  if (noColor) return;
+void sendBlueMode() {
+  if (lastModeSent == 3) return;
+  lastModeSent = 3;
   deferSignalHandling();
   fflush(NULL);
   printf("\e[0m\e[34m");
@@ -559,6 +832,59 @@ void blueMode() {
   resumeSignalHandling();
 }
 
+void queueNormalMode() {
+  queuedMode = 1;
+}
+
+void queueRedMode() {
+  queuedMode = 2;
+}
+
+void queueBlueMode() {
+  queuedMode = 3;
+}
+
+void unqueueMode() {
+  switch (queuedMode) {
+  case 1:
+    sendNormalMode();  
+    break;
+  case 2:
+    sendRedMode();  
+    break;
+  case 3:
+    sendBlueMode();  
+    break;
+  default:
+    break;
+  }
+  queuedMode = 0;
+}
+
+void normalMode() {
+  if (displayColor == 0) return;
+  displayColor = 0;
+  if (noColor) return;
+  if (eliminatePromptBackup) return;
+  queueNormalMode();
+}
+
+void redMode() {
+  if (displayColor == 1) return;
+  displayColor = 1;
+  if (noColor) return;
+  if (eliminatePromptBackup) return;
+  queueRedMode();
+}
+
+void blueMode() {
+  if (displayColor == 2) return;
+  displayColor = 2;
+  if (eliminatePromptBackup) return;
+  if (noColor) return;
+  queueBlueMode();
+}
+
 void setDisplayColor(int i) {
   switch (i) {
   case 2:
@@ -590,26 +916,6 @@ void restoreMode() {
   setDisplayColor(oldMode);
 }
 
-
-void blinkMode() {
-  if (eliminatePromptBackup) return;
-  deferSignalHandling();
-  fflush(NULL);
-  printf("\e[5m");
-  fflush(NULL);
-  resumeSignalHandling();
-}
-
-void unblinkMode() {
-  if (eliminatePromptBackup) return;
-  deferSignalHandling();
-  fflush(NULL);
-  printf("\e[25m");
-  fflush(NULL);
-  resumeSignalHandling();
-}
-
-
 void parseMode() {
   normalMode();
 }
@@ -622,7 +928,6 @@ void warningMode() {
   redMode();
 }
 
-
 void *safeCalloc (size_t nmemb, size_t size) {
   void *ptr;
   size_t myNmemb, mySize;
@@ -1019,24 +1324,31 @@ void newTokenLexed() {
   lastCorrectlyExecuted = 0;
 }
 
+int sollyaInternalVfprintfQueued(FILE *fd, const char *format, va_list varlist) {
+  if (fd == stdout) {
+    unqueueMode();
+  }
+  return sollyaInternalVfprintf(fd, format, varlist);
+}
+
 /* Precondition: fd can only be one of stdout and stderr */
 int sollyaVfprintfSpecial(FILE *fd, const char *format, va_list varlist) {
   int res;
 
-  if (printMode == PRINT_MODE_LEGACY) return sollyaInternalVfprintf(fd,format,varlist);
+  if (printMode == PRINT_MODE_LEGACY) return sollyaInternalVfprintfQueued(fd,format,varlist);
 
   if ((printMode == PRINT_MODE_WARNING_TO_STDERR) &&
       (displayColor == 1))
-    return sollyaInternalVfprintf(stderr,format,varlist);
+    return sollyaInternalVfprintfQueued(stderr,format,varlist);
 
   if ((printMode == PRINT_MODE_WARNING_TO_FILE) &&
       (displayColor == 1) &&
       (warnFile != NULL)) {
-    res = sollyaInternalVfprintf(warnFile,format,varlist);
+    res = sollyaInternalVfprintfQueued(warnFile,format,varlist);
     return res;
   }
 
-  return sollyaInternalVfprintf(fd,format,varlist);
+  return sollyaInternalVfprintfQueued(fd,format,varlist);
 }
 
 int sollyaVfprintf(FILE *fd, const char *format, va_list varlist) {
@@ -1044,7 +1356,7 @@ int sollyaVfprintf(FILE *fd, const char *format, va_list varlist) {
   if ((fd == stdout) || (fd == stderr))
     return sollyaVfprintfSpecial(fd,format,varlist);
 
-  return sollyaInternalVfprintf(fd,format,varlist);
+  return sollyaInternalVfprintfQueued(fd,format,varlist);
 }
 
 int installMessageCallback(int (*msgHandler) (sollya_msg_t, void *), void *data) {
@@ -1378,7 +1690,6 @@ void popTimeCounter(char *s) {
 
 void printPrompt(void) {
   if (eliminatePromptBackup) return;
-  if (readStack != NULL) return;
   parseMode();
   if (oldrlwrapcompatible && (!noColor)) {
     deferSignalHandling();
@@ -1417,30 +1728,15 @@ void freeFunctionSpecialVariables() {
 
 void freeTool() {
   if(variablename != NULL) safeFree(variablename);
-  if(newReadFilename != NULL) safeFree(newReadFilename);
   if(suppressedMessages != NULL) freeBitfield(suppressedMessages);
 
   if (!(eliminatePromptBackup == 1)) {
     removePlotFiles();
   }
 
-  while ((readStack != NULL) && (readStack2 != NULL)) {
-    temp_fd = *((FILE **) (readStack2->value));
-    fclose(temp_fd);
-    safeFree(readStack2->value);
-    readStackTemp = readStack2->next;
-    safeFree(readStack2);
-    readStack2 = readStackTemp;
-    safeFree(readStack->value);
-    readStackTemp = readStack->next;
-    safeFree(readStack);
-    readStack = readStackTemp;
-  }
   parserAlreadyRunning = 0;
   yylex_destroy(scanner);
-  freeFunctionLibraries();
-  freeConstantLibraries();
-  freeProcLibraries();
+  freeLibraries();
   freeCounter();
   freeSymbolTable(symbolTable, freeThingOnVoid);
   symbolTable = NULL;
@@ -1449,13 +1745,18 @@ void freeTool() {
   freeFunctionSpecialVariables();
   freeGlobalReusedMPFIVars();
   freeGlobalReusedMPFRVars();
+  freeVariableCache();
+  polynomialFreeCaches();
+  freeMemrefChainCaches();
   polynomialFreeCaches();
   declaredSymbolTable = NULL;
   backtraceStack = NULL;
   mpfr_clear(statediam);
   safeFree(temporyDirectory); temporyDirectory = NULL;
+  safeFree(gnuplotName); gnuplotName = NULL;
   safeFree(uniqueIdentifier); uniqueIdentifier = NULL;
   mpfr_free_cache();
+  memref_chain_start = NULL;
   normalMode();
 }
 
@@ -1475,7 +1776,6 @@ void initToolDefaults() {
   tools_precision = DEFAULTPRECISION;
   defaultpoints = DEFAULTPOINTS;
   taylorrecursions = DEFAULTTAYLORRECURSIONS;
-  memRefChainStart = NULL;
   dyadic = 0;
   verbosity = 1;
   activateMessageNumbers = 0;
@@ -1483,6 +1783,7 @@ void initToolDefaults() {
   fileNumber = 0;
   autosimplify = 1;
   rationalMode = 0;
+  inexactPrinting = 0ull;
   timecounting = 0;
   if (timeStack != NULL) freeCounter();
   timeStack=NULL;
@@ -1528,6 +1829,18 @@ void initToolDefaults() {
   tempNode = addMemRef(tempNode);
   symbolTable = addEntry(symbolTable, "__argv", tempNode, copyThingOnVoid);
   freeThing(tempNode);
+  if (sollyaStartName != NULL) {
+    tempNode = makeString(sollyaStartName);
+  } else {
+    if (strlen(PACKAGE_NAME) != 0) {
+      tempNode = makeString(PACKAGE_NAME);
+    } else {
+      tempNode = makeString("sollya");
+    }
+  }
+  tempNode = addMemRef(tempNode);
+  symbolTable = addEntry(symbolTable, "__program_name", tempNode, copyThingOnVoid);
+  freeThing(tempNode);  
   pidStr = getUniqueId();
   uniqueStr = (char *) safeCalloc(4 + strlen(PACKAGE_STRING) + 1 + strlen(pidStr) + 1 + 8 * sizeof(int) + 1, sizeof(char));
   mySeed = (unsigned int) ((unsigned int) time(NULL)) + ((unsigned int) globalSeed);
@@ -1567,14 +1880,17 @@ void restartTool() {
   freeFunctionSpecialVariables();
   freeGlobalReusedMPFIVars();
   freeGlobalReusedMPFRVars();
+  freeVariableCache();
+  polynomialFreeCaches();
+  freeMemrefChainCaches();
   polynomialFreeCaches();
   declaredSymbolTable = NULL;
   backtraceStack = NULL;
-  freeFunctionLibraries();
-  freeConstantLibraries();
-  freeProcLibraries();
+  freeLibraries();
   mpfr_free_cache();
   initToolDefaults();
+  executingExternalCode = 0;
+  queuedMode = 0;
   parseMode();
 }
 
@@ -1624,6 +1940,26 @@ char *initTempDir() {
   return res;
 }
 
+char *initGnuplotName() {
+  char *res;
+  char *staticRes;
+  size_t len;
+
+  staticRes = getenv("SOLLYA_GNUPLOT");
+  if ((staticRes != NULL) && ((len = strlen(staticRes)) != 0)) {
+    res = safeCalloc(len + 1, sizeof(char));
+    strcpy(res, staticRes);
+  } else {
+    staticRes = "gnuplot";
+    len = strlen(staticRes);
+    res = safeCalloc(len + 1, sizeof(char));
+    strcpy(res, staticRes);
+  }
+
+  return res;
+}
+
+
 char *initUniqueId() {
   char *res, *str;
   pid_t pid;
@@ -1652,6 +1988,15 @@ char *getTempDir() {
   return temporyDirectory;
 }
 
+char *getGnuplotName() {
+
+  if (gnuplotName == NULL) {
+    gnuplotName = initGnuplotName();
+  }
+
+  return gnuplotName;
+}
+
 char *getUniqueId() {
 
   if (uniqueIdentifier == NULL) {
@@ -1822,6 +2167,7 @@ int initializeLibraryMode(void *(*myActualMalloc)(size_t),
   void *ptr;
   int k, allArgsOkay;
   libraryMode = 1;
+  memref_chain_start = NULL;
   oldGMPMalloc = NULL;
   oldGMPRealloc = NULL;
   oldGMPFree = NULL;
@@ -1846,6 +2192,8 @@ int initializeLibraryMode(void *(*myActualMalloc)(size_t),
   lastMessageSuppressedResult = -1;
   inputFileOpened = 0;
   flushOutput = 0;
+  executingExternalCode = 0;
+  queuedMode = 0;
   oldAutoPrint = 0;
   oldExternalProcedurePrint = 0;
   printMode = PRINT_MODE_LEGACY;
@@ -1865,6 +2213,7 @@ int initializeLibraryMode(void *(*myActualMalloc)(size_t),
       }
     }
   }
+  sollyaStartName = NULL;
   initToolDefaults();
   handlingCtrlC = 0;
   lastHandledSignal = 0;
@@ -1888,10 +2237,12 @@ int finalizeLibraryMode() {
     safeFree(argsArgv);
     argsArgv = NULL;
   }
+  if (sollyaStartName != NULL) {
+    safeFree(sollyaStartName);
+    sollyaStartName = NULL;
+  }
   if(variablename != NULL) safeFree(variablename);
   variablename = NULL;
-  if(newReadFilename != NULL) safeFree(newReadFilename);
-  newReadFilename = NULL;
   if(suppressedMessages != NULL) freeBitfield(suppressedMessages);
   suppressedMessages = NULL;
 
@@ -1899,21 +2250,7 @@ int finalizeLibraryMode() {
     removePlotFiles();
   }
 
-  while ((readStack != NULL) && (readStack2 != NULL)) {
-    temp_fd = *((FILE **) (readStack2->value));
-    fclose(temp_fd);
-    safeFree(readStack2->value);
-    readStackTemp = readStack2->next;
-    safeFree(readStack2);
-    readStack2 = readStackTemp;
-    safeFree(readStack->value);
-    readStackTemp = readStack->next;
-    safeFree(readStack);
-    readStack = readStackTemp;
-  }
-  freeFunctionLibraries();
-  freeConstantLibraries();
-  freeProcLibraries();
+  freeLibraries();
   freeCounter();
   freeSymbolTable(symbolTable, freeThingOnVoid);
   symbolTable = NULL;
@@ -1922,11 +2259,15 @@ int finalizeLibraryMode() {
   freeFunctionSpecialVariables();
   freeGlobalReusedMPFIVars();
   freeGlobalReusedMPFRVars();
+  freeVariableCache();
+  polynomialFreeCaches();
+  freeMemrefChainCaches();
   polynomialFreeCaches();
   declaredSymbolTable = NULL;
   backtraceStack = NULL;
   mpfr_clear(statediam);
   safeFree(temporyDirectory); temporyDirectory = NULL;
+  safeFree(gnuplotName); gnuplotName = NULL;
   safeFree(uniqueIdentifier); uniqueIdentifier = NULL;
   mpfr_free_cache();
   uninstallMessageCallback();
@@ -1942,11 +2283,12 @@ int finalizeLibraryMode() {
   oldGMPMalloc = NULL;
   oldGMPRealloc = NULL;
   oldGMPFree = NULL;
+  memref_chain_start = NULL;
   libraryMode = 0;
   return 1;
 }
 
-double sollya_mpfr_get_d(mpfr_t op, mpfr_rnd_t rnd) {
+double sollya_mpfr_get_d(mpfr_srcptr op, mpfr_rnd_t rnd) {
   union {
     uint64_t i;
     double d;
@@ -2357,6 +2699,7 @@ static int general(int argc, char *argv[]) {
   int printBanner;
   int printBuiltInfo;
 
+  memref_chain_start = NULL;
   oldGMPMalloc = NULL;
   oldGMPRealloc = NULL;
   oldGMPFree = NULL;
@@ -2375,10 +2718,13 @@ static int general(int argc, char *argv[]) {
   doNotModifyStackSize = 0;
   inputFileOpened = 0;
   flushOutput = 0;
+  executingExternalCode = 0;
+  queuedMode = 0;
   oldAutoPrint = 0;
   oldExternalProcedurePrint = 0;
   printMode = PRINT_MODE_LEGACY;
   warnFile = NULL;
+  sollyaStartName = NULL;
 
   argsArgRead = 0;
   argsArgv = NULL;
@@ -2437,7 +2783,7 @@ static int general(int argc, char *argv[]) {
 	  sollyaPrintf("  --warninfile[append] <file>  print warning messages into a file instead on the standard output\n");
 	  sollyaPrintf("  --warnonstderr               print warning messages on error output instead on the standard output\n");
 	  sollyaPrintf("\nFor help on %s commands type \"help;\" on the %s prompt.\n",PACKAGE_NAME,PACKAGE_NAME);
-	  sollyaPrintf("A complete documentation on %s is available on the %s website <http://sollya.gforge.inria.fr/>.\n",PACKAGE_NAME,PACKAGE_NAME);
+	  sollyaPrintf("A complete documentation on %s is available on the %s website <http://sollya.org/>.\n",PACKAGE_NAME,PACKAGE_NAME);
           sollyaPrintf("\nReport bugs to <%s>\n",PACKAGE_BUGREPORT);
 	} else { /* argv[i] is "--version" */
           if (!printBanner) sollyaPrintf("%s\n\n",PACKAGE_STRING); /* For help2man to recognize the name of the tool */
@@ -2451,7 +2797,7 @@ static int general(int argc, char *argv[]) {
           }
 	  sollyaPrintf("\n");
 	}
- 	return 1;
+ 	return 0;
       } else
 	if (strcmp(argv[i],"--args") == 0) argsArgRead = 1; else
 	  if (strcmp(argv[i],"--nocolor") == 0) noColor = 1; else
@@ -2549,6 +2895,17 @@ static int general(int argc, char *argv[]) {
     }
   }
 
+  if (argc >= 1) {
+    if (strlen(argv[0]) != 0) {
+      sollyaStartName = (char *) safeCalloc(strlen(argv[0]) + 1, sizeof(char));
+      strcpy(sollyaStartName, argv[0]);
+    } else {
+      sollyaStartName = NULL;
+    }
+  } else {
+    sollyaStartName = NULL;
+  }
+  
   if ((!inputFileOpened) && (!eliminatePromptBackup)) {
     if (tcgetattr(0,&termAttr) == -1) {
       eliminatePromptBackup = 1;
@@ -2633,6 +2990,7 @@ static int general(int argc, char *argv[]) {
     lastWasSyntaxError = 0;
     finishedBeforeParsing = parserCheckEof();
     (void) finishedBeforeParsing;
+    resetVariableCacheHandling();
     parseAbort = yyparse(scanner);
     lastWasError = 0;
     if (parsedThing != NULL) {
@@ -2668,6 +3026,7 @@ static int general(int argc, char *argv[]) {
 	  resumeSignalHandling();
         }
 	libraryMode = 0;
+	executingExternalCode = 0;
 	pushTimeCounter();
 	executeAbort = executeCommand(parsedThing);
 	lastCorrectlyExecuted = 1;
@@ -2689,6 +3048,8 @@ static int general(int argc, char *argv[]) {
 	displayColor = -1; normalMode();
 	blockSignals(eliminatePromptBackup);
 	lastWasError = 1;
+	executingExternalCode = 0;
+	unqueueMode();
 	if (lastHandledSignal != 0) {
 	  switch (lastHandledSignal) {
 	  case HANDLING_SIGINT:
@@ -2749,6 +3110,8 @@ static int general(int argc, char *argv[]) {
     if (parseAbort || executeAbort) break;
     promptToBePrinted = 1;
     helpNotFinished = 0;
+    executingExternalCode = 0;
+    unqueueMode();
   }
 
   freeTool();
@@ -2760,6 +3123,10 @@ static int general(int argc, char *argv[]) {
     safeFree(argsArgv);
     argsArgv = NULL;
   }
+  if (sollyaStartName != NULL) {
+    safeFree(sollyaStartName);
+    sollyaStartName = NULL;
+  }
   
   if (!eliminatePromptBackup) sollyaPrintf("\n");
 
@@ -2786,7 +3153,7 @@ static int general(int argc, char *argv[]) {
   }
 }
 
-int sollya_tool_main(int argc, char *argv[]) {
+EXPORTED_FUNC_DEF(int, sollya_tool_main, int argc, char *argv[]) {
   return general(argc, argv);
 }
 
