@@ -1,10 +1,13 @@
 /*
 
-  Copyright 2007-2016 by
+  Copyright 2007-2021 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
+
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
   UPMC Univ Paris 06
@@ -12,16 +15,25 @@
   Boîte Courrier 169
   4, place Jussieu
   F-75252 Paris Cedex 05
-  France
+  France,
+
+  Sorbonne Université
+  CNRS, Laboratoire d'Informatique de Paris 6, LIP6
+  F - 75005 Paris
+  France,
+
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
+  Sophia Antipolis, France
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
-  Sophia Antipolis, France.
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -122,18 +134,12 @@ extern int fullParentheses;
 extern int midpointMode;
 extern int dieOnErrorMode;
 extern int rationalMode;
+extern unsigned long long int inexactPrinting;
 extern int noRoundingWarnings;
 extern int hopitalrecursions;
 extern int eliminatePromptBackup;
 extern int libraryMode;
-extern chain *readStack;
-extern chain *readStackTemp;
-extern chain *readStack2;
 extern void *scanner;
-extern char *newReadFilename;
-extern char *newReadFilenameTemp;
-extern FILE **tempFDPtr;
-extern FILE *temp_fd;
 extern node *temp_node;
 extern mpfr_t *mpfr_temp;
 extern mpfr_t *mpfr_temp2;
@@ -161,8 +167,9 @@ extern int *tempIntPtr;
 extern FILE *inputFile;
 extern int inputFileOpened;
 extern int flushOutput;
+extern int executingExternalCode;
 
-extern node *memRefChainStart;
+extern node *memref_chain_start;
 
 extern void *rpl_malloc(size_t n);
 extern void *rpl_realloc(void *, size_t n);
@@ -187,6 +194,8 @@ struct __sollya_time_struct_t {
 };
 typedef struct __sollya_time_struct_t sollya_time_t;
 
+void safeFree(void *);
+
 void printPrompt(void);
 void recoverFromError(void);
 void demaskString(char*, char*);
@@ -194,7 +203,6 @@ char *maskString(char *);
 void *safeCalloc (size_t nmemb, size_t size);
 void *safeMalloc (size_t size);
 void *safeRealloc (void *ptr, size_t size);
-void safeFree(void *ptr);
 int printMessage(int verb, int msgNum, const char *format, ...);
 int messageHasEnoughVerbosityAndIsNotSuppressed(int verb, int msgNum);
 int sollyaVfprintf(FILE *fd, const char *format, va_list varlist);
@@ -275,7 +283,7 @@ int getRoundingWarnings();
 void setRoundingWarnings(int);
 int getRationalMode();
 void setRationalMode(int);
-double sollya_mpfr_get_d(mpfr_t, mpfr_rnd_t);
+double sollya_mpfr_get_d(mpfr_srcptr, mpfr_rnd_t);
 sollya_mpfi_t *getReusedGlobalMPFIVars(unsigned int, mp_prec_t);
 void returnReusedGlobalMPIVars(unsigned int);
 mpfr_t *getReusedGlobalMPFRVars(unsigned int, mp_prec_t);
@@ -289,6 +297,9 @@ size_t sollya_fwrite(const void *, size_t, size_t, FILE *);
 int sollya_gettime(sollya_time_t *);
 void readManipulate();
 void parserFlushInput();
+void enterExternalCode();
+void leaveExternalCode();
+char *getGnuplotName();
 
 /* Compatibility macros */
 
