@@ -1,6 +1,9 @@
 /*
 
-  Copyright 2014-2016 by
+  Copyright 2014-2021 by
+
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -9,11 +12,26 @@
   Boîte Courrier 169
   4, place Jussieu
   F-75252 Paris Cedex 05
-  France
+  France,
 
-  Contributor Ch. Lauter
+  Sorbonne Université
+  CNRS, Laboratoire d'Informatique de Paris 6, LIP6
+  F - 75005 Paris
+  France,
 
-  christoph.lauter@lip6.fr
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage
+
+  and by
+
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe FACTAS,
+  Sophia Antipolis, France.
+
+  Contributors Ch. Lauter, S. Chevillard
+
+  christoph.lauter@christoph-lauter.org
+  sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
   environment for safe floating-point code development. It is
@@ -58,6 +76,499 @@
 #include "infnorm.h"
 #include "sturm.h"
 #include "hooks.h"
+#include "expression.h"
+
+#define STURM_MAX_DEGREE          (1000)
+#define DESCARTES_RULE_MAX_DEGREE (16000)
+
+static inline int evaluateSignAtPoint(int *sign, node *f, mpfr_t c) {
+  node *g, *h;
+  int res, s;
+
+  h = addMemRef(makeConstant(c));
+  tryRepresentAsPolynomial(h);
+  g = substitute(f, h);
+  free_memory(h);
+
+  res = evaluateSign(&s, g);
+  
+  free_memory(g);
+  if (res) {
+    *sign = s;
+  }
+
+  return res;
+}
+
+
+/* Basic check if the polynomial f has a strictly positive zero */
+static inline int descartesRuleCoefficients(int *hasZero, node **coefficients, int n) {
+  int currSign, changes, i, s = 0;
+
+  for (i=n-1;i>0;i--) {
+    if (coefficients[i] != NULL) {
+      break;
+    }
+  }
+  if (coefficients[i] == NULL) return 0;
+  for (;i>0;i--) {
+    if (coefficients[i] != NULL) {
+      if (!evaluateSign(&s, coefficients[i])) return 0;
+      if (s != 0) {
+	break;
+      }
+    }
+  }
+  currSign = s;
+  changes = 0;
+  for (i--;i>=0;i--) {
+    if (coefficients[i] != NULL) {
+      if (!evaluateSign(&s, coefficients[i])) return 0;
+    } else {
+      s = 0;
+    }
+    if (s != 0) {
+      if (s!=currSign) {
+	changes++;
+      }
+      currSign = s;
+    }
+  }
+  if (changes == 0) {
+    *hasZero = 0;
+    return 1;
+  }
+  if ((changes & 1) == 0) {
+    return 0;
+  }
+  *hasZero = 1;
+  return 1;
+}
+
+/* Basic check if the polynomial f has a zero on the interval [0;c] */
+static inline int descartesRuleBasic(int *hasZero, node *f, mpfr_t c) {
+  mp_prec_t p;
+  sollya_mpfi_t x, y;
+  mpfr_t t;
+  int deg;
+  node **coefficients;
+  node **rev_coefficients;
+  node *tt, *q, *g;
+  int resA, hasZerosA, i;
+  int s;
+  
+  p = mpfr_get_prec(c);
+  mpfr_init2(t, p);
+  mpfr_set_si(t, 0, GMP_RNDN);
+  sollya_mpfi_init2(x, p);
+  sollya_mpfi_init2(y, ((p > ((mp_prec_t) 100)) ? p : ((mp_prec_t) 100)));
+  sollya_mpfi_interv_fr(x, t, c);
+  evaluateInterval(y, f, NULL, x);
+  if (sollya_mpfi_has_nan(y) ||
+      sollya_mpfi_has_infinity(y)) {
+    sollya_mpfi_clear(x);
+    sollya_mpfi_clear(y);
+    mpfr_clear(t);
+    return 0;
+  }
+  if (!sollya_mpfi_has_zero(y)) {
+    sollya_mpfi_clear(x);
+    sollya_mpfi_clear(y);
+    mpfr_clear(t);
+    *hasZero = 0;
+    return 1;
+  } 
+  sollya_mpfi_set_si(x, 0);
+  evaluateInterval(y, f, NULL, x);
+  if (sollya_mpfi_has_nan(y) ||
+      sollya_mpfi_has_infinity(y)) {
+    sollya_mpfi_clear(x);
+    sollya_mpfi_clear(y);
+    mpfr_clear(t);
+    return 0;
+  }
+  if (sollya_mpfi_is_zero(y)) {
+    sollya_mpfi_clear(x);
+    sollya_mpfi_clear(y);
+    mpfr_clear(t);
+    *hasZero = 1;
+    return 1;
+  }
+  if (sollya_mpfi_has_zero(y)) {
+    if (evaluateSignAtPoint(&s, f, t)) {
+      if (s == 0) {
+	sollya_mpfi_clear(x);
+        sollya_mpfi_clear(y);
+	mpfr_clear(t);
+	*hasZero = 1;
+	return 1;
+      }
+    }
+  }
+  sollya_mpfi_interv_fr(x, c, c);
+  evaluateInterval(y, f, NULL, x);
+  if (sollya_mpfi_has_nan(y) ||
+      sollya_mpfi_has_infinity(y)) {
+    sollya_mpfi_clear(x);
+    sollya_mpfi_clear(y);
+    mpfr_clear(t);
+    return 0;
+  }
+  if (sollya_mpfi_is_zero(y)) {
+    sollya_mpfi_clear(x);
+    sollya_mpfi_clear(y);
+    mpfr_clear(t);
+    *hasZero = 1;
+    return 1;
+  }
+  if (sollya_mpfi_has_zero(y)) {
+    if (evaluateSignAtPoint(&s, f, c)) {
+      if (s == 0) {
+	sollya_mpfi_clear(x);
+	sollya_mpfi_clear(y);
+        mpfr_clear(t);
+	*hasZero = 1;
+	return 1;
+      }
+    }
+  }
+  sollya_mpfi_clear(y);
+  sollya_mpfi_clear(x);
+  mpfr_clear(t);
+
+  getCoefficients(&deg, &coefficients, f);
+  if (deg < 0) {
+    return 0;
+  }
+  deg++;
+  if (deg <= 0) {
+    return 0;
+  }
+
+  resA = descartesRuleCoefficients(&hasZerosA, coefficients, deg);
+
+  if (!resA) {
+    rev_coefficients = safeCalloc(deg, sizeof(node *));
+    for (i=0;i<deg;i++) {
+      if (coefficients[(deg-1)-i] != NULL) {
+	rev_coefficients[i] = copyTree(coefficients[(deg-1)-i]);
+      } else {
+	rev_coefficients[i] = makeConstantInt(0);
+      }
+    }
+    q = makePolynomialConstantExpressions(rev_coefficients, deg-1);
+    for (i=0;i<deg;i++) {
+      free_memory(rev_coefficients[i]);
+    }
+    safeFree(rev_coefficients);
+
+    tt = addMemRef(makeAdd(makeVariable(), makeDiv(makeConstantInt(1), makeConstant(c))));
+    tryRepresentAsPolynomial(tt);
+    g = substitute(q, tt);
+    free_memory(tt);
+    free_memory(q); 
+
+    mpfr_init2(t, 12);
+    mpfr_set_si(t, 0, GMP_RNDN);
+    if (evaluateSignAtPoint(&s, g, t)) {
+      mpfr_clear(t);
+      if (s == 0) {
+	free_memory(g);
+	*hasZero = 1;
+	return 1;
+      } else {
+	for (i=0;i<deg;i++) {
+	  if (coefficients[i] != NULL) {
+	    free_memory(coefficients[i]);
+	  }
+	}
+	safeFree(coefficients);
+	getCoefficients(&deg, &coefficients, g);
+        free_memory(g);
+	if (deg < 0) {
+	  return 0;
+	}
+	deg++;
+	if (deg <= 0) {
+	  return 0;
+	}
+ 
+	resA = descartesRuleCoefficients(&hasZerosA, coefficients, deg);
+       
+      }
+    }
+  }
+  
+  for (i=0;i<deg;i++) {
+    if (coefficients[i] != NULL) {
+      free_memory(coefficients[i]);
+    }
+  }
+  safeFree(coefficients);
+  if (resA) {
+    *hasZero = hasZerosA;
+  }
+  
+  return resA;
+}
+
+/* Determine if the polynomial f has a zero on the interval [0;c] */
+static inline int descartesRuleInner(int *hasZero, node *f, mpfr_t c) {
+  int resA, hasZeroA;
+  int n, hasZeroTmp;
+  mpfr_t a, b, h, hh;
+  mp_prec_t p;
+  node *g, *t;
+  int deg;
+
+  resA = descartesRuleBasic(&hasZeroA, f, c);
+  if (resA) {
+    *hasZero = hasZeroA;
+    return 1;
+  }
+
+  deg = getDegreeSilent(f);
+  if (deg < 0) return 0;
+  if (deg > DESCARTES_RULE_MAX_DEGREE) return 0;
+
+  n = deg * 2;
+  p = mpfr_get_prec(c);
+  mpfr_init2(a, p);
+  mpfr_init2(b, p);
+  mpfr_init2(h, p);
+  mpfr_init2(hh, p);
+  mpfr_div_si(h, c, n, GMP_RNDU);
+  mpfr_set_si(a, 0, GMP_RNDN);
+  mpfr_set(b, h, GMP_RNDN);
+  hasZeroTmp = 0;
+  while ((mpfr_cmp(a, c) < 0) && (!hasZeroTmp)) {
+    if (mpfr_cmp(b, c) > 0) {
+      mpfr_set(b, c, GMP_RNDN);
+    }
+    mpfr_sub(hh, b, a, GMP_RNDU);
+    t = addMemRef(makeAdd(makeVariable(), makeConstant(a)));
+    tryRepresentAsPolynomial(t);
+    g = substitute(f, t);
+    free_memory(t);
+    resA = descartesRuleBasic(&hasZeroA, g, hh);
+    free_memory(g);
+    if (!resA) {
+      mpfr_clear(hh);
+      mpfr_clear(h);
+      mpfr_clear(b);
+      mpfr_clear(a);
+      return 0;
+    }
+    hasZeroTmp = hasZeroTmp || hasZeroA;
+    mpfr_set(a, b, GMP_RNDN);
+    mpfr_add(b, b, h, GMP_RNDU);
+  }
+  mpfr_clear(hh);
+  mpfr_clear(h);
+  mpfr_clear(b);
+  mpfr_clear(a);
+
+  *hasZero = hasZeroTmp;
+  return 1;
+}
+
+static inline int descartesRule(int *hasZero, node *f, sollya_mpfi_t dom) {
+  mp_prec_t p;
+  mpfr_t a, b;
+  sollya_mpfi_t domA, domB;
+  int res, resA, resB, hasZeroA, hasZeroB = 0;
+  node *g, *h;
+  int deg;
+  
+  if (sollya_mpfi_has_nan(dom)) return 0;
+  if (sollya_mpfi_has_infinity(dom)) return 0;
+
+  deg = getDegreeSilent(f);
+  if (deg < 0) return 0;
+  if (deg > DESCARTES_RULE_MAX_DEGREE) return 0;
+  
+  if (sollya_mpfi_has_zero_inside(dom)) {
+    p = sollya_mpfi_get_prec(dom);
+    mpfr_init2(a, p);
+    mpfr_init2(b, p);
+    sollya_mpfi_init2(domA, p);
+    sollya_mpfi_init2(domB, p);
+    sollya_mpfi_get_left(a, dom);
+    mpfr_set_si(b, 0, GMP_RNDN);
+    sollya_mpfi_interv_fr(domA, a, b);
+    sollya_mpfi_get_right(b, dom);
+    mpfr_set_si(a, 0, GMP_RNDN);
+    sollya_mpfi_interv_fr(domB, a, b);
+    resB = descartesRule(&hasZeroB, f, domB);
+    if (resB) {
+      resA = descartesRule(&hasZeroA, f, domA);
+    } else {
+      resA = 0;
+    }
+    if (resA && resB) {
+      *hasZero = hasZeroA || hasZeroB;
+      res = 1;
+    } else {
+      res = 0;
+    }
+    sollya_mpfi_clear(domB);
+    sollya_mpfi_clear(domA);
+    mpfr_clear(b);
+    mpfr_clear(a);
+    return res;
+  }
+
+  if (sollya_mpfi_has_negative_numbers(dom)) {
+    p = sollya_mpfi_get_prec(dom);
+    sollya_mpfi_init2(domA, p);
+    sollya_mpfi_neg(domA, dom);
+    h = addMemRef(makeNeg(makeVariable()));
+    tryRepresentAsPolynomial(h);
+    g = substitute(f, h);
+    free_memory(h);
+    resA = descartesRule(&hasZeroA, g, domA);
+    if (resA) {
+      *hasZero = hasZeroA;
+    }
+    free_memory(g);
+    sollya_mpfi_clear(domA);
+    return resA;
+  }
+
+  if (!sollya_mpfi_has_negative_numbers(dom)) {
+    if (sollya_mpfi_has_zero(dom)) {
+      p = sollya_mpfi_get_prec(dom);
+      mpfr_init2(b, p);
+      sollya_mpfi_get_right(b, dom);
+      resA = descartesRuleInner(&hasZeroA, f, b);
+      if (resA) {
+	*hasZero = hasZeroA;
+      }
+      mpfr_clear(b);
+      return resA;
+    } else {
+      p = sollya_mpfi_get_prec(dom);
+      mpfr_init2(a, p);
+      mpfr_init2(b, p);
+      sollya_mpfi_get_left(a, dom);
+      sollya_mpfi_get_right(b, dom);
+      mpfr_sub(b, b, a, GMP_RNDU);
+      h = addMemRef(makeAdd(makeVariable(), makeConstant(a)));
+      tryRepresentAsPolynomial(h);
+      g = substitute(f, h);
+      free_memory(h);
+      resA = descartesRuleInner(&hasZeroA, g, b);
+      if (resA) {
+	*hasZero = hasZeroA;
+      }      
+      free_memory(g);
+      mpfr_clear(b);
+      mpfr_clear(a);
+      return resA;
+    }
+  }
+  
+  return 0;
+}
+
+static inline int hookRoots(mpfr_t res, node *f, sollya_mpfi_t dom, mp_prec_t prec, int lazy) {
+  mpfr_t a, b, h, t, s;
+  sollya_mpfi_t d, y, dd, yy;
+  mp_prec_t p;
+  int okay, found;
+  int deg;
+  int pt;
+  int desc = 0;
+
+  p = sollya_mpfi_get_prec(dom);
+  if (prec > p) p = prec;
+  mpfr_init2(a, p);
+  mpfr_init2(b, p);
+  mpfr_init2(h, p);
+  mpfr_init2(t, p);
+  mpfr_init2(s, p);
+  sollya_mpfi_init2(d, p);
+  sollya_mpfi_init2(dd, p);
+  sollya_mpfi_init2(y, p);
+  sollya_mpfi_init2(yy, p);
+  sollya_mpfi_get_left(a, dom);
+  sollya_mpfi_get_right(b, dom);
+  mpfr_sub(h, b, a, GMP_RNDU);
+  pt = defaultpoints;
+  if (pt < 100) pt = 100;
+  deg = getDegreeSilent(f);
+  if (deg > 0) {
+    if (deg > pt) pt = deg;
+  }
+  mpfr_div_si(h, h, 2 * pt + 17, GMP_RNDU);
+  mpfr_set(t, a, GMP_RNDN);
+  okay = 1;
+  found = 0;
+  while (okay && (mpfr_cmp(t, b) < 0)) {
+    mpfr_add(s, t, h, GMP_RNDU);
+    if (mpfr_cmp(s, b) > 0) {
+      mpfr_set(s, b, GMP_RNDN);
+    }
+    sollya_mpfi_interv_fr(d, t, s);
+    evaluateInterval(y, f, NULL, d);
+    if (sollya_mpfi_has_nan(y) ||
+	sollya_mpfi_has_infinity(y) ||
+	sollya_mpfi_has_zero(y)) {
+      okay = 0;
+      sollya_mpfi_interv_fr(d, t, t);
+      sollya_mpfi_interv_fr(dd, s, s);
+      evaluateInterval(y, f, NULL, d);
+      evaluateInterval(yy, f, NULL, dd);
+      if (sollya_mpfi_is_zero(y) ||
+	  sollya_mpfi_is_zero(yy)) {
+	found = 1;
+      } else {
+	sollya_mpfi_mul(yy, y, yy);
+	if ((!sollya_mpfi_has_nan(yy)) &&
+	    (!sollya_mpfi_has_infinity(yy)) &&
+	    (!sollya_mpfi_has_zero(yy)) &&
+	    (sollya_mpfi_has_negative_numbers(yy))) {
+	  found = 1;
+	}
+      }
+    }
+    mpfr_set(t, s, GMP_RNDN);
+  }
+  if (okay) {
+    mpfr_set_si(res, 0, GMP_RNDN);
+  }
+  if (found) {
+    mpfr_set_si(res, 1, GMP_RNDN);
+  }
+  sollya_mpfi_clear(y);
+  sollya_mpfi_clear(d);
+  sollya_mpfi_clear(yy);
+  sollya_mpfi_clear(dd);
+  mpfr_clear(s);
+  mpfr_clear(t);
+  mpfr_clear(h);
+  mpfr_clear(b);
+  mpfr_clear(a);
+  if (okay) return 1;
+  if (found) return 1;
+
+  if (descartesRule(&desc, f, dom)) {
+    if (desc) {
+      mpfr_set_si(res, 1, GMP_RNDN);
+    } else {
+      mpfr_set_si(res, 0, GMP_RNDN);
+    }
+    return 1;
+  }
+
+  deg = getDegreeSilent(f);
+  if (deg < 0) return 0;
+  if (deg > STURM_MAX_DEGREE) return 0;
+
+  if (lazy) return 0;
+
+  return getNrRoots(res, f, dom, 3 * prec, 1);
+}
 
 int addEvaluationHook(eval_hook_t **hookPtr, 
 		      void *data, 
@@ -152,7 +663,7 @@ void freeEvaluationHook(eval_hook_t **hookPtr) {
   *hookPtr = NULL;
 }
 
-int evaluateWithEvaluationHook(sollya_mpfi_t y, sollya_mpfi_t x, mp_prec_t prec, int tight, eval_hook_t *hook) {
+int evaluateWithEvaluationHookInner(sollya_mpfi_t y, sollya_mpfi_t x, mp_prec_t prec, int tight, eval_hook_t *hook) {
   eval_hook_t *curr;
   int res, resCurr, yOnceSet, inconsistency;
 
@@ -210,6 +721,16 @@ int evaluateWithEvaluationHook(sollya_mpfi_t y, sollya_mpfi_t x, mp_prec_t prec,
   return res;
 }
 
+int evaluateWithEvaluationHook(sollya_mpfi_t y, sollya_mpfi_t x, mp_prec_t prec, int tight, eval_hook_t *hook) {
+  int res;
+  
+  if (hook == NULL) return 0;
+
+  res = evaluateWithEvaluationHookInner(y, x, prec, tight, hook);
+
+  return res;
+}
+
 node_eval_hook_t *createNodeEvalHook(node *func, sollya_mpfi_t dom, sollya_mpfi_t delta, sollya_mpfi_t t) {
   node_eval_hook_t *newNodeEvalHook;
 
@@ -278,10 +799,13 @@ int evaluateNodeEvalHook(sollya_mpfi_t y, sollya_mpfi_t x, mp_prec_t prec, int t
   sollya_mpfi_init2(myYRndWithDelta, pY + 1);
 
   sollya_mpfi_set(myYRnd, myY);
-  sollya_mpfi_blow_1ulp(myYRnd);
+  if (!sollya_mpfi_is_zero(myYRnd)) {
+    sollya_mpfi_blow_1ulp(myYRnd);
+  }
   sollya_mpfi_add(myYRndWithDelta, myY, hook->delta);
 
-  if (sollya_mpfi_is_inside(myYRndWithDelta, myYRnd) && 
+  if ((sollya_mpfi_is_inside(myYRndWithDelta, myYRnd) ||
+       (sollya_mpfi_is_zero(myYRndWithDelta) && sollya_mpfi_is_zero(myYRnd))) && 
       (!(sollya_mpfi_has_nan(myYRndWithDelta) || 
 	 (sollya_mpfi_has_infinity(myYRndWithDelta) && 
 	  (!sollya_mpfi_is_infinity(myYRndWithDelta)))))) okay = 1;
@@ -428,7 +952,7 @@ poly_eval_hook_t *createPolyEvalHook(int degree, mpfr_t *coeffs, sollya_mpfi_t d
       } else {
 	/* We still do not know */
 	mpfr_init2(nbRoots, 8 * sizeof(int) + 5);
-	if (getNrRoots(nbRoots, derivPoly, X, maxPrec, 1)) {
+	if (hookRoots(nbRoots, derivPoly, X, maxPrec, 0)) {
 	  if (mpfr_zero_p(nbRoots)) {
 	    /* We surely know that there is no root of the derivative polynomial in the domain */
 	    newPolyEvalHook->polynomialIsMonotone = 1; 
@@ -472,7 +996,7 @@ poly_eval_hook_t *createPolyEvalHook(int degree, mpfr_t *coeffs, sollya_mpfi_t d
 	    sollya_mpfi_has_infinity(Y) ||
 	    sollya_mpfi_has_zero(Y)) {
 	  mpfr_init2(nbRoots, 8 * sizeof(int) + 5);
-	  if (getNrRoots(nbRoots, poly, X, maxPrec, 1)) {
+	  if (hookRoots(nbRoots, poly, X, maxPrec, 1)) {
 	    if (mpfr_number_p(nbRoots) && (!mpfr_zero_p(nbRoots))) {
 	      newPolyEvalHook->polynomialHasZero = 1; 
 	    }
@@ -763,10 +1287,14 @@ int evaluatePolyEvalHook(sollya_mpfi_t y, sollya_mpfi_t x, mp_prec_t prec, int t
 	  hook->reusedVarMyYRndWithDeltaInit = 1;
 	}
 	sollya_mpfi_set(hook->reusedVarMyYRnd, hook->reusedVarMyY);
-	sollya_mpfi_blow_1ulp(hook->reusedVarMyYRnd);
+	if (!sollya_mpfi_is_zero(hook->reusedVarMyYRnd)) {
+	  sollya_mpfi_blow_1ulp(hook->reusedVarMyYRnd);
+	}
 	sollya_mpfi_add(hook->reusedVarMyYRndWithDelta, hook->reusedVarMyY, hook->delta);
 	
-	if (sollya_mpfi_is_inside(hook->reusedVarMyYRndWithDelta, hook->reusedVarMyYRnd) && 
+	if ((sollya_mpfi_is_inside(hook->reusedVarMyYRndWithDelta, hook->reusedVarMyYRnd) ||
+	     (sollya_mpfi_is_zero(hook->reusedVarMyYRndWithDelta) &&
+	      sollya_mpfi_is_zero(hook->reusedVarMyYRnd))) && 
 	    (!(sollya_mpfi_has_nan(hook->reusedVarMyYRndWithDelta) || 
 	       (sollya_mpfi_has_infinity(hook->reusedVarMyYRndWithDelta) && 
 		(!sollya_mpfi_is_infinity(hook->reusedVarMyYRndWithDelta)))))) okay = 1;
@@ -806,10 +1334,14 @@ int evaluatePolyEvalHook(sollya_mpfi_t y, sollya_mpfi_t x, mp_prec_t prec, int t
 
     sollya_mpfi_set(hook->reusedVarMyYRnd, hook->reusedVarMyY);
 
-    sollya_mpfi_blow_1ulp(hook->reusedVarMyYRnd);
+    if (!sollya_mpfi_is_zero(hook->reusedVarMyYRnd)) {
+      sollya_mpfi_blow_1ulp(hook->reusedVarMyYRnd);
+    }
     sollya_mpfi_add(hook->reusedVarMyYRndWithDelta, hook->reusedVarMyY, hook->delta);
 
-    if (sollya_mpfi_is_inside(hook->reusedVarMyYRndWithDelta, hook->reusedVarMyYRnd) && 
+    if ((sollya_mpfi_is_inside(hook->reusedVarMyYRndWithDelta, hook->reusedVarMyYRnd) ||
+	 (sollya_mpfi_is_zero(hook->reusedVarMyYRndWithDelta) &&
+	  sollya_mpfi_is_zero(hook->reusedVarMyYRnd))) && 
 	(!(sollya_mpfi_has_nan(hook->reusedVarMyYRndWithDelta) || 
 	   (sollya_mpfi_has_infinity(hook->reusedVarMyYRndWithDelta) && 
 	    (!sollya_mpfi_is_infinity(hook->reusedVarMyYRndWithDelta)))))) okay = 1;
@@ -919,7 +1451,9 @@ int addPolyEvaluationHook(eval_hook_t **hookPtr, node *func, sollya_mpfi_t dom,
       coeffsOkay = coeffsOkay && mpfr_number_p(evaluatedCoeffs[i]);
       sollya_mpfi_sub_fr(c, c, evaluatedCoeffs[i]);
     }
-    sollya_mpfi_mul(globalDelta, globalDelta, shiftedDom);
+    if (!sollya_mpfi_is_zero(globalDelta)) {
+      sollya_mpfi_mul(globalDelta, globalDelta, shiftedDom);
+    }
     sollya_mpfi_add(globalDelta, globalDelta, c);
   }
   safeFree(coeffs);
@@ -965,7 +1499,7 @@ int copyFunctionAndChooseAndAddEvaluationHook(node **copyPtr, node *orig, node *
     return 0;
   }
 
-  okay = chooseAndAddEvaluationHook(&(copy->evaluationHook), func, dom, delta, t, prec);
+  okay = chooseAndAddEvaluationHook(&(copy->cache->evaluationHook), func, dom, delta, t, prec);
     
   if (okay) {
     *copyPtr = copy;
