@@ -1,18 +1,27 @@
 /*
 
-  Copyright 2006-2013 by
+  Copyright 2006-2017 by
 
   Laboratoire de l'Informatique du Parallelisme,
-  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668
+  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
+
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France
 
   and by
 
-  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
-  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France.
+  Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
+  Sorbonne Universités
+  UPMC Univ Paris 06
+  UMR 7606, LIP6
+  Boîte Courrier 169
+  4, place Jussieu
+  F-75252 Paris Cedex 05
+  France.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -62,7 +71,7 @@
 #include "expression.h"
 #include "general.h"
 
-#define CODESIZE 4000000
+#define CODESIZE 16000000
 
 
 node *implementpoly(node *func, rangetype range, mpfr_t *accur, int variablePrecision,
