@@ -1,13 +1,14 @@
 /*
 
-  Copyright 2007-2016 by
+  Copyright 2007-2021 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
-  LORIA (CNRS, INPL, INRIA, UHP, U-Nancy 2)
+  LORIA (CNRS, INPL, INRIA, UHP, U-Nancy 2),
 
-  and by
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -18,9 +19,15 @@
   F-75252 Paris Cedex 05
   France
 
+  and by
+
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
+
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -82,16 +89,40 @@
 #include "expression.h"
 #include "general.h"
 #include "library.h"
+
+typedef struct libraryHandleStruct libraryHandle;
+struct libraryHandleStruct
+{
+  char *libraryName;
+  void *libraryDescriptor;
+  chain *functionList;
+  int shallowCopy;
+  int closeFuncSet;
+  int closeFuncExists;
+  int closeFuncAlreadyCalled;
+  int (*closeFunc)(void);  
+};
+
+
 #include "chain.h"
 #include "execute.h"
 
+
 #ifndef RTLD_LOCAL
 #define RTLD_LOCAL 0
 #endif
 
-#define LIBRARYFUNCTION_CASE 0
-#define LIBRARYCONSTANT_CASE 1
-#define LIBRARYPROC_CASE 2
+#ifndef RTLD_GLOBAL
+#define RTLD_GLOBAL 0
+#endif
+
+typedef enum {
+	      LIBRARYFUNCTION_CASE = 0,
+	      LIBRARYCONSTANT_CASE,
+	      LIBRARYPROC_CASE,
+	      LIBRARYDATA_CASE
+} library_case_t;
+
 
 /* List of Sollya keywords we must not bind dynamic objects to.
 
@@ -123,6 +154,7 @@ static const char* sollyaKeywords[] = { "D",
 					"autosimplify",
 					"bashexecute",
 					"begin",
+					"bezout",
 					"binary",
 					"bind",
 					"boolean",
@@ -148,6 +180,7 @@ static const char* sollyaKeywords[] = { "D",
 					"dirtyintegral",
 					"dirtysimplify",
 					"display",
+					"div",
 					"do",
 					"double",
 					"doubledouble",
@@ -164,6 +197,7 @@ static const char* sollyaKeywords[] = { "D",
 					"expand",
 					"expm1",
 					"exponent",
+					"externaldata",
 					"externalplot",
 					"externalproc",
 					"false",
@@ -178,6 +212,7 @@ static const char* sollyaKeywords[] = { "D",
 					"from",
 					"fullparentheses",
 					"function",
+					"gcd",
 					"getsuppressedmessages",
 					"getbacktrace",
 					"guessdegree",
@@ -187,6 +222,7 @@ static const char* sollyaKeywords[] = { "D",
 					"hopitalrecursions",
 					"horner",
 					"if",
+					"interpolate",
 					"implementpoly",
 					"implementconstant",
 					"in",
@@ -209,6 +245,7 @@ static const char* sollyaKeywords[] = { "D",
 					"mid",
 					"midpointmode",
 					"min",
+					"mod",
 					"nearestint",
 					"numberroots",
 					"nop",
@@ -304,10 +341,357 @@ static const char* sollyaKeywords[] = { "D",
 chain *openedFunctionLibraries = NULL;
 chain *openedConstantLibraries = NULL;
 chain *openedProcLibraries = NULL;
+chain *openedDataLibraries = NULL;
 
 chain *globalLibraryFunctions = NULL;
 chain *globalLibraryConstants = NULL;
 chain *globalLibraryProcedures = NULL;
+chain *globalLibraryData = NULL;
+
+chain *librariesInOrder = NULL;
+chain *symbolsInOrder = NULL;
+
+struct __library_symbol_struct_t {
+  library_case_t type;
+  union {
+    libraryFunction  *function;
+    libraryProcedure *procedure;
+    libraryData      *data;
+  } value;
+};
+typedef struct __library_symbol_struct_t library_symbol_t;
+
+
+void freeFunctionLibraries();
+void freeConstantLibraries();
+void freeProcLibraries();
+void freeDataLibraries();
+
+void prepareFreeFunctionLibraries();
+void prepareFreeConstantLibraries();
+void prepareFreeProcLibraries();
+void prepareFreeDataLibraries();
+
+void prepareUndoDlsymInOrder() {
+  chain *libraries[] = { openedFunctionLibraries,
+			 openedConstantLibraries,
+			 openedProcLibraries,
+			 openedDataLibraries };
+  chain *l;
+  size_t i;
+  libraryHandle *lh;
+  int (*fct)(void);
+
+  for (i=0;i<(sizeof(libraries)/sizeof(libraries[0]));i++) {
+    for (l=libraries[i];l!=NULL;l=l->next) {
+      lh = (libraryHandle *) l->value;
+      if (!lh->closeFuncSet) {
+	lh->closeFuncSet = 1;
+	lh->closeFuncAlreadyCalled = 0;
+	dlerror();
+	fct = (int (*)(void)) dlsym(lh->libraryDescriptor, "sollya_external_lib_close");
+	if (dlerror() == NULL) {
+	  lh->closeFuncExists = 1;
+	  lh->closeFunc = fct;
+	} else {
+	  lh->closeFuncExists = 0;
+	}
+      }
+    }
+  }
+}
+
+int finishUndoDlsymInOrderOneHaveGo(libraryHandle *lh) {
+  chain *libraries[] = { openedFunctionLibraries,
+			 openedConstantLibraries,
+			 openedProcLibraries,
+			 openedDataLibraries };
+  chain *l;
+  size_t i;
+  libraryHandle *oh;
+
+  for (i=0;i<(sizeof(libraries)/sizeof(libraries[0]));i++) {
+    for (l=libraries[i];l!=NULL;l=l->next) {
+      oh = (libraryHandle *) l->value;
+      if (lh->libraryDescriptor == oh->libraryDescriptor) {
+	if (lengthChain(oh->functionList) != 0) return 0;
+      }
+    }
+  }
+  return 1;
+}
+
+void finishUndoDlsymInOrderOneRemove(chain **lib, void *hdl) {
+  chain *c, *p;
+  libraryHandle *lh;
+
+  for (c=*lib, p=NULL; c!=NULL; c=(p=c)->next) {
+    lh = (libraryHandle *) c->value;
+    if (lh->libraryDescriptor == hdl) {
+      if (lengthChain(lh->functionList) != 0) return;
+      if (p==NULL) {
+	*lib = c->next;
+      } else {
+	p->next = c->next;
+      }
+      safeFree(lh->libraryName);
+      safeFree(lh);
+      safeFree(c);
+      return;
+    }
+  }
+}
+
+void finishUndoDlsymInOrderOne(libraryHandle *lh) {
+  chain **libraries[] = { &openedFunctionLibraries,
+			  &openedConstantLibraries,
+			  &openedProcLibraries,
+			  &openedDataLibraries };
+  void *hdl;
+  char *libName;
+  int haveFunc;
+  int (*fct)(void);
+  size_t i;
+  int res;
+
+  if (!finishUndoDlsymInOrderOneHaveGo(lh)) return;
+  hdl = lh->libraryDescriptor;
+  libName = (char *) safeCalloc(strlen(lh->libraryName)+1,sizeof(char));
+  strcpy(libName, lh->libraryName);
+  haveFunc = 0;
+  fct = NULL;
+  if (lh->closeFuncSet && lh->closeFuncExists && (!lh->closeFuncAlreadyCalled)) {
+    haveFunc = 1;
+    fct = lh->closeFunc;
+  }
+  lh->closeFuncAlreadyCalled = 1;
+  for (i=0;i<(sizeof(libraries)/sizeof(libraries[0]));i++) {
+    finishUndoDlsymInOrderOneRemove(libraries[i],hdl);
+  }
+  if (haveFunc) {
+    enterExternalCode();
+    res = fct();
+    leaveExternalCode();
+    if (res) {
+      printMessage(1,
+		   SOLLYA_MSG_LIBRARY_CLOSER_ERROR,
+		   "Warning: while closing library \"%s\", the function \"sollya_external_lib_close\" was found and called but it signaled the error %d\n",
+		   libName,
+		   res);
+    }
+  }
+  dlerror();
+  if (dlclose(hdl) != 0) {
+    printMessage(1,
+		 SOLLYA_MSG_COULD_NOT_CLOSE_LIBRARY,
+		 "Warning: could not close library \"%s\": %s\n",
+		 libName,
+		 dlerror());
+  } 
+  safeFree(libName);
+}
+
+void finishUndoDlsymInOrder() {
+  chain *curr, *next;
+
+  curr = librariesInOrder;
+  while (curr != NULL) {
+    finishUndoDlsymInOrderOne((libraryHandle *) curr->value);
+    next = curr->next;
+    safeFree(curr);
+    curr = next;
+  }
+  librariesInOrder = NULL;
+}
+
+void addSymbolInOrder(library_symbol_t symb) {
+  library_symbol_t sym = symb;
+  library_symbol_t *new_sym;
+
+  new_sym = (library_symbol_t *) safeMalloc(sizeof(library_symbol_t));
+  memcpy(new_sym, &sym, sizeof(library_symbol_t));
+  symbolsInOrder = addElement(symbolsInOrder, new_sym);
+}
+
+void undoDlsymInOrderOneFunction(libraryFunction *function) {
+  chain *l, *c, *p;
+  libraryFunction *fct;
+  
+  if (function->dealloc != NULL) {
+    if (function->hasData) {
+      enterExternalCode();
+      ((void (*)(void *)) function->dealloc)(function->data);
+      leaveExternalCode();
+    } else {
+      enterExternalCode();
+      ((void (*)()) function->dealloc)();
+      leaveExternalCode();
+    }
+    function->dealloc = NULL;
+  }
+  for (l=openedFunctionLibraries;l!=NULL;l=l->next) {
+    for (c=((libraryHandle *) (l->value))->functionList, p=NULL;c!=NULL;c=(p=c)->next) {
+      fct = (libraryFunction *) c->value;
+      if (fct == function) {
+	if (p == NULL) {
+	  ((libraryHandle *) (l->value))->functionList = c->next;
+	} else {
+	  p->next = c->next;
+	}
+	safeFree(function->functionName);	
+	safeFree(function);
+	safeFree(c);
+	return;
+      }
+    }
+  }
+}
+
+void undoDlsymInOrderOneConstant(libraryFunction *function) {
+  chain *l, *c, *p;
+  libraryFunction *fct;
+  
+  if (function->dealloc != NULL) {
+    if (function->hasData) {
+      enterExternalCode();
+      ((void (*)(void *)) function->dealloc)(function->data);
+      leaveExternalCode();
+    } else {
+      enterExternalCode();
+      ((void (*)()) function->dealloc)();
+      leaveExternalCode();
+    }
+    function->dealloc = NULL;
+  }
+  for (l=openedConstantLibraries;l!=NULL;l=l->next) {
+    for (c=((libraryHandle *) (l->value))->functionList, p=NULL;c!=NULL;c=(p=c)->next) {
+      fct = (libraryFunction *) c->value;
+      if (fct == function) {
+	if (p == NULL) {
+	  ((libraryHandle *) (l->value))->functionList = c->next;
+	} else {
+	  p->next = c->next;
+	}
+	safeFree(function->functionName);	
+	safeFree(function);
+	safeFree(c);
+	return;
+      }
+    }
+  }
+}
+
+void undoDlsymInOrderOneProcedure(libraryProcedure *procedure) {
+  chain *l, *c, *p;
+  libraryProcedure *prc;
+  
+  if (procedure->dealloc != NULL) {
+    if (procedure->hasData) {
+      enterExternalCode();
+      ((void (*)(void *)) procedure->dealloc)(procedure->data);
+      leaveExternalCode();
+    } else {
+      enterExternalCode();
+      ((void (*)()) procedure->dealloc)();
+      leaveExternalCode();
+    }
+    procedure->dealloc = NULL;
+  }
+  for (l=openedProcLibraries;l!=NULL;l=l->next) {
+    for (c=((libraryHandle *) (l->value))->functionList, p=NULL;c!=NULL;c=(p=c)->next) {
+      prc = (libraryProcedure *) c->value;
+      if (prc == procedure) {
+	if (p == NULL) {
+	  ((libraryHandle *) (l->value))->functionList = c->next;
+	} else {
+	  p->next = c->next;
+	}
+	safeFree(procedure->procedureName);
+	freeChain(procedure->signature,freeIntPtr);
+	safeFree(procedure);
+	safeFree(c);
+	return;
+      }
+    }
+  }
+}
+
+void undoDlsymInOrderOneData(libraryData *data) {
+  chain *l, *c, *p;
+  libraryData *dt;
+  
+  if (data->dealloc != NULL) {
+    enterExternalCode();
+    ((void (*)(void *)) data->dealloc)(data->data);
+    leaveExternalCode();
+    data->dealloc = NULL;
+  }
+  for (l=openedDataLibraries;l!=NULL;l=l->next) {
+    for (c=((libraryHandle *) (l->value))->functionList, p=NULL;c!=NULL;c=(p=c)->next) {
+      dt = (libraryData *) c->value;
+      if (dt == data) {
+	if (p == NULL) {
+	  ((libraryHandle *) (l->value))->functionList = c->next;
+	} else {
+	  p->next = c->next;
+	}
+	safeFree(data->dataName);	
+	safeFree(data);
+	safeFree(c);
+	return;
+      }
+    }
+  }
+}
+
+void undoDlsymInOrderOne(library_symbol_t *sym) {
+  switch (sym->type) {
+  case LIBRARYFUNCTION_CASE:
+    undoDlsymInOrderOneFunction(sym->value.function);
+    break;
+  case LIBRARYCONSTANT_CASE:
+    undoDlsymInOrderOneConstant(sym->value.function);
+    break;
+  case LIBRARYPROC_CASE:
+    undoDlsymInOrderOneProcedure(sym->value.procedure);
+    break;
+  case LIBRARYDATA_CASE:
+    undoDlsymInOrderOneData(sym->value.data);
+    break;
+  }
+}
+
+void undoDlsymInOrder() {
+  chain *c, *n;
+  
+  c = symbolsInOrder;
+  while (c != NULL) {
+    undoDlsymInOrderOne((library_symbol_t *) c->value);
+    safeFree(c->value);
+    n = c->next;
+    safeFree(c);
+    c = n;
+  }
+  symbolsInOrder = NULL;
+}
+
+void prepareFreeLibraries() {
+  prepareFreeFunctionLibraries();
+  prepareFreeConstantLibraries();
+  prepareFreeProcLibraries();
+  prepareFreeDataLibraries();
+  prepareUndoDlsymInOrder();
+  undoDlsymInOrder();
+  finishUndoDlsymInOrder();
+}
+
+void freeLibraries() {
+  prepareFreeLibraries();
+  freeFunctionLibraries();
+  freeConstantLibraries();
+  freeDataLibraries();
+  freeProcLibraries();
+}
 
 void copyIdentifierSymbols(char *ptr, const char *src) {
   char *currPtr;
@@ -459,13 +843,13 @@ char *unifySymbolName(char *basename) {
   return res;
 }
 
-libraryHandle *getLibraryHandle(char *libraryName, int type) {
+libraryHandle *getLibraryHandle(char *libraryName, library_case_t type) {
   chain *curr;
   chain *openedLibraries;
   libraryHandle *currHandle;
   void *dlfcnHandle, *otherDlfcnHandle;
   int shallowCopy, foundElsewhere, i;
-  chain *otherLibraries[2];
+  chain *otherLibraries[3];
   
   openedLibraries = NULL;
 
@@ -473,6 +857,7 @@ libraryHandle *getLibraryHandle(char *libraryName, int type) {
   case LIBRARYFUNCTION_CASE: openedLibraries = openedFunctionLibraries; break;
   case LIBRARYCONSTANT_CASE: openedLibraries = openedConstantLibraries; break;
   case LIBRARYPROC_CASE: openedLibraries = openedProcLibraries; break;
+  case LIBRARYDATA_CASE: openedLibraries = openedDataLibraries; break;
   }
 
   curr = openedLibraries;
@@ -487,21 +872,29 @@ libraryHandle *getLibraryHandle(char *libraryName, int type) {
   case LIBRARYFUNCTION_CASE:
     otherLibraries[0] = openedConstantLibraries;
     otherLibraries[1] = openedProcLibraries;
+    otherLibraries[2] = openedDataLibraries;
     break;
   case LIBRARYCONSTANT_CASE:
     otherLibraries[0] = openedFunctionLibraries;
     otherLibraries[1] = openedProcLibraries;
+    otherLibraries[2] = openedDataLibraries;
     break;
   case LIBRARYPROC_CASE:
     otherLibraries[0] = openedConstantLibraries;
     otherLibraries[1] = openedFunctionLibraries;
+    otherLibraries[2] = openedDataLibraries;
+    break;
+  case LIBRARYDATA_CASE:
+    otherLibraries[0] = openedConstantLibraries;
+    otherLibraries[1] = openedFunctionLibraries;
+    otherLibraries[2] = openedProcLibraries;
     break;
   default:
     sollyaFprintf(stderr,"Error: getLibraryHandle: unknown type (%d)\n", type);
     exit(1);
   }
   otherDlfcnHandle = NULL;
-  for (i=0,foundElsewhere=0;(i<2)&&(!foundElsewhere);i++) {
+  for (i=0,foundElsewhere=0;(i<3)&&(!foundElsewhere);i++) {
     curr = otherLibraries[i];
     while (curr != NULL) {
       currHandle = (libraryHandle *) curr->value;
@@ -516,7 +909,11 @@ libraryHandle *getLibraryHandle(char *libraryName, int type) {
 
   if (!foundElsewhere) {
     dlerror();
-    dlfcnHandle = dlopen(libraryName, RTLD_LOCAL | RTLD_NOW);
+    dlfcnHandle = dlopen(libraryName, RTLD_GLOBAL | RTLD_NOW);
+    if (dlfcnHandle == NULL) {
+      dlerror();
+      dlfcnHandle = dlopen(libraryName, RTLD_LOCAL | RTLD_NOW);
+    }
     if (dlfcnHandle == NULL)
       return NULL;
     shallowCopy = 0;
@@ -531,13 +928,21 @@ libraryHandle *getLibraryHandle(char *libraryName, int type) {
   currHandle->libraryDescriptor = dlfcnHandle;
   currHandle->functionList = NULL;
   currHandle->shallowCopy = shallowCopy;
+  currHandle->closeFuncSet = 0;
+  currHandle->closeFuncExists = 0;
+  currHandle->closeFunc = NULL;
+  currHandle->closeFuncAlreadyCalled = 0;
 
   openedLibraries = addElement(openedLibraries,currHandle);
+  if (!shallowCopy) {
+    librariesInOrder = addElement(librariesInOrder,currHandle);
+  }
 
   switch(type) {
   case LIBRARYFUNCTION_CASE: openedFunctionLibraries = openedLibraries; break;
   case LIBRARYCONSTANT_CASE: openedConstantLibraries = openedLibraries; break;
   case LIBRARYPROC_CASE: openedProcLibraries = openedLibraries; break;
+  case LIBRARYDATA_CASE: openedDataLibraries = openedLibraries; break;
   }
 
   return currHandle;
@@ -555,6 +960,9 @@ libraryHandle *getProcLibraryHandle(char *libraryName) {
   return getLibraryHandle(libraryName, LIBRARYPROC_CASE);
 }
 
+libraryHandle *getDataLibraryHandle(char *libraryName) {
+  return getLibraryHandle(libraryName, LIBRARYDATA_CASE);
+}
 
 /* Functions related to library functions */
 libraryFunction *bindFunction(char* libraryName, char *functionName) {
@@ -563,6 +971,7 @@ libraryFunction *bindFunction(char* libraryName, char *functionName) {
   int (*myFunction)(sollya_mpfi_t, sollya_mpfi_t, int);
   char *error;
   sollya_mpfi_t op, rop;
+  library_symbol_t sym;
 
   currFunct = getFunction(functionName);
   if (currFunct != NULL) {
@@ -587,7 +996,9 @@ libraryFunction *bindFunction(char* libraryName, char *functionName) {
   sollya_mpfi_init2(op,12);
   sollya_mpfi_set_d(op,1.0);
 
+  enterExternalCode();
   myFunction(rop,op,0);
+  leaveExternalCode();
 
   sollya_mpfi_clear(rop);
   sollya_mpfi_clear(op);
@@ -601,6 +1012,9 @@ libraryFunction *bindFunction(char* libraryName, char *functionName) {
   currFunct->dealloc = NULL;
 
   libHandle->functionList = addElement(libHandle->functionList,currFunct);
+  sym.type = LIBRARYFUNCTION_CASE;
+  sym.value.function = currFunct; 
+  addSymbolInOrder(sym);
 
   return currFunct;
 }
@@ -723,21 +1137,22 @@ libraryFunction *bindFunctionByPtrWithData(char *suggestedName, int (*func)(mpfi
   return __bindFunctionByPtrImpl(suggestedName, ((void *) func), 1, data, ((void *) dealloc));
 }
 
-void freeFunctionLibraries() {
-  chain *currLibList, *currFunList, *prevFunList, *prevLibList;
+void prepareFreeFunctionLibraries() {
+  chain *currFunList, *prevFunList;
   libraryFunction *currFunct;
-  libraryHandle *currLibHandle;
-  int (*myFunction)(void);
-  int res;
-
+  
   currFunList = globalLibraryFunctions;
   while (currFunList != NULL) {
     currFunct = (libraryFunction *) currFunList->value;
     if (currFunct->dealloc != NULL) {
       if (currFunct->hasData) {
+	enterExternalCode();
 	((void (*)(void *)) currFunct->dealloc)(currFunct->data);
+	leaveExternalCode();
       } else {
+	enterExternalCode();
 	((void (*)()) currFunct->dealloc)();
+	leaveExternalCode();
       }
     }
     safeFree(currFunct->functionName);
@@ -746,15 +1161,61 @@ void freeFunctionLibraries() {
     currFunList = currFunList->next;
     safeFree(prevFunList);
   }
+  globalLibraryFunctions = NULL;
+}
 
+void freeFunctionLibraries() {
+  chain *currLibList, *currFunList, *prevFunList, *prevLibList;
+  libraryFunction *currFunct;
+  libraryHandle *currLibHandle;
+  int (*myFunction)(void);
+  int res;
+  int haveFunc;
+
+  currFunList = globalLibraryFunctions;
+  while (currFunList != NULL) {
+    currFunct = (libraryFunction *) currFunList->value;
+    if (currFunct->dealloc != NULL) {
+      if (currFunct->hasData) {
+	enterExternalCode();
+	((void (*)(void *)) currFunct->dealloc)(currFunct->data);
+	leaveExternalCode();
+      } else {
+	enterExternalCode();
+	((void (*)()) currFunct->dealloc)();
+	leaveExternalCode();
+      }
+    }
+    safeFree(currFunct->functionName);
+    safeFree(currFunList->value);
+    prevFunList = currFunList;
+    currFunList = currFunList->next;
+    safeFree(prevFunList);
+  }
+  globalLibraryFunctions = NULL;
+  
   currLibList = openedFunctionLibraries;
   while (currLibList != NULL) {
     currLibHandle = (libraryHandle *) currLibList->value;
     if (!(currLibHandle->shallowCopy)) {
-      dlerror();
-      myFunction = (int (*)(void)) dlsym(currLibHandle->libraryDescriptor, "sollya_external_lib_close");
-      if (dlerror() == NULL) {
+      haveFunc = 0;
+      myFunction = NULL;
+      if (!currLibHandle->closeFuncSet) {
+	dlerror();
+	myFunction = (int (*)(void)) dlsym(currLibHandle->libraryDescriptor, "sollya_external_lib_close");
+	if (dlerror() == NULL) {
+	  haveFunc = 1;
+	}
+      } else {
+	if (currLibHandle->closeFuncExists && (!currLibHandle->closeFuncAlreadyCalled)) {
+	  haveFunc = 1;
+	  myFunction = currLibHandle->closeFunc;
+	}
+      }
+      if (haveFunc) {
+	enterExternalCode();
 	res = myFunction();
+	leaveExternalCode();
 	if (res) {
 	  printMessage(1,SOLLYA_MSG_LIBRARY_CLOSER_ERROR,"Warning: while closing library \"%s\", the function \"sollya_external_lib_close\" was found and called but it signaled the error %d\n",currLibHandle->libraryName,res);
 	}
@@ -765,9 +1226,13 @@ void freeFunctionLibraries() {
       currFunct = (libraryFunction *) currFunList->value;
       if (currFunct->dealloc != NULL) {
 	if (currFunct->hasData) {
+	  enterExternalCode();
 	  ((void (*)(void *)) currFunct->dealloc)(currFunct->data);
+	  leaveExternalCode();
 	} else {
+	  enterExternalCode();
 	  ((void (*)()) currFunct->dealloc)();
+	  leaveExternalCode();
 	}
       }
       safeFree(currFunct->functionName);
@@ -799,6 +1264,7 @@ libraryFunction *bindConstantFunction(char* libraryName, char *functionName) {
   void (*myFunction)(mpfr_t, mp_prec_t);
   char *error;
   mpfr_t op;
+  library_symbol_t sym;
 
   currFunct = getConstantFunction(functionName);
   if (currFunct != NULL) {
@@ -820,7 +1286,10 @@ libraryFunction *bindConstantFunction(char* libraryName, char *functionName) {
   }
 
   mpfr_init2(op,20);
+
+  enterExternalCode();
   myFunction(op,5);
+  leaveExternalCode();
 
   mpfr_clear(op);
 
@@ -833,7 +1302,10 @@ libraryFunction *bindConstantFunction(char* libraryName, char *functionName) {
   currFunct->dealloc = NULL;
 
   libHandle->functionList = addElement(libHandle->functionList,currFunct);
-
+  sym.type = LIBRARYCONSTANT_CASE;
+  sym.value.function = currFunct; 
+  addSymbolInOrder(sym);
+  
   return currFunct;
 }
 
@@ -956,21 +1428,22 @@ libraryFunction *bindConstantFunctionByPtrWithData(char *suggestedName, void (*f
   return __bindConstantFunctionByPtrImpl(suggestedName, ((void *) func), 1, data, ((void *) dealloc));
 }
 
-void freeConstantLibraries() {
-  chain *currLibList, *currFunList, *prevFunList, *prevLibList;
+void prepareFreeConstantLibraries() {
+  chain *currFunList, *prevFunList;
   libraryFunction *currFunct;
-  libraryHandle *currLibHandle;
-  int res;
-  int (*myFunction)(void);
-
+  
   currFunList = globalLibraryConstants;
   while (currFunList != NULL) {
     currFunct = (libraryFunction *) currFunList->value;
     if (currFunct->dealloc != NULL) {
       if (currFunct->hasData) {
+	enterExternalCode();
 	((void (*)(void *)) currFunct->dealloc)(currFunct->data);
+	leaveExternalCode();
       } else {
+	enterExternalCode();
 	((void (*)()) currFunct->dealloc)();
+	leaveExternalCode();
       }
     }
     safeFree(currFunct->functionName);
@@ -979,15 +1452,61 @@ void freeConstantLibraries() {
     currFunList = currFunList->next;
     safeFree(prevFunList);
   }
+  globalLibraryConstants = NULL;
+}
+
+void freeConstantLibraries() {
+  chain *currLibList, *currFunList, *prevFunList, *prevLibList;
+  libraryFunction *currFunct;
+  libraryHandle *currLibHandle;
+  int res;
+  int (*myFunction)(void);
+  int haveFunc;
+
+  currFunList = globalLibraryConstants;
+  while (currFunList != NULL) {
+    currFunct = (libraryFunction *) currFunList->value;
+    if (currFunct->dealloc != NULL) {
+      if (currFunct->hasData) {
+	enterExternalCode();
+	((void (*)(void *)) currFunct->dealloc)(currFunct->data);
+	leaveExternalCode();
+      } else {
+	enterExternalCode();
+	((void (*)()) currFunct->dealloc)();
+	leaveExternalCode();
+      }
+    }
+    safeFree(currFunct->functionName);
+    safeFree(currFunList->value);
+    prevFunList = currFunList;
+    currFunList = currFunList->next;
+    safeFree(prevFunList);
+  }
+  globalLibraryConstants = NULL;
 
   currLibList = openedConstantLibraries;
   while (currLibList != NULL) {
     currLibHandle = (libraryHandle *) currLibList->value;
     if (!(currLibHandle->shallowCopy)) {
-      dlerror();
-      myFunction = (int (*)(void)) dlsym(currLibHandle->libraryDescriptor, "sollya_external_lib_close");
-      if (dlerror() == NULL) {
+      haveFunc = 0;
+      myFunction = NULL;
+      if (!currLibHandle->closeFuncSet) {
+	dlerror();
+	myFunction = (int (*)(void)) dlsym(currLibHandle->libraryDescriptor, "sollya_external_lib_close");
+	if (dlerror() == NULL) {
+	  haveFunc = 1;
+	}
+      } else {
+	if (currLibHandle->closeFuncExists && (!currLibHandle->closeFuncAlreadyCalled)) {
+	  haveFunc = 1;
+	  myFunction = currLibHandle->closeFunc;
+	}
+      }
+      if (haveFunc) {
+	enterExternalCode();
 	res = myFunction();
+	leaveExternalCode();
 	if (res) {
 	  printMessage(1,SOLLYA_MSG_LIBRARY_CLOSER_ERROR,"Warning: while closing library \"%s\", the function \"sollya_external_lib_close\" was found and called but it signaled the error %d\n",currLibHandle->libraryName,res);
 	}
@@ -998,9 +1517,13 @@ void freeConstantLibraries() {
       currFunct = (libraryFunction *) currFunList->value;
       if (currFunct->dealloc != NULL) {
 	if (currFunct->hasData) {
+	  enterExternalCode();
 	  ((void (*)(void *)) currFunct->dealloc)(currFunct->data);
+	  leaveExternalCode();
 	} else {
+	  enterExternalCode();
 	  ((void (*)()) currFunct->dealloc)();
+	  leaveExternalCode();
 	}
       }
       safeFree(currFunct->functionName);
@@ -1029,6 +1552,7 @@ libraryProcedure *bindProcedure(char* libraryName, char *procedureName, chain *s
   libraryProcedure *currProc;
   char *error;
   void *myFunction;
+  library_symbol_t sym;
 
   currProc = getProcedure(procedureName);
   if (currProc != NULL) {
@@ -1060,7 +1584,11 @@ libraryProcedure *bindProcedure(char* libraryName, char *procedureName, chain *s
 
 
   libHandle->functionList = addElement(libHandle->functionList,currProc);
+  sym.type = LIBRARYPROC_CASE;
+  sym.value.procedure = currProc; 
+  addSymbolInOrder(sym);
 
+  
   return currProc;
 }
 
@@ -1213,21 +1741,22 @@ libraryProcedure *bindProcedureByPtrWithData(int resType, int *argTypes, int ari
   return __bindProcedureByPtrImpl(resType, argTypes, arity, suggestedName, func, 1, data, ((void *) dealloc));
 }
 
-void freeProcLibraries() {
-  chain *currLibList, *currProcList, *prevProcList, *prevLibList;
+void prepareFreeProcLibraries() {
+  chain *currProcList, *prevProcList;
   libraryProcedure *currProc;
-  libraryHandle *currLibHandle;
-  int res;
-  int (*myFunction)(void);
-
+  
   currProcList = globalLibraryProcedures;
   while (currProcList != NULL) {
     currProc = (libraryProcedure *) currProcList->value;
     if (currProc->dealloc != NULL) {
       if (currProc->hasData) {
+	enterExternalCode();
 	((void (*)(void *)) currProc->dealloc)(currProc->data);
+	leaveExternalCode();
       } else {
+	enterExternalCode();
 	((void (*)()) currProc->dealloc)();
+	leaveExternalCode();
       }
     }
     safeFree(currProc->procedureName);
@@ -1237,15 +1766,62 @@ void freeProcLibraries() {
     currProcList = currProcList->next;
     safeFree(prevProcList);
   }
+  globalLibraryProcedures = NULL;
+}
+
+void freeProcLibraries() {
+  chain *currLibList, *currProcList, *prevProcList, *prevLibList;
+  libraryProcedure *currProc;
+  libraryHandle *currLibHandle;
+  int res;
+  int (*myFunction)(void);
+  int haveFunc;
+
+  currProcList = globalLibraryProcedures;
+  while (currProcList != NULL) {
+    currProc = (libraryProcedure *) currProcList->value;
+    if (currProc->dealloc != NULL) {
+      if (currProc->hasData) {
+	enterExternalCode();
+	((void (*)(void *)) currProc->dealloc)(currProc->data);
+	leaveExternalCode();
+      } else {
+	enterExternalCode();
+	((void (*)()) currProc->dealloc)();
+	leaveExternalCode();
+      }
+    }
+    safeFree(currProc->procedureName);
+    freeChain(currProc->signature,freeIntPtr);
+    safeFree(currProcList->value);
+    prevProcList = currProcList;
+    currProcList = currProcList->next;
+    safeFree(prevProcList);
+  }
+  globalLibraryProcedures = NULL;
   
   currLibList = openedProcLibraries;
   while (currLibList != NULL) {
     currLibHandle = (libraryHandle *) currLibList->value;
     if (!(currLibHandle->shallowCopy)) {
-      dlerror();
-      myFunction = (int (*)(void)) dlsym(currLibHandle->libraryDescriptor, "sollya_external_lib_close");
-      if (dlerror() == NULL) {
+      haveFunc = 0;
+      myFunction = NULL;
+      if (!currLibHandle->closeFuncSet) {
+	dlerror();
+	myFunction = (int (*)(void)) dlsym(currLibHandle->libraryDescriptor, "sollya_external_lib_close");
+	if (dlerror() == NULL) {
+	  haveFunc = 1;
+	}
+      } else {
+	if (currLibHandle->closeFuncExists && (!currLibHandle->closeFuncAlreadyCalled)) {
+	  haveFunc = 1;
+	  myFunction = currLibHandle->closeFunc;
+	}
+      }
+      if (haveFunc) {
+	enterExternalCode();
 	res = myFunction();
+	leaveExternalCode();
 	if (res) {
 	  printMessage(1,SOLLYA_MSG_LIBRARY_CLOSER_ERROR,"Warning: while closing library \"%s\", the function \"sollya_external_lib_close\" was found and called but it signaled the error %d\n",currLibHandle->libraryName,res);
 	}
@@ -1282,3 +1858,247 @@ void freeProcLibraries() {
   openedProcLibraries = NULL;
 }
 
+/* Functions related to external data */
+libraryData *bindData(char* libraryName, char *symbolName) {
+  libraryHandle *libHandle;
+  libraryData *currData;
+  char *error;
+  void *myFunction;
+  library_symbol_t sym;
+
+  currData = getData(symbolName);
+  if (currData != NULL) {
+    printMessage(1,SOLLYA_MSG_EXTERNAL_FUNC_OR_PROC_ALREADY_BOUND,"Warning: a data symbol named \"%s\" has already been bound.\n",symbolName);
+    return currData;
+  }
+
+  libHandle = getDataLibraryHandle(libraryName);
+  if (libHandle == NULL) {
+    printMessage(1,SOLLYA_MSG_COULD_NOT_OPEN_LIBRARY_WITH_EXTERN_FUNC_OR_PROC,"Error: could not open library \"%s\" for binding \"%s\": %s\n",libraryName,symbolName,dlerror());
+    return NULL;
+  }
+
+  dlerror();
+  myFunction = dlsym(libHandle->libraryDescriptor, symbolName);
+  if ((error = dlerror()) != NULL) {
+    printMessage(1, SOLLYA_MSG_EXTERNAL_FUNC_OR_PROC_NOT_FOUND_IN_LIBRARY, "Error: could not find symbol \"%s\" in library \"%s\" for binding: %s\n",symbolName,libraryName,error);
+    return NULL;
+  }
+
+  currData = (libraryData *) safeMalloc(sizeof(libraryData));
+  currData->dataName = (char *) safeCalloc(strlen(symbolName)+1,sizeof(char));
+  strcpy(currData->dataName,symbolName);
+  currData->data = myFunction;
+  currData->dealloc = NULL;
+
+
+  libHandle->functionList = addElement(libHandle->functionList,currData);
+  sym.type = LIBRARYDATA_CASE;
+  sym.value.data = currData; 
+  addSymbolInOrder(sym);
+  
+  return currData;
+}
+
+libraryData *getData(char *symbolName) {
+  chain *currLibList, *currDataList;
+  libraryData *currData;
+  libraryHandle *currLibHandle;
+
+  for (currDataList=globalLibraryData;currDataList!=NULL;currDataList=currDataList->next) {
+    currData = (libraryData *) currDataList->value;
+    if (strcmp(currData->dataName,symbolName) == 0)
+      return currData;
+  }
+
+  currLibList = openedDataLibraries;
+  while (currLibList != NULL) {
+    currLibHandle = (libraryHandle *) currLibList->value;
+    currDataList = currLibHandle->functionList;
+    while (currDataList != NULL) {
+      currData = (libraryData *) currDataList->value;
+      if (strcmp(currData->dataName,symbolName) == 0)
+	return currData;
+      currDataList = currDataList->next;
+    }
+    currLibList = currLibList->next;
+  }
+
+  return NULL;
+}
+
+libraryData *getDataByPtr(void *ptr) {
+  chain *currLibList, *currDataList;
+  libraryData *currData;
+  libraryHandle *currLibHandle;
+
+  for (currDataList=globalLibraryData;currDataList!=NULL;currDataList=currDataList->next) {
+    currData = (libraryData *) currDataList->value;
+    if (currData->data == ptr)
+      return currData;
+  }
+
+  currLibList = openedDataLibraries;
+  while (currLibList != NULL) {
+    currLibHandle = (libraryHandle *) currLibList->value;
+    currDataList = currLibHandle->functionList;
+    while (currDataList != NULL) {
+      currData = (libraryData *) currDataList->value;
+      if (currData->data == ptr)
+	return currData;
+      currDataList = currDataList->next;
+    }
+    currLibList = currLibList->next;
+  }
+
+  return NULL;
+}
+
+libraryData *bindDataByPtr(char *suggestedName, void *data, void (*dealloc)(void *)) {
+  libraryData *res;
+  char *unifiedName, *basename, *filteredSuggestedName, *filteredBaseName;
+
+  res = getDataByPtr(data);
+  if (res != NULL) {
+    if (res->dealloc != dealloc) {
+      res->dealloc = dealloc;
+    }
+    return res;
+  }
+    
+  if (suggestedName != NULL) {
+    filteredSuggestedName = filterSymbolName(suggestedName);
+    if (filteredSuggestedName[0] == '\0') {
+      basename = getBaseFunctionName(data, "data", 0, NULL);
+      filteredBaseName = filterSymbolName(basename);
+      safeFree(basename);
+      if (filteredBaseName[0] == '\0') {
+	unifiedName = unifySymbolName("data");
+      } else {
+	unifiedName = unifySymbolName(filteredBaseName);
+      }
+      safeFree(filteredBaseName);
+    } else {
+      unifiedName = unifySymbolName(filteredSuggestedName);
+    }
+    safeFree(filteredSuggestedName);
+  } else {
+    basename = getBaseFunctionName(data, "data", 0, NULL);
+    filteredBaseName = filterSymbolName(basename);
+    safeFree(basename);
+    if (filteredBaseName[0] == '\0') {
+      unifiedName = unifySymbolName("data");
+    } else {
+      unifiedName = unifySymbolName(filteredBaseName);
+    }
+    safeFree(filteredBaseName);
+  }
+
+  res = (libraryData *) safeMalloc(sizeof(libraryData));
+  res->dataName = unifiedName;
+  res->data = data;
+  res->dealloc = dealloc;
+
+  globalLibraryData = addElement(globalLibraryData, res);
+
+  return res;
+}
+
+void prepareFreeDataLibraries() {
+  chain *currDataList, *prevDataList;
+  libraryData *currData;
+  
+  currDataList = globalLibraryData;
+  while (currDataList != NULL) {
+    currData = (libraryData *) currDataList->value;
+    if (currData->dealloc != NULL) {
+      enterExternalCode();
+      ((void (*)(void *)) currData->dealloc)(currData->data);
+      leaveExternalCode();
+    }
+    safeFree(currData->dataName);
+    safeFree(currDataList->value);
+    prevDataList = currDataList;
+    currDataList = currDataList->next;
+    safeFree(prevDataList);
+  }
+  globalLibraryData = NULL;
+}
+
+void freeDataLibraries() {
+  chain *currLibList, *currDataList, *prevDataList, *prevLibList;
+  libraryData *currData;
+  libraryHandle *currLibHandle;
+  int res;
+  int (*myFunction)(void);
+  int haveFunc;
+
+  currDataList = globalLibraryData;
+  while (currDataList != NULL) {
+    currData = (libraryData *) currDataList->value;
+    if (currData->dealloc != NULL) {
+      enterExternalCode();
+      ((void (*)(void *)) currData->dealloc)(currData->data);
+      leaveExternalCode();
+    }
+    safeFree(currData->dataName);
+    safeFree(currDataList->value);
+    prevDataList = currDataList;
+    currDataList = currDataList->next;
+    safeFree(prevDataList);
+  }
+  globalLibraryData = NULL;
+  
+  currLibList = openedDataLibraries;
+  while (currLibList != NULL) {
+    currLibHandle = (libraryHandle *) currLibList->value;
+    if (!(currLibHandle->shallowCopy)) {
+      haveFunc = 0;
+      myFunction = NULL;
+      if (!currLibHandle->closeFuncSet) {
+	dlerror();
+	myFunction = (int (*)(void)) dlsym(currLibHandle->libraryDescriptor, "sollya_external_lib_close");
+	if (dlerror() == NULL) {
+	  haveFunc = 1;
+	}
+      } else {
+	if (currLibHandle->closeFuncExists && (!currLibHandle->closeFuncAlreadyCalled)) {
+	  haveFunc = 1;
+	  myFunction = currLibHandle->closeFunc;
+	}
+      }
+      if (haveFunc) {
+	enterExternalCode();
+	res = myFunction();
+	leaveExternalCode();
+	if (res) {
+	  printMessage(1,SOLLYA_MSG_LIBRARY_CLOSER_ERROR,"Warning: while closing library \"%s\", the function \"sollya_external_lib_close\" was found and called but it signaled the error %d\n",currLibHandle->libraryName,res);
+	}
+      }
+    }
+    currDataList = currLibHandle->functionList;
+    while (currDataList != NULL) {
+      currData = (libraryData *) currDataList->value;
+      if (currData->dealloc != NULL) {
+	((void (*)(void *)) currData->dealloc)(currData->data);
+      }
+      safeFree(currData->dataName);
+      safeFree(currDataList->value);
+      prevDataList = currDataList;
+      currDataList = currDataList->next;
+      safeFree(prevDataList);
+    }
+    if (!(currLibHandle->shallowCopy)) {
+      dlerror();
+      if (dlclose(currLibHandle->libraryDescriptor) != 0)
+	printMessage(1,SOLLYA_MSG_COULD_NOT_CLOSE_LIBRARY,"Warning: could not close library \"%s\": %s\n",currLibHandle->libraryName,dlerror());
+    }
+    safeFree(currLibHandle->libraryName);
+    safeFree(currLibHandle);
+    prevLibList = currLibList;
+    currLibList = currLibList->next;
+    safeFree(prevLibList);
+  }
+  openedDataLibraries = NULL;
+}
+
