@@ -1,17 +1,27 @@
 /*
 
-  Copyright 2011-2014 by
+  Copyright 2011-2022 by
 
   Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
-  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France.
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
-  Contributors Ch. Lauter
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
+  Sophia Antipolis, France
 
-  christoph.lauter@ens-lyon.org
+  and by
+
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
+
+  Contributors Ch. Lauter, S. Chevillard
+
+  christoph.lauter@christoph-lauter.org
+  sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
   environment for safe floating-point code development. It is
-  particularily targeted to the automatized implementation of
+  particularly targeted to the automated implementation of
   mathematical floating-point libraries (libm). Amongst other features,
   it offers a certified infinity norm, an automatic polynomial
   implementer and a fast Remez algorithm.
@@ -55,7 +65,7 @@
 #include <stdlib.h>
 
 /* A small wrapper around evaluateThing to be sure that the returned
-   object has no memory references 
+   object has no memory references
 */
 node *evaluateThingDeepCopy(node *thing) {
   node *tempNode, *res;
@@ -168,52 +178,26 @@ node *headFunction(node *tree) {
   case LIBRARYCONSTANT:
     copy = NULL;
     break;
-  case SQRT:
-  case EXP:
-  case LOG:
-  case LOG_2:
-  case LOG_10:
-  case SIN:
-  case COS:
-  case TAN:
-  case ASIN:
-  case ACOS:
-  case ATAN:
-  case SINH:
-  case COSH:
-  case TANH:
-  case ASINH:
-  case ACOSH:
-  case ATANH:
   case NEG:
-  case ABS:
-  case DOUBLE:
-  case SINGLE:
-  case QUAD:
-  case HALFPRECISION:
-  case DOUBLEDOUBLE:
-  case TRIPLEDOUBLE:
-  case ERF:
-  case ERFC:
-  case LOG_1P:
-  case EXP_M1:
-  case DOUBLEEXTENDED:
-  case CEIL:
-  case FLOOR:
-  case NEARESTINT:
-    copy = (node *) safeMalloc(sizeof(node));
+    copy = allocateNode();
     copy->nodeType = tree->nodeType;
     copy->child1 = makeVariable();
     break;
+  case UNARY_BASE_FUNC:
+    copy = allocateNode();
+    copy->nodeType = UNARY_BASE_FUNC;
+    copy->baseFun = tree->baseFun;
+    copy->child1 = makeVariable();
+    break;
   case LIBRARYFUNCTION:
-    copy = (node*) safeMalloc(sizeof(node));
+    copy = allocateNode();
     copy->nodeType = LIBRARYFUNCTION;
     copy->libFun = tree->libFun;
     copy->libFunDeriv = tree->libFunDeriv;
     copy->child1 = makeVariable();
     break;
   case PROCEDUREFUNCTION:
-    copy = (node*) safeMalloc(sizeof(node));
+    copy = allocateNode();
     copy->nodeType = PROCEDUREFUNCTION;
     copy->libFunDeriv = tree->libFunDeriv;
     copy->child1 = makeVariable();
@@ -260,14 +244,14 @@ int tryMatchExtendedPureTree(chain **associations, node *thingToMatch, node *pos
     /* Check if recursion is possible on the thing to match */
     switch (accessThruMemRef(thingToMatch)->nodeType) {
     case VARIABLE:
-    case CONSTANT:
-    case PI_CONST:
-    case LIBRARYCONSTANT:
-    case ADD:
-    case SUB:
-    case MUL:
-    case DIV:
-    case POW:
+      case CONSTANT:
+      case PI_CONST:
+      case LIBRARYCONSTANT:
+      case ADD:
+      case SUB:
+      case MUL:
+      case DIV:
+      case POW:
       return 0;
       break;
     default:
@@ -342,14 +326,14 @@ int tryMatchExtendedPureTree(chain **associations, node *thingToMatch, node *pos
     /* Check if recursion is possible on the thing to match */
     switch (accessThruMemRef(thingToMatch)->nodeType) {
     case VARIABLE:
-    case CONSTANT:
-    case PI_CONST:
-    case LIBRARYCONSTANT:
-    case ADD:
-    case SUB:
-    case MUL:
-    case DIV:
-    case POW:
+      case CONSTANT:
+      case PI_CONST:
+      case LIBRARYCONSTANT:
+      case ADD:
+      case SUB:
+      case MUL:
+      case DIV:
+      case POW:
       return 0;
       break;
     default:
@@ -504,42 +488,12 @@ int tryMatchExtendedPureTree(chain **associations, node *thingToMatch, node *pos
     if (rightAssoc != NULL) freeChain(rightAssoc, freeEntryOnVoid);
     return okay;
     break;
-  case SQRT:
-  case EXP:
-  case LOG:
-  case LOG_2:
-  case LOG_10:
-  case SIN:
-  case COS:
-  case TAN:
-  case ASIN:
-  case ACOS:
-  case ATAN:
-  case SINH:
-  case COSH:
-  case TANH:
-  case ASINH:
-  case ACOSH:
-  case ATANH:
   case NEG:
-  case ABS:
-  case DOUBLE:
-  case SINGLE:
-  case QUAD:
-  case HALFPRECISION:
-  case DOUBLEDOUBLE:
-  case TRIPLEDOUBLE:
-  case ERF:
-  case ERFC:
-  case LOG_1P:
-  case EXP_M1:
-  case DOUBLEEXTENDED:
-  case CEIL:
-  case FLOOR:
-  case NEARESTINT:
+  case UNARY_BASE_FUNC:
     /* Unary functions: we have a match if we have a match on recursion
        Possible associations are established on recursion.
     */
+    if (accessThruMemRef(possibleMatcher)->baseFun != accessThruMemRef(thingToMatch)->baseFun) return 0;
     return tryMatchExtendedPureTree(associations, accessThruMemRef(thingToMatch)->child1, accessThruMemRef(possibleMatcher)->child1);
     break;
   case PI_CONST:
@@ -1399,7 +1353,7 @@ int tryCutPrefixList(chain **associations, node **restList, node *mainList, node
   case LIST:
     switch (accessThruMemRef(mainList)->nodeType) {
     case LIST:
-    case FINALELLIPTICLIST:
+      case FINALELLIPTICLIST:
       lenPrefix = lengthChain(accessThruMemRef(prefix)->arguments);
       possibleMatchingPrefix = makeEmptyList();
       myRestList = copyThing(mainList);
@@ -1575,6 +1529,15 @@ int tryMatchInner(chain **associations, node *thingToMatch, node *possibleMatche
   /* Default case: a match for everything, no association to perform */
   if (accessThruMemRef(possibleMatcher)->nodeType == DEFAULT) return 1;
 
+  /* Special case: error is not considered correctly typed nor equal
+     to itself but does match itself.
+
+     We do not have to perform any association, though.
+
+  */
+  if ((accessThruMemRef(possibleMatcher)->nodeType == ERRORSPECIAL) &&
+      (accessThruMemRef(thingToMatch)->nodeType == ERRORSPECIAL)) return 1;
+  
   /* Base symbols: a match is obtained if the thing to match is equal
      to the matcher */
   if (isCorrectlyTypedBaseSymbol(possibleMatcher)) {
