@@ -1,10 +1,13 @@
 /*
 
-  Copyright 2007-2016 by
+  Copyright 2007-2018 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
+
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
   UPMC Univ Paris 06
@@ -14,14 +17,18 @@
   F-75252 Paris Cedex 05
   France
 
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
+  Sophia Antipolis, France
+
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
-  Sophia Antipolis, France.
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
 
   Contributors Ch. Lauter, S. Chevillard, M. Joldes
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
   joldes@laas.fr
 
@@ -90,8 +97,8 @@ int sollya_mpfi_set_positive_inf(sollya_mpfi_t rop);
 int sollya_mpfi_has_zero(sollya_mpfi_t op);
 int sollya_mpfi_has_zero_inside(sollya_mpfi_t op);
 int sollya_mpfi_is_zero(sollya_mpfi_t op);
-int mpfr_is_positive_infinity(mpfr_t op);
-int mpfr_is_negative_infinity(mpfr_t op);
+int sollya_mpfr_is_positive_infinity(mpfr_t op);
+int sollya_mpfr_is_negative_infinity(mpfr_t op);
 int sollya_mpfi_has_positive_numbers(sollya_mpfi_t op);
 int sollya_mpfi_has_negative_numbers(sollya_mpfi_t op);
 int sollya_mpfi_is_nonneg(sollya_mpfi_t op);
@@ -114,6 +121,7 @@ int sollya_mpfi_set_ui(sollya_mpfi_t rop, unsigned long op);
 int sollya_mpfi_set_z_2exp(sollya_mpfi_t rop, mpz_t op1, mp_exp_t op2);
 int sollya_mpfi_interv_d(sollya_mpfi_t rop, double op1, double op2);
 int sollya_mpfi_interv_fr(sollya_mpfi_t rop, mpfr_t op1, mpfr_t op2);
+int sollya_mpfi_interv_symmetrical_fr(sollya_mpfi_t rop, mpfr_t op1);
 int sollya_mpfi_interv_si(sollya_mpfi_t rop, long op1, long op2);
 int sollya_mpfi_interv_d_safe(sollya_mpfi_t rop, double op1, double op2);
 int sollya_mpfi_interv_fr_safe(sollya_mpfi_t rop, mpfr_t op1, mpfr_t op2);
@@ -179,6 +187,25 @@ int sollya_mpfi_equal_p(sollya_mpfi_t op1, sollya_mpfi_t op2);
 mp_exp_t sollya_mpfi_max_exp(sollya_mpfi_t op);
 int sollya_mpfi_fr_in_interval(mpfr_t op1, sollya_mpfi_t op2);
 int sollya_mpfi_enclosure_accurate_enough(sollya_mpfi_t op1, mp_prec_t op2);
+int sollya_mpfi_get_effective_precision(mp_prec_t *p, sollya_mpfi_t op);
+int sollya_mpfi_pow_ulong(sollya_mpfi_t z, sollya_mpfi_t x, unsigned long t);
+int sollya_mpfi_pow(sollya_mpfi_t z, sollya_mpfi_t x, sollya_mpfi_t y);
+int sollya_mpfi_round_to_double(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_round_to_single(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_round_to_quad(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_round_to_halfprecision(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_round_to_doubledouble(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_round_to_tripledouble(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_round_to_doubleextended(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_erf(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_erfc(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_ceil(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_floor(sollya_mpfi_t rop, sollya_mpfi_t op);
+int sollya_mpfi_nearestint(sollya_mpfi_t rop, sollya_mpfi_t op);
+void sollya_mpfi_swap(sollya_mpfi_t op1, sollya_mpfi_t op2);
+
+int sollya_mpfr_max(mpfr_t z, mpfr_t x, mpfr_t y, mp_rnd_t rnd);
+int sollya_mpfr_min(mpfr_t z, mpfr_t x, mpfr_t y, mp_rnd_t rnd);
 
 #endif /* ifdef MPFI_COMPAT_H */
 
