@@ -1,21 +1,24 @@
 /*
 
-  Copyright 2006-2013 by
+  Copyright 2006-2021 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
   LORIA (CNRS, INPL, INRIA, UHP, U-Nancy 2),
 
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France
+
   and by
 
-  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
-  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
-
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -774,6 +777,8 @@ int fprintInfnormTheo(FILE *fd, infnormTheo *theo, int start) {
   fprintDerivativeLemma(fd,theo->numeratorOfDerivative,theo->derivativeOfNumeratorOfDerivative,theo->number,3);
   nextNumber++;
 
+  sollyaFprintf(fd,"DEPRECATION WARNING: THE PROOF GENERATION FEATURE OF THE infnorm COMMAND IS ****DEPRECATED****!!!\n\nTHE FEATURE WILL BE REMOVED IN FUTURE VERSIONS OF THIS TOOL.\n\n");
+    
   sollyaFprintf(fd,"Theorem %d:\n",theo->number);
   sollyaFprintf(fd,"Assuming that f is C^2, the infinity norm of\nf(%s) = ",((variablename == NULL) ? "_x_" : variablename));
   fprintTree(fd,theo->function);
