@@ -1,6 +1,6 @@
 /*
 
-  Copyright 2006-2014 by
+  Copyright 2006-2021 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
@@ -10,12 +10,13 @@
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
+  Centre de recherche INRIA Sophia Antipolis Mediterranee,
+  Equipes APICS, FACTAS,
   Sophia Antipolis, France.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -72,9 +73,13 @@ chain *uncertifiedFindZeros(node *tree, mpfr_t a, mpfr_t b, unsigned long int po
 node *remez(node *func, node *weight, chain *monomials, mpfr_t a, mpfr_t b, mpfr_t quality, mpfr_t satisfying_error, mpfr_t target_error, mp_prec_t prec);
 
 rangetype guessDegree(node *func, node *weight, mpfr_t a, mpfr_t b, mpfr_t eps, int bound);
-node *constructPolynomial(mpfr_t *coeff, chain *monomials, mp_prec_t prec);
+node *constructPolyFromMpfrChain(chain *coeffs, chain *monomials);
+node *constructPolynomial(mpfr_t *coeff, chain *monomials);
 node *constructPolynomialFromArray(mpfr_t *coeff, node **monomials, int n);
 
+void perturbPoints(mpfr_t *x, int n, int k, int perturb_bounds, mp_prec_t prec);
 void findZero(mpfr_t res, node *f, node *f_diff, mpfr_t a, mpfr_t b, int sgnfa, mpfr_t *x0, int n, mp_prec_t prec);
-
+int isProvablyZero(node *tree);
+int seemsToBeZero(node *f, mpfr_t a, mpfr_t b);
+int isTrivialRemezCase(node **coeffs, node *tree, chain *monomials, mpfr_t a, mpfr_t b);
 #endif /* ifdef REMEZ_H*/
