@@ -1,6 +1,9 @@
 /*
 
-  Copyright 2013-2016 by
+  Copyright 2013-2021 by
+
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -11,9 +14,16 @@
   F-75252 Paris Cedex 05
   France
 
-  Contributors Ch. Lauter
+  and by
 
-  christoph.lauter@ens-lyon.org
+  Centre de recherche INRIA Sophia Antipolis Mediterranee,
+  Equipe FACTAS,
+  Sophia Antipolis, France.
+
+  Contributors Ch. Lauter, S. Chevillard
+
+  christoph.lauter@christoph-lauter.org
+  sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
   environment for safe floating-point code development. It is
@@ -127,15 +137,10 @@ void signalHandler(int i) {
   }
 }
 
-RETSIGTYPE signalHandlerForSignal(int i) {
+void signalHandlerForSignal(int i) {
   if (!emulatedBlockedSignalCounter) {
     signalHandler(i);
   }
-  return (RETSIGTYPE) 1; /* RETSIGTYPE is either int or void. When it
-			    is int, nothing happens. When it is void,
-			    C supports that cast and returns
-			    nothing. 
-			 */
 }
 
 static inline void deferSignalHandlingInner() {
