@@ -1,6 +1,9 @@
 /*
 
-  Copyright 2011-2016 by
+  Copyright 2011-2022 by
+
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -9,16 +12,21 @@
   Boîte Courrier 169
   4, place Jussieu
   F-75252 Paris Cedex 05
-  France
+  France,
+
+  Centre de recherche INRIA Sophia Antipolis Mediterranee,
+  Equipes APICS, FACTAS,
+  Sophia Antipolis, France
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
-  Sophia Antipolis, France.
+  Department of Computer Science & Engineering
+  UAA College of Engineering
+  University of Alaska Anchorage.
 
-  Contributor Ch. Lauter, S. Chevillard
+  Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -551,27 +559,48 @@ static const messageEntry messageTextsTable[] = {
   { SOLLYA_MSG_DEG_OF_MAX_POLY_DIV_IS_NEGATIVE                        , "An attempt was made to compute the degree of the maximal polynomial divider of a polynomial in an expression using a power operator with an exponent which is negative" },
   { SOLLYA_MSG_RATIONALAPPROX_SECOND_ARG_MUST_BE_GREATER_THAN_ONE     , "Error in rationalapprox: the second argument of rationalapprox must be greater or equal to 2" },
   { SOLLYA_MSG_ROUND_PREC_MUST_BE_AT_LEAST_TWO_BITS                   , "The precision specified when rounding to a particular format must be at least 2 bits" },
-  { SOLLYA_MSG_NAN_CONVERTED_TO_NUMBER_ON_CONSTANT_RETRIEVAL          , "A Not-A-Number value has been converted to a number upon retrieval of a constant." },
-  { SOLLYA_MSG_CHEBYSHEVFORM_DEGREE_MUST_NOT_BE_NEGATIVE              , "The degree of a Chebyshev form must be at least zero."},
+  { SOLLYA_MSG_NAN_CONVERTED_TO_NUMBER_ON_CONSTANT_RETRIEVAL          , "A Not-A-Number value has been converted to a number upon retrieval of a constant" },
+  { SOLLYA_MSG_CHEBYSHEVFORM_DEGREE_MUST_NOT_BE_NEGATIVE              , "The degree of a Chebyshev form must be at least zero"},
   { SOLLYA_MSG_CHEBYSHEVFORM_DOMAIN_MUST_NOT_BE_POINT_INTERVAL        , "The domain of a Chebyshev Model can not be a point interval"},
-  { SOLLYA_MSG_CHEBYSHEVFORM_ERROR_IN_COMPUTATION                     , "An error occurred during computation of a Chebyshev form."},
-  { SOLLYA_MSG_ERROR_IN_CHEBYSHEVFORM_COPYING_INCOMPAT_MODELS         , "Error in Chebyshev form computation: copy of incompatible models."},
-  { SOLLYA_MSG_ERROR_IN_CHEBYSHEVFORM_UNKNOWN_FUNC_FOR_ZUMKELLER      , "Error in Chebyshev form computation: unknown type of function used with Zumkeller's technique."},
-  { SOLLYA_MSG_ERROR_IN_CHEBYSHEVFORM_NOT_A_POLYNOMIAL                , "Error in Chebyshev form computation: the given function is not a polynomial, no modification is made." },
-  { SOLLYA_MSG_SPECIAL_ALGORITHM_USED_FOR_COEFF                       , "A special algorithm is used to extract the i-th coefficient of an expression that is a polynomial." },
-  { SOLLYA_MSG_NO_CORRECT_ROUNDING_FOR_ROUND_OPERATOR                 , "No correctly rounded result could be obtained upon evaluation of a rounding operator." },
-  { SOLLYA_MSG_ROUNDING_OF_BOUNDARY_INSTEAD_OF_CORRECT_ROUNDING       , "The returned rounding is the rounding of the rounding boundary because correct rounding has not been possible." },
-  { SOLLYA_MSG_NO_CORRECT_TERNARY_VALUE_FOR_ROUND_BUT_CORRECT_ROUNDING, "A correct rounding was possible but the actual rounding direction could not be determined." },
-  { SOLLYA_MSG_NO_CORRECT_TERNARY_VALUE_FOR_ROUND                     , "The actual rounding direction during a round operation could not be determined." },
-  { SOLLYA_MSG_LIBRARY_CLOSER_ERROR                                   , "While closing a library, the function \"sollya_external_lib_close\" was found and called but it signaled an error." },
-  { SOLLYA_MSG_INF_CONVERTED_TO_NUMBER_ON_CONSTANT_RETRIEVAL          , "An infinity has been converted to a number upon retrieval of a constant." },
-  { SOLLYA_MSG_ANNOTATION_COULD_NOT_BE_SET_UP                         , "The annotation could not be set up. The function is returned as-is." },
-  { SOLLYA_MSG_GENERIC_SOLLYA_LIBRARY_MSG                             , "An external program using the Sollya library has displayed a message." },
-  { SOLLYA_MSG_GUESSDEGREE_POSSIBLE_SINGULAR_WEIGHT                   , "The weight function in guessdegree might not be continuous over the given interval." },
-  { SOLLYA_MSG_SAFE_ROUNDING_FOR_EXPR_THAT_SHOULD_BE_CONST            , "At least one of the given expressions is not a constant but requires evaluation. Evaluation is guaranteed to ensure the inclusion property." },
-  { SOLLYA_MSG_DEGREE_OF_POLYNOMIAL_LARGER_THAN_MULTIPRECISION_INT    , "The degree of the given polynomial is larger than the largest multiprecision integer that can be held in memory. The polynomial's degree will be returned as -1." },
-  { SOLLYA_MSG_ANNOTATION_INCOHERENT                                  , "An inconsistency has been detected between the results obtained with the evaluation of two distinct annotations of the same function." },
-  { SOLLYA_MSG_REMEZ_SWITCHING_DEFINITIVELY_TO_SLOW_ALGORITHM         , "Remez: multipoint algorithm failed due to lack of Haar condition. Switching back to single point algorithm." },
+  { SOLLYA_MSG_CHEBYSHEVFORM_ERROR_IN_COMPUTATION                     , "An error occurred during computation of a Chebyshev form"},
+  { SOLLYA_MSG_ERROR_IN_CHEBYSHEVFORM_COPYING_INCOMPAT_MODELS         , "Error in Chebyshev form computation: copy of incompatible models"},
+  { SOLLYA_MSG_ERROR_IN_CHEBYSHEVFORM_UNKNOWN_FUNC_FOR_ZUMKELLER      , "Error in Chebyshev form computation: unknown type of function used with Zumkeller's technique"},
+  { SOLLYA_MSG_ERROR_IN_CHEBYSHEVFORM_NOT_A_POLYNOMIAL                , "Error in Chebyshev form computation: the given function is not a polynomial, no modification is made" },
+  { SOLLYA_MSG_SPECIAL_ALGORITHM_USED_FOR_COEFF                       , "A special algorithm is used to extract the i-th coefficient of an expression that is a polynomial" },
+  { SOLLYA_MSG_NO_CORRECT_ROUNDING_FOR_ROUND_OPERATOR                 , "No correctly rounded result could be obtained upon evaluation of a rounding operator" },
+  { SOLLYA_MSG_ROUNDING_OF_BOUNDARY_INSTEAD_OF_CORRECT_ROUNDING       , "The returned rounding is the rounding of the rounding boundary because correct rounding has not been possible" },
+  { SOLLYA_MSG_NO_CORRECT_TERNARY_VALUE_FOR_ROUND_BUT_CORRECT_ROUNDING, "A correct rounding was possible but the actual rounding direction could not be determined" },
+  { SOLLYA_MSG_NO_CORRECT_TERNARY_VALUE_FOR_ROUND                     , "The actual rounding direction during a round operation could not be determined" },
+  { SOLLYA_MSG_LIBRARY_CLOSER_ERROR                                   , "While closing a library, the function \"sollya_external_lib_close\" was found and called but it signaled an error" },
+  { SOLLYA_MSG_INF_CONVERTED_TO_NUMBER_ON_CONSTANT_RETRIEVAL          , "An infinity has been converted to a number upon retrieval of a constant" },
+  { SOLLYA_MSG_ANNOTATION_COULD_NOT_BE_SET_UP                         , "The annotation could not be set up. The function is returned as-is" },
+  { SOLLYA_MSG_GENERIC_SOLLYA_LIBRARY_MSG                             , "An external program using the Sollya library has displayed a message" },
+  { SOLLYA_MSG_GUESSDEGREE_POSSIBLE_SINGULAR_WEIGHT                   , "The weight function in guessdegree might not be continuous over the given interval" },
+  { SOLLYA_MSG_SAFE_ROUNDING_FOR_EXPR_THAT_SHOULD_BE_CONST            , "At least one of the given expressions is not a constant but requires evaluation. Evaluation is guaranteed to ensure the inclusion property" },
+  { SOLLYA_MSG_DEGREE_OF_POLYNOMIAL_LARGER_THAN_MULTIPRECISION_INT    , "The degree of the given polynomial is larger than the largest multiprecision integer that can be held in memory. The polynomial's degree will be returned as -1" },
+  { SOLLYA_MSG_ANNOTATION_INCOHERENT                                  , "An inconsistency has been detected between the results obtained with the evaluation of two distinct annotations of the same function" },
+  { SOLLYA_MSG_REMEZ_SWITCHING_DEFINITIVELY_TO_SLOW_ALGORITHM         , "Remez: multipoint algorithm failed due to lack of Haar condition. Switching back to single point algorithm" },
+  { SOLLYA_MSG_IDENTIFIER_IS_EXTERNAL_DATA_CANNOT_BE_LOCAL_VAR        , "An identifier is already bound to an external data symbol. It cannot be declared as a local variable" },
+  { SOLLYA_MSG_IDENTIFIER_IS_EXTERNAL_DATA_CANNOT_BE_EXTERNAL         , "An identifier is already bound to an external data symbol. It cannot be bound to an external procedure" },
+  { SOLLYA_MSG_IDENTIFIER_IS_EXTERNAL_DATA_CANNOT_BE_LIBRARY          , "An identifier is already bound to an external data symbol. It cannot be bound to a library function or constant" },
+  { SOLLYA_MSG_IDENTIFIER_IS_EXTERNAL_DATA_CANNOT_BE_MATCHED          , "An identifier is already bound to an external data symbol. It cannot be used as a match variable" },
+  { SOLLYA_MSG_IDENTIFIER_IS_EXTERNAL_DATA_CANNOT_BE_PARAMETER        , "An identifier is already bound to an external data symbol. It cannot be used as a formal parameter of a procedure" },
+  { SOLLYA_MSG_IDENTIFIER_IS_EXTERNAL_DATA_CANNOT_BE_EXT_DATA         , "An identifier is already bound to an external data symbol. It cannot be bound to an external data symbol" },
+
+  { SOLLYA_MSG_IDENTIFIER_IS_FREE_VAR_CANNOT_BE_EXT_DATA              , "An identifier is already bound to the current free variable. It cannot be bound to an external data symbol" },
+  { SOLLYA_MSG_IDENTIFIER_IS_BOUND_TO_VAR_CANNOT_BE_EXT_DATA          , "An identifier is already bound to some identifier. It cannot be bound to an external data symbol" },
+  { SOLLYA_MSG_IDENTIFIER_IS_LIBRARY_CONST_CANNOT_BE_EXT_DATA         , "An identifier is already bound to a library constant. It cannot be bound to an external data symbol" },
+  { SOLLYA_MSG_IDENTIFIER_IS_LIBRARY_FUNC_CANNOT_BE_EXT_DATA          , "An identifier is already bound to a library function. It cannot be bound to an external data symbol" },
+  { SOLLYA_MSG_IDENTIFIER_IS_EXTERNAL_PROC_CANNOT_BE_EXT_DATA         , "An identifier is already bound to an external procedure. It cannot be bound to an external data symbol" },
+  { SOLLYA_MSG_CANNOT_PERFORM_BIND                                    , "An requested binding cannot be performed. The procedure is returned unchanged." },
+  { SOLLYA_MSG_FPMINIMAX_FORMAT_IS_ELLIPTIC_WITH_INT                  , "fpminimax: the list of formats is an numerical end-elliptic list. This is deprecated" },
+  { SOLLYA_MSG_REMEZ_IS_TRIVIAL                                       , "remez: the function is its own best approximation" },
+  { SOLLYA_MSG_FPMINIMAX_CHEBPOINTS_BECAUSE_EXACT                     , "fpminimax: the minimax is the function itself, so we choose Chebyshev points" },
+  { SOLLYA_MSG_FPMINIMAX_NOT_ENOUGH_COEFFICIENTS                      , "fpminimax: not enough coefficients provided for the real minimax"},
+  { SOLLYA_MSG_POLYNOMIAL_GCD_CANNOT_BE_DECIDED                       , "In a gcd computation for polynomials, the tool could not decide if the gcd is 1 or a polynomial of a higher degree" },
+  { SOLLYA_MSG_ROUNDING_AT_DECIMAL_DISPLAY                            , "For a certain number of the constants displayed in decimal, rounding has happened" },
+  { SOLLYA_MSG_CONSTANT_PRECISION_TOO_HIGH                            , "A constant to be displayed has a precision that is too high for the current tool's precision setting when automatically printing the constant at the prompt." },
+  { SOLLYA_MSG_MATCHING_OF_ERROR                                      , "The evaluation of the expression to be matched in a match... with construct yields error due to a syntax error or an error on a side-effect. The effect or return value of the match... with construct may be meaningless" },
   /* Add your new warning message texts above here. */
 
   { -1, "Unknown message number -1" }   /* Do not ever change the text in this default entry, which must be the last entry in the list */
