@@ -1,10 +1,13 @@
 /*
 
-  Copyright 2008-2016 by
+  Copyright 2008-2017 by
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
 
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
+
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
   UPMC Univ Paris 06
@@ -16,13 +19,13 @@
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
+  Centre de recherche INRIA Sophia Antipolis Mediterranee, equipe APICS,
   Sophia Antipolis, France.
 
   Contributors M. Joldes, Ch. Lauter, S. Chevillard
 
   joldes@laas.fr
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -341,8 +344,8 @@ int sturm_mpq(int *n, mpq_t *p, int p_degree, sollya_mpfi_t x, mp_prec_t precisi
   mpq_init(aq);
   mpq_init(bq);
 
-  mpfr_to_mpq(aq,a);
-  mpfr_to_mpq(bq,b);
+  sollya_mpfr_to_mpq(aq,a);
+  sollya_mpfr_to_mpq(bq,b);
 
   evalResA = (mpq_t *)safeMalloc((p_degree+1)*sizeof(mpq_t));
   evalResB = (mpq_t *)safeMalloc((p_degree+1)*sizeof(mpq_t));
@@ -586,6 +589,7 @@ int sturm_mpfi(int *n, mpq_t *pMpq, int p_degree, sollya_mpfi_t x, mp_prec_t pre
     if (tempprec > prec) prec = tempprec;
   }
   prec = 2 * prec;
+  if (prec > 32 * precision) prec = 32 * precision;
 
   printMessage(2,SOLLYA_MSG_STURM_INTERVAL_A_CERTAIN_PREC_HAS_BEEN_CHOSEN,"Information: in sturm_mpfi: chosen working precision is %d\n",(int) prec);
 
@@ -825,12 +829,12 @@ int getNrRoots(mpfr_t res, node *f, sollya_mpfi_t range, mp_prec_t precision, in
 	      printMessage(1,SOLLYA_MSG_STURM_COEFF_ROUNDED_TO_ZERO,"Warning: Rounded the coefficient %d to 0.\n",i);
 	    }
 	  }
-	  mpfr_to_mpq(qCoefficients[i], tempValue2);
+	  sollya_mpfr_to_mpq(qCoefficients[i], tempValue2);
 	  printMessage(3,SOLLYA_MSG_STURM_COEFF_EVALUATED_TO_RATIONAL_NUMBER,"Information: evaluated the %dth coefficient to %r\n",i,qCoefficients[i]);
 	}
       }
       else {
-	mpfr_to_mpq(qCoefficients[i], *(accessThruMemRef(tempTree)->value));
+	sollya_mpfr_to_mpq(qCoefficients[i], *(accessThruMemRef(tempTree)->value));
       }
       free_memory(tempTree);
     } else {
@@ -845,6 +849,9 @@ int getNrRoots(mpfr_t res, node *f, sollya_mpfi_t range, mp_prec_t precision, in
 
   if (deg >= 0) {
     resMpfi = sturm_mpfi(&nr, qCoefficients, deg,x,precision);
+    if (!resMpfi) {
+      resMpfi = sturm_mpfi(&nr, qCoefficients, deg,x,8 * precision);
+    }
     if (!resMpfi) {
       if (!silent) {
 	printMessage(2,SOLLYA_MSG_STURM_USING_SLOWER_ALGORITHM_ON_RATIONALS,"Information: using slower GMP MPQ version\n");
