@@ -10,11 +10,10 @@
   Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
   UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France.
 
-  Contributors Ch. Lauter, S. Chevillard, N. Jourdan
+  Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
-  nicolas.jourdan@ens-lyon.fr
 
   This software is a computer program whose purpose is to provide an
   environment for safe floating-point code development. It is
