@@ -1,6 +1,9 @@
 /*
 
-  Copyright 2010-2016 by
+  Copyright 2010-2021 by
+
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
 
   Laboratoire d'Informatique de Paris 6 - Équipe PEQUAN
   Sorbonne Universités
@@ -9,19 +12,20 @@
   Boîte Courrier 169
   4, place Jussieu
   F-75252 Paris Cedex 05
-  France
+  France,
 
   Laboratoire de l'Informatique du Parallelisme,
   UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668
 
   and by
 
-  Centre de recherche INRIA Sophia-Antipolis Mediterranee, equipe APICS,
+  Centre de recherche INRIA Sophia Antipolis Mediterranee,
+  Equipes APICS, FACTAS,
   Sophia Antipolis, France.
 
   Contributors Ch. Lauter, M. Joldes, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   joldes@laas.fr
   sylvain.chevillard@ens-lyon.org
 
@@ -946,7 +950,18 @@ int computeTaylorModelOfLeastDegree(node **poly, node *func, sollya_mpfi_t dom,
     else nMin = n; /* Not OK with n, so the new interval is [n,nMax] */
   }
 
-  /* Now we know that bestPoly is the polynomial that we want, but we need to translate it */
+  /* Though unlikely, it could be possible that some coefficient of the polynomial be a NaN or Inf */
+  /*   (despite a successful return of checkDegreeTaylorModel...). So we check it. */
+  if (!isPolynomialWithConstantDyadicFiniteRealCoefficients(bestPoly)) {
+    /* Here, we are in the strange case that should almost never
+       happen.  We indicate failure and hope the problem will go
+       away by bisecting dom.
+    */
+    if (bestPoly != NULL) free_memory(bestPoly);
+    return 0;
+  }
+
+  /* Now we know that bestPoly is the polynomial that we want, but we might need to translate it */
   if (singu == NULL) {
     mpfr_init2(x0,sollya_mpfi_get_prec(dom) + 1);
     sollya_mpfi_mid(x0,dom);
@@ -955,7 +970,10 @@ int computeTaylorModelOfLeastDegree(node **poly, node *func, sollya_mpfi_t dom,
     mpfr_init2(x0,mpfr_get_prec(*singu));
     mpfr_set(x0,*singu,GMP_RNDN); /* exact */
   }
-
+  if (!mpfr_number_p(x0)) {
+    mpfr_clear(x0);
+    return 0;
+  }
   shifterBack = makeSub(makeVariable(),makeConstant(x0));
   pShifted = substitute(bestPoly,shifterBack);
   free_memory(shifterBack);
@@ -1359,25 +1377,6 @@ int supnormAbsolute(sollya_mpfi_t result, node *poly, node *func, sollya_mpfi_t
      safety of the algorithm, i.e. validates it.
 
   */
-  if ( (!isPolynomialWithConstantDyadicFiniteRealCoefficients(s1)) ||
-       (!isPolynomialWithConstantDyadicFiniteRealCoefficients(s2))) {
-    /* I believe that this is dead code. It is here to cover the case where
-       checkDegreeTaylorModel would successfully return, but the returned
-       polynomial would contain non-dyadic coefficients. I think that this
-       cannot happen.
-    */
-    mpfr_clear(ell);
-    mpfr_clear(gamma);
-    mpfr_clear(errMax);
-    mpfr_clear(bound);
-    free_memory(T);
-    free_memory(boundAsNode);
-    free_memory(s1);
-    free_memory(s2);
-    free_memory(pMinusT);
-    free_memory(TMinusp);
-    return SUPNORM_NO_TAYLOR;
-  }
   if ((!showPositivity(s1, dom, prec)) ||
       (!showPositivity(s2, dom, prec))) {
     /* At least one of s1 or s2 has a point in dom where it is non-positive
@@ -1661,30 +1660,6 @@ int supnormRelativeNoSingularity(sollya_mpfi_t result, node *poly, node *func, s
      safety of the algorithm, i.e. validates it.
 
   */
-  if ( (!isPolynomialWithConstantDyadicFiniteRealCoefficients(s1)) ||
-       (!isPolynomialWithConstantDyadicFiniteRealCoefficients(s2))) {
-    /* I believe that this is dead code. It is here to cover the case where
-       checkDegreeTaylorModel would successfully return, but the returned
-       polynomial would contain non-dyadic coefficients. I think that this
-       cannot happen.
-    */
-    mpfr_clear(F);
-    mpfr_clear(ell);
-    mpfr_clear(gamma);
-    mpfr_clear(u);
-    mpfr_clear(thirtyoneThirtySecond);
-    mpfr_clear(errMax);
-    mpfr_clear(midDom);
-    mpfr_clear(bound);
-    mpfr_clear(signT);
-    free_memory(T);
-    free_memory(boundTimesT);
-    free_memory(s1);
-    free_memory(s2);
-    free_memory(pMinusT);
-    free_memory(TMinusp);
-    return SUPNORM_NO_TAYLOR;
-  }
   if ((!showPositivity(s1, dom, prec)) ||
       (!showPositivity(s2, dom, prec))) {
     /* At least one of s1 or s2 has a point in dom where it is non-positive
