@@ -12,7 +12,7 @@
 
   Contributors Ch. Lauter, M. Joldes
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   joldes@laas.fr
 
   This software is a computer program whose purpose is to provide an
