@@ -1,18 +1,23 @@
 /*
 
-  Copyright 2006-2011 by
+  Copyright 2006-2018 by
 
   Laboratoire de l'Informatique du Parallelisme,
-  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668
+  UMR CNRS - ENS Lyon - UCB Lyon 1 - INRIA 5668,
+
+  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
+  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France
 
   and by
 
-  Laboratoire d'Informatique de Paris 6, equipe PEQUAN,
-  UPMC Universite Paris 06 - CNRS - UMR 7606 - LIP6, Paris, France,
+  Sorbonne Université
+  CNRS, Laboratoire d'Informatique de Paris 6, LIP6
+  F - 75005 Paris
+  France.
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -59,7 +64,7 @@
 #include "taylor.h"
 #include "expression.h"
 #include "general.h"
-
+#include "execute.h"
 
 
 
@@ -72,7 +77,7 @@ node *taylor(node* tree, int degree, node* point, mp_prec_t prec) {
   if (!isConstant(point)) {
     printMessage(1,SOLLYA_MSG_DEVELOPMENT_POINT_NOT_CONSTANT,"Warning: the expression given for the development point is not constant.\n");
     printMessage(1,SOLLYA_MSG_CONTINUATION,"Will evaluate the expression in %s = 0 before using it as development point.\n",((variablename == NULL) ? "_x_" : variablename));
-    temp = (node *) safeMalloc(sizeof(node));
+    temp = allocateNode();
     temp->nodeType = CONSTANT;
     value = (mpfr_t *) safeMalloc(sizeof(mpfr_t));
     mpfr_init2(*value,prec);
@@ -89,77 +94,81 @@ node *taylor(node* tree, int degree, node* point, mp_prec_t prec) {
   value = (mpfr_t *) safeMalloc(sizeof(mpfr_t));
   mpfr_init2(*value,prec);
   mpfr_set_d(*value,0.0,GMP_RNDN);
-  copy = (node *) safeMalloc(sizeof(node));
+  copy = allocateNode();
   copy->nodeType = CONSTANT;
   copy->value = value;
 
   mpz_init(denominatorGMP);
   fderiv = copyTree(tree);
   for (i=0;i<=degree;i++) {
-    temp = substitute(fderiv,pointTemp);
-    fderivsubst = simplifyTreeErrorfree(temp);
+    temp = addMemRef(substitute(fderiv,pointTemp));
+    fderivsubst = addMemRef(simplifyTreeErrorfree(temp));
+    tryRepresentAsPolynomial(fderivsubst);
     free_memory(temp);
     mpz_fac_ui(denominatorGMP,(unsigned int) i);
     value = (mpfr_t *) safeMalloc(sizeof(mpfr_t));
-    mpfr_init2(*value,prec);
+    mpfr_init2(*value,((prec > 74) ? (prec) : (74)));
     if(mpfr_set_z(*value,denominatorGMP,GMP_RNDN) != 0) {
       if (!noRoundingWarnings) {
 	printMessage(1,SOLLYA_MSG_ROUNDING_ON_COMPUTATION_OF_TAYLOR_COEFFICIENT,"Warning: rounding occurred on computing a taylor constant factor.\n");
 	printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
       }
     }
-    denominator = (node *) safeMalloc(sizeof(node));
+    denominator = allocateNode();
     denominator->nodeType = CONSTANT;
     denominator->value = value;
     value = (mpfr_t *) safeMalloc(sizeof(mpfr_t));
     mpfr_init2(*value,prec);
     mpfr_set_d(*value,1.0,GMP_RNDN);
-    numerator = (node *) safeMalloc(sizeof(node));
+    numerator = allocateNode();
     numerator->nodeType = CONSTANT;
     numerator->value = value;
-    temp = (node *) safeMalloc(sizeof(node));
+    temp = allocateNode();
     temp->nodeType = DIV;
-    temp->child1 = numerator;
-    temp->child2 = denominator;
-    temp2 = (node *) safeMalloc(sizeof(node));
+    temp->child1 = addMemRef(numerator);
+    temp->child2 = addMemRef(denominator);
+    temp2 = allocateNode();
     temp2->nodeType = MUL;
-    temp2->child1 = temp;
-    temp2->child2 = fderivsubst;
-    variable = (node *) safeMalloc(sizeof(node));
-    variable->nodeType = VARIABLE;
+    temp2->child1 = addMemRef(temp);
+    temp2->child2 = addMemRef(fderivsubst);
+    variable = addMemRef(makeVariable());
     value = (mpfr_t *) safeMalloc(sizeof(mpfr_t));
-    mpfr_init2(*value,prec);
+    mpfr_init2(*value,((prec > 74) ? (prec) : (74)));
     if(mpfr_set_si(*value,i,GMP_RNDN) != 0) {
       if (!noRoundingWarnings) {
 	printMessage(1,SOLLYA_MSG_ROUNDING_ON_COMPUTATION_OF_TAYLOR_POWER,"Warning: rounding occurred on computing a taylor exponent.\n");
 	printMessage(1,SOLLYA_MSG_CONTINUATION,"Try to increase the working precision.\n");
       }
     }
-    expon = (node *) safeMalloc(sizeof(node));
+    expon = allocateNode();
     expon->nodeType = CONSTANT;
     expon->value = value;
-    temp = (node *) safeMalloc(sizeof(node));
+    temp = allocateNode();
     temp->nodeType = POW;
     temp->child1 = variable;
-    temp->child2 = expon;
-    term = (node *) safeMalloc(sizeof(node));
+    temp->child2 = addMemRef(expon);
+    term = allocateNode();
     term->nodeType = MUL;
-    term->child1 = temp2;
-    term->child2 = temp;
-    temp = (node *) safeMalloc(sizeof(node));
+    term->child1 = addMemRef(temp2);
+    term->child2 = addMemRef(temp);
+    temp = allocateNode();
     temp->nodeType = ADD;
-    temp->child1 = copy;
-    temp->child2 = term;
-    copy = temp;
-    temp = differentiate(fderiv);
-    free_memory(fderiv);
-    fderiv = temp;
+    temp->child1 = addMemRef(copy);
+    temp->child2 = addMemRef(term);
+    copy = addMemRef(temp);
+    if (i < degree) {
+      temp = differentiate(fderiv);
+      free_memory(fderiv);
+      fderiv = temp;
+    }
   }
   mpz_clear(denominatorGMP);
 
   free_memory(fderiv);
 
-  temp = horner(copy);
+  copy = addMemRef(copy);
+  tryRepresentAsPolynomial(copy);
+  temp = addMemRef(horner(copy));
   free_memory(copy);
   free_memory(pointTemp);
   return temp;
