@@ -7,8 +7,8 @@
 
 
 #define A_DIM 15
-#define TEMP_IMPLEMENTATION_FILE "timplementpoly.implementation.c"
-#define TEMP_PROOF_FILE "timplementpoly.implementation.gappa"
+#define TEMP_IMPLEMENTATION_FILE "tv_implementpoly.implementation.c"
+#define TEMP_PROOF_FILE "tv_implementpoly.implementation.gappa"
 #define READBUFFERSIZE 1024
 
 int callback(sollya_msg_t msg, void *data) {
