@@ -12,7 +12,7 @@
 
   Contributors Ch. Lauter, S. Chevillard
 
-  christoph.lauter@ens-lyon.org
+  christoph.lauter@christoph-lauter.org
   sylvain.chevillard@ens-lyon.org
 
   This software is a computer program whose purpose is to provide an
@@ -289,7 +289,7 @@ int searchGalValue(chain *funcs, mpfr_t foundValue, mpfr_t startValue, mp_prec_t
     currEpsilon = epsilons;
     while ((currFunc != NULL) && (currFormat != NULL) && (currEpsilon != NULL)) {
       evaluateFaithful(yCurrLeft, (node *) (currFunc->value), currLeft, p);
-      mpfr_round_to_format(yCurrLeftRound, yCurrLeft, *((int *) (currFormat->value)));
+      sollya_mpfr_round_to_format(yCurrLeftRound, yCurrLeft, *((int *) (currFormat->value)));
       mpfr_sub(errorLeft,yCurrLeftRound,yCurrLeft,GMP_RNDN);
       mpfr_div(errorLeft,errorLeft,yCurrLeft,GMP_RNDN);
       if (!(mpfr_number_p(errorLeft) && (mpfr_cmpabs(errorLeft,*((mpfr_t *) (currEpsilon->value))) <= 0))) {
@@ -310,7 +310,7 @@ int searchGalValue(chain *funcs, mpfr_t foundValue, mpfr_t startValue, mp_prec_t
     currEpsilon = epsilons;
     while ((currFunc != NULL) && (currFormat != NULL) && (currEpsilon != NULL)) {
       evaluateFaithful(yCurrRight, (node *) (currFunc->value), currRight, p);
-      mpfr_round_to_format(yCurrRightRound, yCurrRight, *((int *) (currFormat->value)));
+      sollya_mpfr_round_to_format(yCurrRightRound, yCurrRight, *((int *) (currFormat->value)));
       mpfr_sub(errorRight,yCurrRightRound,yCurrRight,GMP_RNDN);
       mpfr_div(errorRight,errorRight,yCurrRight,GMP_RNDN);
       if (!(mpfr_number_p(errorRight) && (mpfr_cmpabs(errorRight,*((mpfr_t *) (currEpsilon->value))) <= 0))) {
